#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 04.06.2009

import sys
#Zufallsgenerator zur Farbgenerierung
import random
#Importpfad für übergeordnete Verzeichnisse setzen
sys.path.append('..')
#System-Werte zum Laden entfernen
#TODO: Entfernen, wenn alle Tests abgeschlossen
strTemp = sys.argv
sys.argv = []

from QTurboMachineWidget import QTurboMachineWidget
from QBladeRowWidget import QBladeRowWidget
from QProfile2DWidget import QProfile2DWidget

from PyQt4.QtGui import QMainWindow, QImage, QTreeWidgetItem, QColor, QIcon, QPixmap, QPainter, QBrush, QMessageBox, QFileDialog
from PyQt4.QtCore import QDir, QTranslator, SIGNAL, QFile
from Ui_BladeDesigner import Ui_BladeDesigner as BladeDesigner
from QAboutDialog import QAboutDialog
import gc

#Garbage Collection zu Debuggingzwecken ausschalten
gc.disable()

from os.path import basename, dirname
import pdb

#System-Werte wiederherstellen
#TODO: Entfernen, wenn alle Tests abgeschlossen
sys.argv = strTemp

## @brief Mainclass of BladeDesigner<br><em>Hauptklasse des BladeDesigners</em>
#
# In this class all functionality of the GUI comes together. Therefore QBladeDesigner manages the full turboMachine using QTurboMachineWidget. Furthermore
# it is in charge of data managment (save, load, new) and visualization of the hierarchy. It contains a clipboard and can do fundamental working
# functions(cut, copy, paste).
# The layout of this widget in QtDesignerform is contained in ./UI/BladeDesigner.ui, which can be converted to UI_BladeDesigner by ./CreateGUI.sh script using pyuic4
# and loaded afterwards.
# <br><em>In dieser Klasse läuft die Funktionalität der GUI zusammen. QBladeDesigner verwaltet dafür die gesamte TurboMaschine mithilfe des
# QTurboMachineWidget. Des Weiteren kümmert sie sich um die Verwaltung der Daten (speichern, laden, neu) und die Darstellung der Hierarchie.
# Sie beinhaltet eine Zwischenablage und beherrscht grundlegende Arbeitsfunktionen(Ausschneiden, Kopieren, Einfügen).
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/BladeDesigner.ui, welche von ./CreateGUI.sh script mittels pyuic4 in UI_BladeDesigner konvertiert
# und dann geladen werden kann.
# </em>
#
# @sa QProfile2DWidget::QProfile2DWidget, QBladeRowWidget::QBladeRowWidget, QTurboMachineWidget::QTurboMachineWidget
class QBladeDesigner(QMainWindow, BladeDesigner):
	#Konstanten

	## @brief Constant, which tells the type of the current selection<br><em>Konstante die den Typ der aktuellen Auswahl angibt</em>
	#
	# A turboMachine is selected.
	# <br><em>Eine TurboMachine ist ausgewählt.
	# </em>
	#
	# @sa itemClicked(), __saveChanges()
	__TurboMachine = 0


	## @brief Constant, which tells the type of the current selection<br><em>Konstante die den Typ der aktuellen Auswahl angibt</em>
	#
	# A bladeRow is selected.
	# <br><em>Eine BladeRow ist ausgewählt.
	# </em>
	#
	# @sa itemClicked(), __saveChanges()
	__BladeRow = 1


	## @brief Constant, which tells the type of the current selection<br><em>Konstante die den Typ der aktuellen Auswahl angibt</em>
	#
	# A profile is selected.
	# <br><em>Eine Profile ist ausgewählt.
	# </em>
	#
	# @sa itemClicked(), __saveChanges()
	__Profile = 2


	## @brief Constant, which tells the type of the current selection<br><em>Konstante die den Typ der aktuellen Auswahl angibt</em>
	#
	# Nothing is selected. This may be an error state.
	# <br><em>Eine TurboMachine ist ausgewählt. Das kann ein Fehlerzustand sein.
	# </em>
	#
	# @sa itemClicked(), __saveChanges()
	__None = -1





	## @brief Constructor of QBladeDesigner<br><em>Konstruktor des QBladeDesigner s</em>
	#
	# Loads language files, initializes the mainwindow, creates the context menu and icons. Creates the clipboard and a new turboMachine containing a bladeRow
	# and a profile.
	# <br><em>Läd die Sprachdateien, initialisiert das Hauptfenster, erstellt das Contextmenü und die Icons. Legt die Zwischenablage an und erstellt
	# eine neue TurboMaschine mit einer BladeRow und einem Profil.
	# </em>
	#
	# @sa new(), __Clipboard, __pTurboMachineWidget, __lTranslations
	def __init__(self, pApp):
		#Applikation zwischenspeichern

		## @brief Pointer to the calling application<br><em>Zeiger auf die aufrufende Andwendung</em>
		#
		# Needed for translation purpose.
		# <br><em>Wird zu Übersetzungszwecken benötigt.
		# </em>
		#
		# @sa __lTranslations, languageChanged()
		self.__pApp = pApp

		#Verzeichnis öffnen
		pDir = QDir(":/Sprache/res")
		#Sprachenliste

		## @brief Contains all languages supported<br><em>Beinhaltet alle unterstützen Sprachen</em>
		#
		# This list can be extended with languages by integrating additional language files to the :/Sprache/res branche of ressource file.
		# The following steps are neccessary to do so:
		# - input an other language file template into GUI.pro Section TRANSLATIONS
		# - extract language files from GUI.pro using "pylupdate4 GUI.pro"
		# - translate generated language files with QtLinguist
		# - create an entry in ./UI/BladeDesignerGui.qrc for the language file
		# - create an entry in ./CreateGUI.sh to copy the converted language file
		# - run ./CreateGUI.sh to integrate the new language file into ressource file
		# - Add a new menu entry to language menu in QtDesigner
		# - Add a new function to set the new language and attache it to the new menu entry
		# <br><em>Die Liste kann um Sprachen erweitert werden, indem man weitere Sprachdateien in den :/Sprachen/res Zweig der Ressourcedatei integriert.
		# Dazu sind folgende Schritte nötig:
		# - Weitere Sprachdateivorlage in die GUI.pro in der Section TRANSLATIONS eintragen
		# - Sprachdatei aus GUI.pro mittels "pylupdate4 GUI.pro" extrahieren
		# - Entstandene Sprachdatei in QtLinguist übersetzen
		# - Eintrag in ./UI/BladeDesignerGui.qrc für Sprachdatei erstellen
		# - Eintrag in ./CreateGUI.sh zum kopieren der Konvertierten Sprachdatei erstellen
		# - ./CreateGUI.sh ausführen um die neue Sprachdatei in die Resourcedatei zu integrieren
		# - Im QtDesigner im Menü Sprache einen neuen Eintrag für die Sprache erstellen
		# - Eine neue Funktion zum setzen der Sprache erstellen und an den neuen Menüeintrag hängen
		# </em>
		#
		# @sa languageChanged, changeToGerman(), changeToEnglish()
		self.__lTranslations = []

		#Einträge öffnen
		for pEntry in pDir.entryInfoList():
			#Translation erstellen
			pTranslation = QTranslator()
			#Namen holen
			strFilename = pEntry.absoluteFilePath()
			#Laden
			pTranslation.load(strFilename)
			#Anhängen
			self.__lTranslations.append(pTranslation)

		#Als erster Übersetzer ist der 0. gewählt (=Deutsch)

		## @brief Stores the current Translator<br><em>Speichert den aktuellen Übersetzer</em>
		#
		# The default ist german.
		# <br><em>Standardmäßig auf Deutsch eingestellt.
		# </em>
		#
		# @todo Store the choosen language after closing<br><em>Gewählte Sprache nach dem Beenden beibehalten</em>
		#
		# @sa languageChanged()
		self.__iCurTranslator = 0

		#Erste Übersetzung einstellen
		self.__pApp.installTranslator(self.__lTranslations[self.__iCurTranslator])
		
		#Hauptfenster initialisieren
		QMainWindow.__init__(self)
		#UI laden
		self.setupUi(self)
		
		#Kontextmenü erstellen
		self.createContextMenu()
		#Icons erstellen
		self.__createIcons()
		#Widgetlayout erstellen
		self.WidgetLayout = self.centralWidget().layout()

		#TurboMachineWidget dummy

		## @brief Stores all relevant data of the turboMachine <br><em>Speichert die TurboMachine und damit alle relevanten Daten</em>
		self.__pTurboMachineWidget = None

		#Neues Projekt anlegen
		self.new(True)

		#Zwischenablage einrichten

		## @brief Clipboard for working functions<br><em>Zwischenablage für Arbeitsfunktionen</em>
		#
		# Thanks to pythons typless structure everything can be put into the clipboard. Therefore the clipboard is extendable in any order.
		# Responds to all working functions (cut, copy, paste).
		# <br><em>Dank Pythons typloser Struktur kann man alles mögliche in die Zwischenablage stecken. Von daher ist diese
		# Zwischenablage beliebig erweiterbar. Spricht auf alle Arbeitsfunktionen an(Ausschneiden, Kopieren, Einfügen).
		# </em>
		#
		# @todo Extend the amount of clipboard(10 entrys or something)<br><em>Größe der Zwischenablage erweitern(10 Einträge oÄ)</em>
		#
		# @sa copy(), paste(), cut(), __ClipboardContent
		self.__Clipboard = None
		
		#Was ist ausgewählt?

		## @brief Stores what in the clipboard is<br><em>Speichert was in der Zwischenablage liegt</em>
		#
		# The constants defined in QBladeDesigner are used. It is recommended to insert additional constants on expansion.
		# <br><em>Dafür werden die in QBladeDesigner definierten Konstanten benutzt. Bei Erweiterung der Zwischenablage empfiehlt es sich
		# zusätzliche Konstanten einzufügen.
		# </em>
		#
		# @sa copy(), paste(), cut(), __Clipboard
		self.__ClipboardContent = QBladeDesigner.__None
		
		#Im Falle der BladeRow, sind die Profile ausgeklappt?
		
		## @brief Stores, whether the copied entrys are expanded or are not<br><em>Speichert ob der kopierte Eintrag ausgeklappt oder geschloßen war</em>
		#
		# There is only a cosmetic effect. Is used to determine, whether a bladeRow/turboMachine is expanded on copy, so that the
		# profiles can be seen or not.
		# <br><em>Hat im Grunde nur kosmetischen Effekt. Wird dazu benutzt um zu ermitteln ob eine BladeRow/TurboMaschine beim
		# kopieren geöffnet ist, so dass man ihre Profile sieht oder nicht.
		# </em>
		#
		# @sa __Clipboard, __ClipboardContent, copy(), cut(), paste()
		self.__ClipboardContentExpanded = False
		
		#Name der zuspeichernden Datei
		
		## @brief Holds the working directory where the project is stored.<br><em>Gibt an in welches Arbeitsverzeichnis das Projekt gespeichert wird</em>
		#
		# Is used for simple storage, so the filedialog has not to show up evereytime when the user clicks save.
		# <br><em>Wird für einfaches Speicher benutzt, damit nicht immer der Dateidialog angezeigt werden muß, falls der Benutzer auf speichern klickt.
		# </em>
		#
		# @sa save(), saveAs(), load()
		self.__strSaveTo = ""

		#Noch kein Content verändert
		## @brief Hold, whether something in the data of turboMachine has changed<br><em>Gibt an ob sich innerhalb der TurboMaschinendaten etwas geändert hat</em>
		#
		# Is needed to determine changes(e.g. on closing or loading).Obviates double calculation.
		# <br><em>Wird zum ermitteln einer Änderung (zBsp beim Beenden oder Laden) benötigt. Beugt Doppelberechnungen vor.
		# </em>
		#
		# @sa load(), exit()
		self.__bIsChanged = False

		#Zufallsgenerator aktivieren
		random.seed()

		#Namen setzen
		self.retranslateStrings(False)

		self._isHierarchieVisible = True





	## @brief Loads icons from the resourcefile<br><em>Läd Icons aus der Ressourcedatei</em>
	#
	# Auxiliary function - The data contained by the resourcefile is in the directory ./UI/res.
	# These icons are needed to tinker additional icons (e.g. paste, icons for bladeRow and so on).
	# <br><em>Hilfsfunktion - Die in der Resourcedatei enthaltenen Daten liegen im Ordner ./UI/res.
	# Diese Icons werden benötigt um weitere Icons zu basteln (zBsp: Einfügen, Icons für BladeRow usw)
	# </em>
	#
	# @sa addProfile(), addBladeRow(), __colorImage(), __combineImages()
	def __createIcons(self):
		#Grafische Spielerreien implementieren
		## @brief Neutral icon of bladeRow<br><em>Neutrales BladeRowIcon</em>
		#
		# Is needed for additional icons.
		# <br><em>Wird für weitere Icons benötigt.
		# </em>
		#
		# @sa __createIcons(), __imgKlemmbrett, __imgProfile, __combineImages(), __colorImage()
		self.__imgBladeRow = QImage(":/Icons/res/bladerow.png")
		## @brief Clipboard icon<br><em>Klemmbretticon</em>
		#
		# Is needed for additional icons.
		# <br><em>Wird für weitere Icons benötigt.
		# </em>
		#
		# @sa __createIcons(), __imgBladeRow, __imgProfile, __combineImages(), __colorImage()
		self.__imgKlemmbrett = QImage(":/Icons/res/klemmbrett.png")
		## @brief Neutral icon of profile<br><em>Neutrales ProfilIcon</em>
		#
		# Is needed for additional icons.
		# <br><em>Wird für weitere Icons benötigt.
		# </em>
		#
		# @sa __createIcons(), __imgKlemmbrett, __imgBladeRow, __combineImages(), __colorImage()
		self.__imgProfile = QImage(":/Icons/res/profile_grau.png")

		#Eines nicht gefunden?
		if (self.__imgProfile.isNull()):
			print "profile_grau.png nicht gefunden"

		if (self.__imgBladeRow.isNull()):
			print "bladerow.png nicht gefunden"

		if (self.__imgKlemmbrett.isNull()):
			print "klemmbrett.png nicht gefunden"






	## @brief Colors an image in the corresponding color<br><em>Färbt ein Bild in der entsprechenden Farbe ein</em>
	#
	# Auxiliary function - Is used to create the color of profile- and bladerow icons.
	# Therefore the transparent color is added to the corresponding icon.
	# <br><em>Hilfsfunktion - Wird verwendet um die Farben für Profile- und BladeRowIcons zuerstellen.
	# Dazu wird einfach die Farbe transparent auf das entsprechende Icon aufgetragen.
	# </em>
	#
	# @param imgSource - Image to be colored from type QImage<br><em>Bild, das eingefärbt werden soll vom Typ QImage</em>
	# @param pColor - Color the Image is to color to from type QColor<br><em>Farbe in dies das Bild gefärbt werden soll vom Typ QColor</em>
	#
	# @return Returns the colored Image of type QImage<br><em>Gibt das eingefäbrte Bild zurück als QImage Typ</em>
	#
	# @sa __combineImages(), __createIcons(), addProfile(), addBladeRow()
	def __colorImage(self, imgSource, pColor):
		#Maske erstellen
		imgMask = QImage(imgSource.width(), imgSource.height(), imgSource.format())
		#pPainter erstellen
		pPainter = QPainter(imgMask)
		#Bild löschen
		pPainter.setCompositionMode(QPainter.CompositionMode_Clear)
		pPainter.eraseRect(imgMask.rect())
		#Bild aufblitten
		pPainter.setCompositionMode(QPainter.CompositionMode_SourceOver)
		pPainter.drawImage(imgMask.rect(), imgSource, imgSource.rect())
		#Einfärben
		pPainter.setCompositionMode(QPainter.CompositionMode_SourceIn)
		pPainter.fillRect(imgMask.rect(), QBrush(pColor))
		#Fertig an der Stelle
		pPainter.end()
		#Anderes Bild erstellen
		imgImage = QImage(imgMask.width(), imgMask.height(), imgMask.format())
		#Bild bemalen
		pPainter.begin(imgImage)
		#Bild löschen
		pPainter.setCompositionMode(QPainter.CompositionMode_Clear)
		pPainter.eraseRect(imgImage.rect())
		#Vorlage kopieren
		pPainter.setCompositionMode(QPainter.CompositionMode_SourceOver)
		pPainter.drawImage(imgImage.rect(), imgSource, imgSource.rect())
		#Maske darüber zeichnen
		pPainter.drawImage(imgImage.rect(), imgMask, imgMask.rect())
		#Fertig
		pPainter.end()
		#Icon zurückgeben
		return imgImage






	## @brief Combines a list of images to a single one<br><em>Kombiniert eine Liste von Bilder zu einem Einzigen</em>
	#
	# Auxiliary function - The images are drawn on each other from bottom to top (left to rigth in list order) and returned as new image.
	# <br><em>Hilfsfunktion - Die Bilder werden von unten nach oben(Liste von links nach rechts) aufeinander gezeichnet und dann als neues Bild zurückgeben.
	# </em>
	#
	# @param lImages - List of images to combine<br><em>Liste mit Bilder, die kombiniert werden sollen</em>
	#
	# @return Returns the combined image of type QImage<br><em>Gibt das kombinierte Bild zurück als QImage Typ</em>
	#
	# @sa __colorImage(), __createIcons(), addProfile(), addBladeRow()
	def __combineImages(self,lImages):
		#Keine Bilder vorhanden?
		if (len(lImages) == 0):
			#Icon ohne Bild zurückgeben
			return None
		
		#Zeichengrundlage erstellen
		imgTemp = QImage(lImages[0].width(),lImages[0].height(),lImages[0].format())
		#pPainter erstellen
		pPainter = QPainter(imgTemp)
		#Bild löschen
		pPainter.setCompositionMode(QPainter.CompositionMode_Clear)
		pPainter.eraseRect(imgTemp.rect())
		#Zeichenmodus auf malen setzen
		pPainter.setCompositionMode(QPainter.CompositionMode_SourceOver)
		#Bilder malen
		for Item in lImages:
			pPainter.drawImage(imgTemp.rect(), Item, Item.rect())
		#Fertig
		pPainter.end()
		#Icon zurückgeben
		return imgTemp
		




	## @brief Creates the contextmenu for the hierarchy-tree<br><em>Erstellt das Kontextmenü für den Hierarchie-Baum</em>
	#
	# Auxiliary function - extend as needed
	# <br><em>Hilfsfunktion - Kann bei Bedarf erweitert werden.</em>
	def createContextMenu(self):
		lActions = []
		#Menü für TreeViewWidget hinzufügen
		lActions.append(self.AAddBladeRow)
		lActions.append(self.AAddProfile)
		lActions.append(self.ARemove)
		#Menü hinzufügen
		self.TWHierarchie.addActions(lActions)





	## @brief Creates a new project/turboMachine<br><em>Erstellt ein neues Projekt/TurboMaschine</em>
	#
	# This function checks, whether there are unsaved changes in the current project and warns in case.
	# Afterwards it removes the hierarchy and creates a new turboMachine. Settings are reset.
	# <br><em>Diese Funktion überprüft ob an dem aktuellen Projekt ungespeicherte Änderung vorliegen und warnt gegebenen falls.
	# Danach löscht es die Hierarchie und erstellt eine neue TurboMaschine. Einstellungswerte werden zurückgesetzt.
	# </em>
	#
	# @param bInitialize - Hold, whether the BladeDesigner is started the first time<br><em>Gibt an, ob der BladeDesigner zum ersten Mal gestartet wurde</em>
	#
	# @sa __init__()
	def new(self, bInitialize=False):
		#Nicht Initialzustand und hat sich etwas verändert?
		if (not bInitialize) and (self.__bIsChanged):
			#Sicherheitsfrage stellen
			iResult = QMessageBox.question(self, self.tr('Neue TurboMaschine erstellen', "Titel des Neu-Warnung"),self.tr('Wollen sie wirklich ein neues Projekt beginnen?\nAlle nicht gespeicherten Änderungen gehen dabei verloren.', "Nachricht des Neu-Warnung"), QMessageBox.Yes, QMessageBox.No)

			#Kein neues Projekt anlegen?
			if (iResult == QMessageBox.No):
				return

			#Ansonsten einfach weiter machen, wir haben ja gewarnt
		
		#Alle Einträge löschen
		self.TWHierarchie.clear()
		#Ersten Eintrag für die Turbomaschine erstellen
		pCurrent = QTreeWidgetItem(self.TWHierarchie)
		self.TWHierarchie.addTopLevelItem(pCurrent)
		pCurrent.setText(0, self.tr("TurboMachine", "Bezeichnung der Turbomachine in der Objekthierarchie"))

		#Ansicht löschen, falls bereits etwas erstellt wurde
		if(self.__pTurboMachineWidget != None):
			self.__saveChanges()


		#TurboMaschinenWidget
		self.__pTurboMachineWidget = QTurboMachineWidget()

		#Index der aktuellen BladeRow
		## @brief Returns the index of the current chosen bladeRow in hierarchy tree<br><em>Gibt den Index der aktuell ausgewählten BladeRow im Hierarchie-Baum an</em>
		#
		# Is used to save or add a new bladeRow or profile.
		# <br><em>Wird zum Speichern und Hinzufügen einer neuen BladeRow oder neuen Profil verwendet.
		# </em>
		#
		# @sa itemClicked(), addBladeRow(), addProfile(), __saveChanges()
		self.__iCurBladeRowIndex = -1

		#Index des aktuellen Profils
		## @brief Returns the index of the current chosen profil in hierarchy tree<br><em>Gibt den Index der aktuell ausgewähltes Profil im Hierarchie-Baum an</em>
		#
		# Is used to save or add a new profile.
		# <br><em>Wird zum Speichern und Hinzufügen eines neuen Profil verwendet.
		# </em>
		#
		# @sa itemClicked(), addProfile(), __saveChanges()
		self.__iCurProfileIndex = -1
		
		## @brief Returns the type of the current selected object in hierarchy-tree<br><em>Gibt den Typ des aktuell ausgewähtlten Objektes im Hierarchie-Baum an</em>
		#
		# The constants defined in QBladeDesigner are used. If an new type of selectable objects is added to hierarchy tree, it is recommended to
		# extend the constants.
		# <br><em>Dazu werden die in QBladeDesigner definierten Konstanten benutzt. Sollte einer neuer Typ auswählbarer Objekte im
		# Hierarchie-Baum hinzu kommen empfiehlt es sich diese Konstanten zu erweitern.
		# </em>
		#
		# @sa __iCurBladeRowIndex, __iCurProfileIndex, itemClicked(), __saveChanges()
		self.iSelectionType = QBladeDesigner.__TurboMachine
		#TurboMaschine als erstes Element selektieren
		self.TWHierarchie.setCurrentItem(pCurrent)

		#TurboMaschinenWidget als erstes Element anzeigen
		self.WidgetLayout.addWidget(self.__pTurboMachineWidget)
		#Erste BladeRow hinzufügen
		self.addBladeRow()

		#TurboMaschine auswählen
		self.TWHierarchie.setCurrentItem(pCurrent)
		self.itemClicked(pCurrent, 0)

		#Zuspeichernde Datei resetten
		self.__strSaveTo = ""
		#Titel setzen
		self.setWindowTitle(self.tr("Unbenannt - ", "Starttitel des BladeDesigners") + self.tr("BladeDesigner", "Name der Applikation"))
		#Noch hat sich nichts verändert
		self.__bIsChanged = False

		#TurboMachine signale hooken
		self.connect(self.__pTurboMachineWidget, SIGNAL("contentChanged()"), self.OnContentChanged)






	## @brief Loads an existing project<br><em>Läd ein bereits angelegtes Projekt</em>
	#
	# This function checks, whether there are unsaved changes and warns in case.
	# Afterwards it deletes the current work space and uses the loading function provided by QTurboMachine to load a new turboMachine.
	# Following the hierarchy tree is created to the corresponding hierarchy of QTurboMachineWidget.
	# In the end the initialization is passed to QTurboMachineWidget.
	# This function is attached to load-menu item in QtDesigner.
	# <br><em>Die Funktion überprüft ob ungespeicherte Änderungen vorliegen und warnt gegebenen falls.
	# Danach löscht es die aktuelle Arbeitsfläche und benutzt die von QTurboMachineWidget bereitgestellte Funktion zum Laden einer TurboMaschine.
	# Im Anschluß erstellt es den Hierarchiebaum und die entsprechende Hierarchie im QTurboMachineWidget.
	# Zum Schluß übergibt es die Initialisierung an das QTurboMachineWidget.
	# Diese Funktion ist an den Laden-Menüeintrag im QtDesigner gehangen.
	# </em>
	#
	# @sa QTurboMachineWidget::loadFromWorkDir(), QTurboMachineWidget::setFromObject(), addProfile(), addBladeRow()
	def load(self):
		print "Start Load"
		#Hat sich etwas geändert?
		if (self.__bIsChanged):
			print "Vor Abfrage"
			#pdb.trace()
			#Sicherheitsfrage stellen
			iResult = QMessageBox.question(self, self.tr('TurboMaschine laden', "Titel des Ladewarnungs"),self.tr('Wollen Sie wirklich ein andere Turbomaschine laden?\nAlle nicht gespeicherten Änderungen gehen dabei verloren.', "Nachricht des Ladewarnung"), QMessageBox.Yes, QMessageBox.No)

			#Kein neues Projekt anlegen?
			if (iResult == QMessageBox.No):
				return

			#Ansonsten einfach weiter machen, wir haben ja gewarnt

		print "FileDlg"
		#Laden-Dialog anzeigen
		strDatei = QFileDialog.getExistingDirectory(self, self.tr("Turbomaschine laden ...", "Titel des Ladendialogs"), ".")

		#Abgebrochen?
		if (strDatei == ""):
			return

		#Aktuelle Anzeige ausklinken
		self.__saveChanges()

		#Starteinstellung wiederherstellen
		self.new(True)

		#Geladene Datei merken
		self.__strSaveTo = strDatei

		#Titel wiederherstellen
		self.setWindowTitle(basename(str(strDatei)) + " - " + self.tr("BladeDesigner", "Name der Applikation"))
		#Noch hat sich nichts verändert
		self.__bIsChanged = False
		#Working-Directory extrahieren
		self.__pTurboMachineWidget.LEWorkDir.setText(self.__strSaveTo)

		#Turbomaschine laden
		self.__pTurboMachineWidget.loadFromWorkDir(self.__strSaveTo)

		#Konfiguration holen
		lConfig = self.__pTurboMachineWidget.output().configuration

		#Erste BladeRow auswählen
		self.TWHierarchie.setCurrentItem(self.TWHierarchie.topLevelItem(0).child(0))
		self.itemClicked(self.TWHierarchie.topLevelItem(0).child(0), 0)

		#Bladerows hinzufügen
		for iCurBladeRow in range(len(lConfig)):
			#Erste BladeRow bereits angelegt, ansonsten anlegen
			if (iCurBladeRow != 0):
				#BladeRow anlegen
				self.addBladeRow()

			#Durch Profile gehen
			for iCurProfile in range(lConfig[iCurBladeRow]):
				#Erstes Profil ist bereits vorhanden
				if (iCurProfile != 0):
					#Erstes Profil bereits angelegt, ansonsten anlegen
					self.addProfile()

		#Werte übernahmen anstoßen
		self.__pTurboMachineWidget.setFromObject(self.__pTurboMachineWidget.output())
				





	## @brief Saves a project<br><em>Speichert ein Projekt</em>
	#
	# If a working directory is already chosen, it is save to the working directory. Otherwise saveAs() is instructed to determine a working directory.
	# There is a check, whether a backup already exists, beacuse turboMachine is not able to overwrite an exsiting backup. Is a backup available, it is
	# removed. Afterwards the content of turboMachine is save via QTurboMachineWidget::saveToWorkDir().
	# This function is attached to save-menu item in QtDesigner.
	# <br><em>Falls bereits ein Arbeitsverzeichnis ausgewählt wurde, wird in dieses Arbeitsverzeichnis gespeichert. Ansonsten wird saveAs() beauftragt ein
	# ein Arbeitsverzeichnis zu ermitteln. Es wird geprüft ob bereits eine Sicherung vorhanden ist, da turboMachine keine erneute Sicherung zulässt. Ist bereits
	# eine Sicherung vorhanden, wird diese entfernt. Danach wird mittels QTurboMachineWidget::saveToWorkDir() der Inhalt des TurboMaschine gespeichert.
	# Diese Funktion ist an den Speichern-Menüeintrag im QtDesigner gehangen.
	# </em>
	#
	# @sa saveAs(), load(), QTurboMachineWidget::saveToWorkDir(), __strSaveTo
	def save(self):
		#Muß gespeichert werden?
		if (not self.__bIsChanged):
			return
		
		#Prüfen ob Standarddateiname schon vergeben?
		if (self.__strSaveTo == ""):
			#Falls nicht, öffnen
			self.saveAs()
			return

		#Prüfen ob Datei bereits vorhanden
		strFileName = self.__strSaveTo + "/" + basename(str(self.__strSaveTo)) + ".xml"
		#Datei öffnen
		pFile = QFile(strFileName)
		#Existiert die Datei bereits?
		if (pFile.exists()):
			#Löschen, da es sonst Konflikte mit der Turbomaschine gibt
			pFile.remove()

		#Titel wiederherstellen
		self.setWindowTitle(basename(str(self.__strSaveTo)) + " - " + self.tr("BladeDesigner", "Name der Applikation"))
		#Working-Directory extrahieren
		self.__pTurboMachineWidget.LEWorkDir.setText(self.__strSaveTo)
		#Nun ist nichts mehr verändert
		self.__bIsChanged = False
		#In XML-Speichern
		self.__pTurboMachineWidget.saveToWorkDir()





	## @brief Saves a existing project in an other working directory<br><em>Speichert ein vorhandenes Projekt in ein anderes Arbeitsverzeichnis</em>
	#
	# Opens a filedialog to determine the file path and passes it to save().
	# This function is attached to save-menu item in QtDesigner.
	# <br><em>Öffnet lediglich einen Dateidialog um einen Speicherpfad zu ermitteln und übergibt das Speichern dann an save().
	# Diese Funktion ist an den Speichern als-Menüeintrag im QtDesigner gehangen
	# </em>
	#
	# @sa save(), __strSaveTo
	def saveAs(self):
		#Speichern-Dialog anzeigen
		self.__strSaveTo = QFileDialog.getExistingDirectory(self, self.tr("Turbomaschine speichern ...", "Titel des Speicherndialogs"), ".")

		#Abgebrochen?
		if (self.__strSaveTo == ""):
			return

		#Speichern
		self.save()
		




	## @brief Exits BladeDesigner<br><em>Beendet den BladeDesigner</em>
	#
	# It is checked, whether there are unsaved changes and warned in case.
	# This function is attached to exit-menu item.
	# <br><em>Zuvor wird geprüft, ob ungespeicherte Änderung vorhanden sind und gegebenen falls gewarnt.
	# Diese Funktion ist an den Beenden-Menüeintrag gehangen.
	# </em>
	#
	# @sa __bIsChanged
	def exit(self):
		#Hat sich etwas geändert?
		if (self.__bIsChanged):
			#Sicherheitsfrage stellen
			iResult = QMessageBox.question(self, self.tr('BladeDesigner Beenden', "Titel der Beendenwarunung"),self.tr('Wollen Sie BladeDesigner wirklich beenden?\nAlle nicht gespeicherten Änderungen gehen dabei verloren.', "Nachricht der Beendenwarnung"), QMessageBox.Yes, QMessageBox.No)

			#Kein neues Projekt anlegen?
			if (iResult == QMessageBox.No):
				return

			#Ansonsten einfach weiter machen, wir haben ja gewarnt
		self.close()





	## @brief Copies a selected object<br><em>Kopiert ein ausgewähltes Objekt</em>
	#
	# First it is checked, what to copy. Then the corresponding object is copied to the clipboard and the paste-button is adjusted.
	# This function is attached to copy-menu item and copy-button in QtDesigner.
	# <br><em>Erst wird geprüft, was kopiert werden soll. Dann wird eine das entsprechende Objekt in die Zwischenablage kopiert und der
	# Einfügen-Button entsprechend angepasst. Diese Funktion hängt im QtDesigner am Kopieren-Menüeintrag und Kopieren-Button.
	# </em>
	#
	# @sa __Clipboard, __ClipboardContent, __colorImage(), __combineImages(), paste(), cut()
	def copy(self):
		#Was sollen wir kopieren?
		#Turbomaschine?
		if (self.iSelectionType == QBladeDesigner.__TurboMachine):
			#Geht noch nicht, wieso auch?
			return

		pTMW = self.__pTurboMachineWidget

		#BladeRow?
		if (self.iSelectionType == QBladeDesigner.__BladeRow):
			#Kopie von der aktuell ausgewählten Bladerow ziehen
			self.__Clipboard = QBladeRowWidget()
			#Kopie vom BladeRowWidget das angezeigt wird ziehen
			self.__Clipboard.copyFrom(pTMW.getBladeRowWidget(self.__iCurBladeRowIndex))
			#BladeRow kopiert
			self.__ClipboardContent = QBladeDesigner.__BladeRow
			#Speichern ob der Eintrag ausgeklappt war
			self.__ClipboardContentExpanded = self.TWHierarchie.currentItem().isExpanded()
			#Icon auf BladeRow setzen
			#Icon erstellen
			#Erst einfärben
			imgIcon = self.__colorImage(self.__imgBladeRow, pTMW.getBladeRowWidget(self.__iCurBladeRowIndex).color())
			#Dann aufs Klemmbrett pappen
			imgIcon = self.__combineImages([self.__imgKlemmbrett, imgIcon])
			#Icon erstellen
			self.APaste.setIcon(QIcon(QPixmap.fromImage(imgIcon)))
			return

		#Ansonsten Profil
		#Kopie von dem aktuell ausgewählten Profil machen
		self.__Clipboard = QProfile2DWidget()
		#Kopieren
		self.__Clipboard.copyFrom(pTMW.getBladeRowWidget(self.__iCurBladeRowIndex).getProfileWidget(self.__iCurProfileIndex))
		#Profil kopiert
		self.__ClipboardContent = QBladeDesigner.__Profile
		#Icon auf Profil setzen
		#Gewähltes Icon holen
		#Erst einfärben
		imgIcon = self.__colorImage(self.__imgProfile, pTMW.getBladeRowWidget(self.__iCurBladeRowIndex).getProfileWidget(self.__iCurProfileIndex).color())
		#Dann aufs Klemmbrett pappen
		imgIcon = self.__combineImages([self.__imgKlemmbrett, imgIcon])
		#Neues Icon setzen
		self.APaste.setIcon(QIcon(QPixmap.fromImage(imgIcon)))




	## @brief Pastes the object from the clipboard<br><em>Fügt das Objekt aus der Zwischenablage ein</em>
	#
	# First it is checked, whether there is a valid object in the clipboard. If so it is pasted.
	# This function is attached to paste-menu item and paste-button in QtDesigner.
	# <br><em>Erst wird geprüft ob sich ein gültiges Objekt in der Zwischenablage befindet. Falls ja wird es eingefügt.
	# Diese Funktion ist an den Einfügen-Menüeintrag und den Einfügen-Button im QtDesigner gehangen.
	# </em>
	#
	# @sa __Clipboard, __ClipboardContent, cut(), copy(), addProfile(), addBladeRow()
	def paste(self):
		#BladeRow einfügen
		if (self.__ClipboardContent == QBladeDesigner.__BladeRow):
			#BladeRow aus der Zwischenablage einfügen
			self.addBladeRow(self.__iCurBladeRowIndex, self.__Clipboard)
			#Expansionsmodus setzen
			self.TWHierarchie.currentItem().setExpanded(self.__ClipboardContentExpanded)
			return

		#Profil einfügen
		if (self.__ClipboardContent == QBladeDesigner.__Profile):
			#Profil aus der Zwischenablade einfügen
			self.addProfile(self.__iCurProfileIndex, self.__Clipboard)





	## @brief Cuts a selected object<br><em>Schneidet ein ausgewähltes Objekt aus</em>
	#
	# Basically it is first copy, then remove.
	# This function is attached to cut-menu item and cut-button in QtDesigner.
	# <br><em>Im Grunde ist es erst kopieren, dann entfernen.
	# Diese Funktion hängt im QtDesigner am Ausschneiden-Menüeintrag und Ausschneiden-Button.
	# </em>
	#
	# @sa copy(), remove(), paste()
	def cut(self):
		#Ausschneiden ist kopieren und entfernen
		#Also erst kopieren
		self.copy()
		#Dann entfernen
		self.remove()





	## @brief Removes a selected object<br><em>Entfernt ein ausgewähltes Objekt</em>
	#
	# First it is checked, whether a valid entry is chosen und removed afterwards.
	# This function is attached to remove-menu item and remove-button in QtDesigner.
	# <br><em>Prüft zuerst ob gültige Einträge ausgewählt sind und entfernt dann entsprechend.
	# Diese Funktion ist im QtDesigner am Entfernen-Menüeintrag und Entfernen-Button angehangen.
	# </em>
	#
	# @sa cut(), iSelectionType, delBladeRow(), delProfile()
	def remove(self):
		#Was soll denn entfernt werden?
		#Ist es die Turbomaschine?
		if (self.iSelectionType == QBladeDesigner.__TurboMachine):
			#Turbomaschine kann nicht vernichtet werden
			return

		#Handelt es sich um eine BladeRow?
		if (self.iSelectionType == QBladeDesigner.__BladeRow):
			#BladeRow löschen
			self.delBladeRow()
			return

		#Ansonsten ist es ein Profil
		#Profil löschen
		self.delProfile()





	## @brief Shows the about dialog<br><em>Zeigt den "Über"-Dialog an</em>
	#
	# This function simply creates the about dialog and shows it modal
	# <br><em>Diese Funktion erstellt lediglich den "Über"-Dialog und zeigt ihn modal an.
	# </em>
	#
	# @sa QAboutDialog
	def about(self):
		#AboutDialog modal anzeigen
		pDlg = QAboutDialog()
		pDlg.exec_()
		
		




	## @brief Pushes up an entry in hierachy tree<br><em>Verschiebt einen Eintrag im Hierarchie-Baum nach oben</em>
	#
	# First it checks, what type of entry it is and whether it can be pushed further up.
	# If it is possible it is swaped with the entry above it. Finally the labels are renumbered.
	# <br><em>Prüft zuerst, welche Art Eintrag es ist und ob dieser weiter nach oben verschoben werden kann.
	# Falls ja, wird einfach mit dem darüber liegenden Eintrag getauscht. Danach werden die Label neu durchnummeriert.
	# </em>
	#
	# @sa down(), QTurboMachineWidget::swapBladeRowWidgets(), QBladeRowWidget::swapProfileWidgets(), relabelIndices()
	def up(self):
		#Wobei handelt es sich bei den Ausgewählten Objekt?
		#Ist es die Turbomaschine?
		if (self.iSelectionType == QBladeDesigner.__TurboMachine):
			#Turbomaschine kann nicht verschoben werden
			return

		#Handelt es sich um eine BladeRow?
		if (self.iSelectionType == QBladeDesigner.__BladeRow):
			#Die oberste Bladerow kann nicht weiter nach oben verschoben werden
			if (self.__iCurBladeRowIndex == 0):
				return

			#Index mit der oberen Bladerow tauschen
			self.__pTurboMachineWidget.swapBladeRowWidgets(self.__iCurBladeRowIndex, self.__iCurBladeRowIndex-1)

			#Speichern ob der Meüpunkt ausgeklappt war oder nicht
			bIsExpanded = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).isExpanded()
			#zu tauschende Bladerow herausnehmen
			pBladeRowItem = self.TWHierarchie.topLevelItem(0).takeChild(self.__iCurBladeRowIndex)
			#Einen Index weiter oben einfügen
			self.TWHierarchie.topLevelItem(0).insertChild(self.__iCurBladeRowIndex-1, pBladeRowItem)
			#Expandmodus wieder einstellen
			pBladeRowItem.setExpanded(bIsExpanded)
			#Index anpassen
			self.__iCurBladeRowIndex -= 1
			#Geschobenes selektiert lassen
			self.TWHierarchie.setCurrentItem(pBladeRowItem)
			#Label anpassen
			self.relabelIndices()
			return

		#Ansonsten ist es ein Profil
		#Das oberste Profil kann nicht weiter nach oben verschoben werden
		if (self.__iCurProfileIndex == 0):
			return

		#Zugehöriges BladeRowWidget holen
		pBladeRowWidget = self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex)

		#Index mit den oberen Profile tauschen
		pBladeRowWidget.swapProfileWidgets(self.__iCurProfileIndex-1, self.__iCurProfileIndex)

		#zu tauschendes Profil herausziehen
		pProfileItem = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).takeChild(self.__iCurProfileIndex)
		#Einen Index weiter oben wieder reinstecken
		self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).insertChild(self.__iCurProfileIndex-1, pProfileItem)
		#Geschobenes selektiert lassen
		self.TWHierarchie.setCurrentItem(pProfileItem)
		#Index des aktuellen Profiles wechseln
		self.__iCurProfileIndex -= 1
		#Label anpassen
		self.relabelIndices()





	## @brief Pushes down an entry in hierachy tree<br><em>Verschiebt einen Eintrag im Hierarchie-Baum nach unten</em>
	#
	# First it is checked, what type of entry it is and whether it can be pushed further down.
	# If it is possible it is swaped with the entry under it. Finally the labels are renumbered.
	# <br><em>Prüft zuerst, welche Art Eintrag es ist und ob dieser weiter nach unten verschoben werden kann.
	# Falls ja, wird einfach mit dem darunter liegenden Eintrag getauscht. Danach werden die Label neu durchnummeriert.
	# </em>
	#
	# @sa up(), QTurboMachineWidget::swapBladeRowWidgets(), QBladeRowWidget::swapProfileWidgets(), relabelIndices()
	def down(self):
		#Wobei handelt es sich bei den Ausgewählten Objekt?
		#Ist es die Turbomaschine?
		if (self.iSelectionType == QBladeDesigner.__TurboMachine):
			#Turbomaschine kann nicht verschoben werden
			return

		#Handelt es sich um eine BladeRow?
		if (self.iSelectionType == QBladeDesigner.__BladeRow):
			#Die unterste Bladerow kann nicht weiter nach oben verschoben werden
			if (self.__iCurBladeRowIndex == (self.__pTurboMachineWidget.anzBladeRowWidgets()-1)):
				return

			#Index mit der unteren Bladerow tauschen
			self.__pTurboMachineWidget.swapBladeRowWidgets(self.__iCurBladeRowIndex, self.__iCurBladeRowIndex+1)

			#Speichern ob der Meüpunkt ausgeklappt war oder nicht
			bIsExpanded = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).isExpanded()
			#zu tauschende Bladerow herausnehmen
			pBladeRowItem = self.TWHierarchie.topLevelItem(0).takeChild(self.__iCurBladeRowIndex)
			#Einen Index weiter oben einfügen
			self.TWHierarchie.topLevelItem(0).insertChild(self.__iCurBladeRowIndex+1, pBladeRowItem)
			#Expandmodus wieder einstellen
			pBladeRowItem.setExpanded(bIsExpanded)
			#Index anpassen
			self.__iCurBladeRowIndex += 1
			#Geschobenes selektiert lassen
			self.TWHierarchie.setCurrentItem(pBladeRowItem)
			#Label anpassen
			self.relabelIndices()
			return

		#Ansonsten ist es ein Profil
		#Das unterstes Profil kann nicht weiter nach unten verschoben werden
		if (self.__iCurProfileIndex == (self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex).anzProfileWidgets()-1)):
			return

		#Zugehöriges BladeRowWidget holen
		pBladeRowWidget = self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex)

		#Index mit den unteren Profile tauschen
		pBladeRowWidget.swapProfileWidgets(self.__iCurProfileIndex+1, self.__iCurProfileIndex)

		#zu tauschendes Profil herausziehen
		pProfile = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).takeChild(self.__iCurProfileIndex)
		#Einen Index weiter oben wieder reinstecken
		self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).insertChild(self.__iCurProfileIndex+1, pProfile)
		#Geschobenes selektiert lassen
		self.TWHierarchie.setCurrentItem(pProfile)
		#Index des aktuellen Profiles wechseln
		self.__iCurProfileIndex += 1
		#Label anpassen
		self.relabelIndices()





	## @brief Adds a new bladeRow<br><em>Fügt eine neue BladeRow hinzu</em>
	#
	# Creates a new entry in hierarchy tree. Then depending on the situation an icon and new QBladeRowWidget is created. A first QProfile2DWidget is also
	# added. Afterwards the new QBladeRowWidget is add to the QTurboMachineWidget and the labels are reordered. The new QBladeRowWidget is
	# selected and show immediately.
	# <br><em>Erstellt erst einen neuen Eintrag im Hierarchie-Baum. Danach wird je nachdem eine neues Icon erstellt sowie eine neues QBladeRowWidget.
	# Ein erstes QProfile2DWidget wird ebenfalls hinzugefügt. Anschließend wird das neue QBladeRowWidget dem QTurboMachineWidget hinzugefügt und die
	# Label entsprechend neu geordnet. Das neue QBladeRowWidget wird sofort ausgewählt und angezeigt.
	# </em>
	#
	# @param iIndex - Index of the list the new QBladeRowWidget is added to<br><em>Index in der Liste in der das QBladeRowWidget eingefügt werden soll</em>
	# @param pBladeRow - QBladeRowWidget to add(optional)<br><em>QBladeRowWidget das eingefügt werden soll(optional)</em>
	#
	# @sa addProfile(), relabelIndices(), QTurboMachineWidget::QTurboMachineWidget, QBladeRowWidget::QBladeRowWidget
	def addBladeRow(self, iIndex = -1, pBladeRow = None):
		#Kein konkreter Index ausgewählt?
		if (iIndex == -1):
			#Am Ende einfügen
			iIndex = self.__pTurboMachineWidget.anzBladeRowWidgets()
		
		#Neues BladeRowObjekt erzeugen
		pNewBladeRowItem = QTreeWidgetItem()
		#Neue Bladerow erzeugen
		pNewBladeRowWidget = QBladeRowWidget()

		#Text setzen
		pNewBladeRowItem.setText(0, self.tr("Bladerow ", "Bezeichnung der BladeRow in der Objekthierarchie") + str(iIndex+1))
		
		#Von vorhandener BladeRow erstellen?
		if (pBladeRow == None):
			#Farbe zusammenwürfeln
			pColor = self.__generateColor()
			#Neues Icon setzen
			pNewBladeRowItem.setIcon(0, QIcon(QPixmap.fromImage(self.__colorImage(self.__imgBladeRow, pColor))))
			#Farbe in die BladeRow packen für spätere Benutzung
			pNewBladeRowWidget.setColor(pColor)
		else:
			#Vorhandenes Icon wählen
			pNewBladeRowItem.setIcon(0, QIcon(QPixmap.fromImage(self.__colorImage(self.__imgBladeRow, pBladeRow.color()))))
					
		#Zum Toplevel eine weitere Bladerow hinzufügen
		self.TWHierarchie.topLevelItem(0).insertChild(iIndex, pNewBladeRowItem)
		self.TWHierarchie.setCurrentItem(pNewBladeRowItem, True)

		#Nummer der BladeRow anpassen
		#TODO: Zur Not +1
		pNewBladeRowWidget.LERowNumber.setText(str(iIndex))

		#Link auf TurbomachineWidget
		pTMW = self.__pTurboMachineWidget
		
		#Neues BladeRowWidget ins Array einfügen
		pTMW.insertBladeRowWidget(iIndex, pNewBladeRowWidget)


		#Anzahl der BladeRows und Profile in der Turbomaschine anpassen
		pTMW.LEAnzBladeRows.setText(str(pTMW.anzBladeRowWidgets()))

		#Wenn nicht am Ende eingefügt
		if(iIndex != (pTMW.anzBladeRowWidgets()-1)):
			#Dann ist die darunterliegende BladeRow ausgewählt, Index anpassen
			self.__iCurBladeRowIndex += 1

		#Neue BladeRow auswählen
		self.itemClicked(pNewBladeRowItem, 0)

		#Wenn nicht am Schluß angehangen(Index != -1) neu durchnummerieren
		self.relabelIndices()

		#Neue BladeRow erstellen?
		if (pBladeRow == None):
			#Profil hinzufügen
			self.addProfile()
		else:
			#Von einer vorhandenen BladeRow kopieren
			pNewBladeRowWidget.copyFrom(pBladeRow)
			#Profile als Untereintrag hinzufügen
			for iProfile in range(pNewBladeRowWidget.anzProfileWidgets()):
				#Neuen Profileintrag erstellen
				pProfileItem = QTreeWidgetItem()
				#Text setzen
				pProfileItem.setText(0,self.tr("Profile ", "Bezeichner des Profiles in der Objekthierarchie")+str(iProfile+1))
				#Icon erstellen
				imgIcon = self.__colorImage(self.__imgProfile, pNewBladeRowWidget.getProfileWidget(iProfile).color())
				#Icon setzen
				pProfileItem.setIcon(0, QIcon(QPixmap.fromImage(imgIcon)))
				#Profil anhängen
				pNewBladeRowItem.addChild(pProfileItem)




	## @brief Adds a new profile<br><em>Fügt eine neues Profil hinzu</em>
	#
	# First this function gets the bladeRow the profile is to add. Then a new entry in hierarchy tree is added.
	# Afterwards depending on the situation a new icon and QProfile2DWidget is created. Finally the new QProfile2DWidget is
	# added to the QBladeRowWidget and the labels are reordered. The new QProfile2DWidget is selected and shown
	# immediately.
	# <br><em>Die Funktion holt sich zuerst die BladeRow in der das Profil hinzugefügt werden soll.
	# Erstellt dann erst einen neuen Eintrag im Hierarchie-Baum. Danach wird je nachdem eine neues Icon erstellt sowie eine neues QProfile2DWidget.
	# Anschließend wird das neue QProfile2DWidget dem QBladeRowWidget hinzugefügt und die
	# Label entsprechend neu geordnet. Das neue QProfile2DWidget wird sofort ausgewählt und angezeigt.
	# </em>
	#
	# @param iIndex - Index of the list the QProfile2DWidget is to add<br><em>Index in der Liste in der das QProfile2DWidget eingefügt werden soll</em>
	# @param pProfile - QProfile2DWidget to add(optional)<br><em>QProfile2DWidget das eingefügt werden soll(optional)</em>
	#
	# @sa addProfile(), relabelIndices(), QTurboMachineWidget::QTurboMachineWidget, QBladeRowWidget::QBladeRowWidget
	def addProfile(self, iIndex = -1, pProfile = None):
		#Turbomaschine ausgewählt?
		if (self.iSelectionType == QBladeDesigner.__TurboMachine):
			#Da kann man schlecht Profile hinzufügen
			return

		
		#Zur letztens ausgewählen Bladerow ein neues Profil hinzufügen
		#BladeRow holen
		pBladeRowWidget = self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex)
		#Kein konkreter Index ausgewählt?
		if (iIndex == -1):
			#Am Ende einfügen
			iIndex = pBladeRowWidget.anzProfileWidgets()

		#BladeRow ausgewählt
		#Neues Profil erstellen
		pNewProfileItem = QTreeWidgetItem()
		#Neues Widget erstellen
		pNewProfileWidget = QProfile2DWidget()

		#Icon einstellen
		if (pProfile == None):
			#Neue Farbe würfeln
			pColor = self.__generateColor()
			#Icon einstellen
			pNewProfileItem.setIcon(0, QIcon(QPixmap.fromImage(self.__colorImage(self.__imgProfile, pColor))))
			#Farbe ins neue Profil packen, für spätere Verwendung
			pNewProfileWidget.setColor(pColor)
		else:
			#Vom vorhandenen ProfileWidget kopieren
			pNewProfileWidget.copyFrom(pProfile)
			#Vorhandenes verwenden
			pNewProfileItem.setIcon(0, QIcon(QPixmap.fromImage(self.__colorImage(self.__imgProfile, pNewProfileWidget.color()))))


		#Profil in die BladeRow packen
		self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).insertChild(iIndex, pNewProfileItem)
		#Text setzen
		pNewProfileItem.setText(0, self.tr("Profile ", "Bezeichner des Profiles in der Objekthierarchie") + str(iIndex+1))
		#Neues Profile in die BladeRow packen
		pBladeRowWidget.insertProfileWidget(iIndex, pNewProfileWidget)
		#Selektieren
		self.TWHierarchie.setCurrentItem(pNewProfileItem, True)

		

		#Wenn nicht am Ende eingefügt
		if(iIndex != (pBladeRowWidget.anzProfileWidgets()-1)):
			#Dann ist das darunterliegende Profil ausgewählt, Index anpassen
			self.__iCurProfileIndex += 1

		##Neues Profil auswählen
		self.itemClicked(pNewProfileItem, 0)

		#Nummerierung anpassen
		self.relabelIndices()




		
	## @brief Deletes a selected profile<br><em>Löscht das ausgewählte Profil</em>
	#
	# First it is checked, whether the selected profile is the last one in the bladeRow, because a bladeRow without any profile does not make sense. Afterwards the profile
	# is removed depending on its position and the labels are reordered.
	# <br><em>Prüft erst, ob es das letzte Profil in der BladeRow ist, da BladeRows ohne Profil keinen Sinn machen. Danach entfernt es abhängig von der Position in der
	# Liste das entsprechende Profil und berechnet die Labels neu.
	# </em>
	#
	# @sa delBladeRow(), relabelIndices(), __iCurProfileIndex, __iCurBladeRowIndex
	def delProfile(self):
		#BladewRow holen
		pBladeRow = self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex)
		
		#Nur noch ein Profil vorhanden?
		if (pBladeRow.anzProfileWidgets() == 1):
			#Letzte Bladerow kann nicht gelöscht werden
			QMessageBox.information(self, self.tr("Profil löschen", "Titel der Fehlermeldung beim Profillöschen"), self.tr("Eine BladeRow besitzt mindestens ein Profil!", "Nachricht der Fehlermeldung beim Profillöschen"),
						QMessageBox.Ok)
			return
		
		#Oberstes Profil
		if (self.__iCurProfileIndex == 0):
			#Erst Ansicht wechseln, da sonst itemClicked das entfernte Profil
			#wieder in das Profil darunter zurückschreibt, da __saveChanges aufgerufen wird
			#Eintrag unter dem Profil anzeigen
			pNewSelection = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).child(self.__iCurProfileIndex+1)
			self.TWHierarchie.setCurrentItem(pNewSelection)
			self.itemClicked(pNewSelection, 0)

			#Wenn die Anzeige aktualisiert wurde, kann das Profil gelöscht werden
			#Profil aus der Liste löschen
			#Index-1 da jetzt ja das Profil unter dem zu löschenden ausgewählt ist
			self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).takeChild(self.__iCurProfileIndex-1)
			#ProfileWidget aus der Liste löschen
			pBladeRow.delProfileWidget(self.__iCurProfileIndex-1)

			#Verschobener Index anpassen, da wir das Profil unter der zu löschenden
			#ausgewählt haben, steht jetzt der ProfilIndex auf 1 statt auf 0, wo er eigentlich ist
			self.__iCurProfileIndex -= 1

			#Nummer des obersten Profils anpassen
			self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).child(0).setText(0, self.tr("Profile ", "Bezeichner des Profiles in der Objekthierarchie") + str(1))
		else:
			#Erst Ansicht wechseln, da sonst itemClicked das entfernte Profil
			#wieder in das Profil darüber zurückschreibt, da __saveChanges aufgerufen wird
			#Eintrag über dem Profil anzeigen
			pNewSelection = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).child(self.__iCurProfileIndex-1)
			self.TWHierarchie.setCurrentItem(pNewSelection)
			self.itemClicked(pNewSelection, 0)

			#Wenn die Anzeige aktualisiert wurde, kann dasProfil gelöscht werden
			#Profil aus der Liste löschen
			#Index+1 da jetzt ja das Profil über der zu löschenden ausgewählt ist
			self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).takeChild(self.__iCurProfileIndex+1)
			#BladeRowWidget aus der Liste löschen
			pBladeRow.delProfileWidget(self.__iCurProfileIndex+1)

		#Reihenfolge der Indices anpassen
		self.relabelIndices()





	## @brief Deletes the selected bladeRow<br><em>Löscht die ausgewählte BladeRow</em>
	#
	# First it is checked, whether this is the last bladeRow in turboMachine, because a turboMachine without a bladeRow does not make sense. Afterwards the bladeRow is
	# removed depending on its position and the labels are reordered.
	# <br><em>Prüft erst, ob es die letzte BladeRow in der TurboMaschine ist, da TurboMaschinen ohne BladeRow keinen Sinn machen. Danach entfernt es abhängig von der Position in der
	# Liste das entsprechende BladeRow und berechnet die Labels neu.
	# </em>
	#
	# @sa delProfile(), relabelIndices(), __iCurProfileIndex, __iCurBladeRowIndex
	def delBladeRow(self):
		#Nur noch eine BladeRow vorhanden?
		if (self.__pTurboMachineWidget.anzBladeRowWidgets() == 1):
			#Letzte Bladerow kann nicht gelöscht werden
			QMessageBox.information(self, self.tr("BladeRow löschen", "Titel der Fehlermeldung beim BladeRow löschen"), self.tr("Eine Turbomaschine besteht aus mindestens einer Bladerow!", "Nachricht der Fehlermeldung beim BladeRow löschen"),
						QMessageBox.Ok)
			return
		
		#Oberste BladeRow?
		if (self.__iCurBladeRowIndex == 0):
			#Erst Ansicht wechseln, da sonst itemClicked die entfernte Bladerow
			#wieder in die BladeRow darüber zurückschreibt, da __saveChanges aufgerufen wird
			#Eintrag unter der BladeRow anzeigen
			pNewSelection = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex+1)
			self.TWHierarchie.setCurrentItem(pNewSelection)
			self.itemClicked(pNewSelection, 0)

			#Wenn die Anzeige aktualisiert wurde, kann die BladeRow gelöscht werden
			#BladeRow aus der Liste löschen
			#Index-1 da jetzt ja die BladeRow unter der zu löschenden ausgewählt ist
			self.TWHierarchie.topLevelItem(0).takeChild(self.__iCurBladeRowIndex-1)
			#BladeRowWidget aus der Liste löschen
			self.__pTurboMachineWidget.delBladeRowWidget(self.__iCurBladeRowIndex-1)

			#Verschobener Index anpassen, da wir die BladeRow unter der zu löschenden
			#ausgewählt haben, steht jetzt der BladeRowIndex auf 1 statt auf 0, wo er eigentlich ist
			self.__iCurBladeRowIndex -= 1

			#Nummer der oberen BladeRow anpassen
			self.TWHierarchie.topLevelItem(0).child(0).setText(0, self.tr("Bladerow ", "Bezeichnung der BladeRow in der Objekthierarchie") + str(1))
		else:
			#Erst Ansicht wechseln, da sonst itemClicked die entfernte Bladerow
			#wieder in die BladeRow darüber zurückschreibt, da __saveChanges aufgerufen wird
			#Eintrag über der BladeRow anzeigen
			pNewSelection = self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex-1)
			self.TWHierarchie.setCurrentItem(pNewSelection)
			self.itemClicked(pNewSelection, 0)

			#Wenn die Anzeige aktualisiert wurde, kann die BladeRow gelöscht werden
			#BladeRow aus der Liste löschen
			#Index+1 da jetzt ja die BladeRow über der zu löschenden ausgewählt ist
			self.TWHierarchie.topLevelItem(0).takeChild(self.__iCurBladeRowIndex+1)
			#BladeRowWidget aus der Liste löschen
			self.__pTurboMachineWidget.delBladeRowWidget(self.__iCurBladeRowIndex+1)

		#Indices wieder richtig benennen
		self.relabelIndices()





	## @brief Changes the language to german<br><em>Wechselt die Sprache zu Deutsch</em>
	#
	# Is attached to german-menu item in QtDesigner.
	# <br><em> Ist an den Deutsch-Menüeintrag im QtDesigner gehangen.
	# </em>
	#
	# @sa changeToEnglish(), languageChanged()
	def changeToGerman(self):
		#Aufruf mit entsprechenden Index
		#Der Entsprechende Index ist der Platz der Sprachdatei in der Ressourcedatei
		#Enstprechend müßen auch die Menüs benannt sein
		#zBsp:
		#<qresource prefix="Sprache" >
   		#	<file>res/GUI_de.qm</file>		-> Deutsche Sprachdatei = Index 0
   		#	<file>res/GUI_eng.qm</file>		-> Englische Sprachdatei = Index 1
 		#</qresource>
		self.languageChanged(0)





	## @brief Changes the language to english<br><em>Wechselt die Sprache zu Englisch</em>
	#
	# Is attached to english-menu item in QtDesigner.
	# <br><em> Ist an den English-Menüeintrag im QtDesigner gehangen.
	# </em>
	#
	# @sa changeToGerman(), languageChanged()
	def changeToEnglish(self):
		#Für Informationen siehe changeToGerman(self)
		self.languageChanged(1)





	## @brief Changes the language<br><em>Wechselt die Sprache</em>
	#
	# It is checked, whether there is a language file to the passed index. If so, the language is changed and retranslation is called.
	# <br><em>Prüft ob es an dem Index auch eine Sprachdatei gibt. Falls ja, wird die Sprache geändert und die neu Übersetzung in Gang gesetzt.
	# </em>
	#
	# @param iIndex - Index of the selected language file<br><em>Index der ausgewählten Sprachdatei</em>
	#
	# @sa __lTranslations, changeToEnglish(), changeToGerman()
	def languageChanged(self, iIndex):
		#Einträge holen
		lActions = self.MSprache.actions()
		#Durch alle Einträge im Sprachenmenü gehen
		for iEntry in range(len(lActions)):
			#Eintrag der gesuchte?
			if (iEntry == iIndex):
				#Eintrag anhaken
				lActions[iEntry].setChecked(True)
			else:
				#Aushaken
				lActions[iEntry].setChecked(False)

		#Sprache entfernen
		self.__pApp.removeTranslator(self.__lTranslations[self.__iCurTranslator])
		#Neue Sprache setzen
		self.__iCurTranslator = iIndex
		#Erste Übersetzung einstellen
		self.__pApp.installTranslator(self.__lTranslations[self.__iCurTranslator])

		#Neubennen
		self.retranslateStrings()





	## @brief Renumbers the labels of the entrys in hierarchy tree correctly<br><em>Nummeriert die Label der Einträge des Hierarchie-Baums korrekt durch</em>
	#
	# Auxiliary function - It is in charge of the correct numeration of bladeRows and profiles independed of changes in the order (by up, down, cut, cop and so on).
	# <br><em>Hilfsfunktion - Sorgt dafür, dass die Nummerierung bei BladeRows und Profilen stimmt unabhängig davon ob man mit ihnen arbeitet
	# (hoch, runter, ausschneiden, kopieren, usw ...)
	# </em>
	#
	# @sa up(), down(), paste(), remove(), cut()
	def relabelIndices(self):
		#BladeRow neu durchzählen?
		if (self.iSelectionType == QBladeDesigner.__BladeRow):
			#Reihenfolge der Indices anpassen
			for iIndex in range(self.__pTurboMachineWidget.anzBladeRowWidgets()):
				#Text ändern
				self.TWHierarchie.topLevelItem(0).child(iIndex).setText(0, self.tr("Bladerow ", "Bezeichnung der BladeRow in der Objekthierarchie")+str(iIndex+1))
				self.__pTurboMachineWidget.getBladeRowWidget(iIndex).LERowNumber.setText(str(iIndex))
			return

		#Profiles neu durchzählen
		#BladeRow holen
		pBladeRow = self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex)
		#Reihenfolge der Indices anpassen
		for iIndex in range(pBladeRow.anzProfileWidgets()):
			#Text ändern
			self.TWHierarchie.topLevelItem(0).child(self.__iCurBladeRowIndex).child(iIndex).setText(0, self.tr("Profile ", "Bezeichner des Profiles in der Objekthierarchie")+str(iIndex+1))






	## @brief Shows the corresponding Widget when clicked at an entry in hierarchy tree<br><em>Zeigt beim Klicken auf einen Eintrag im Hierarchie-Baum das entsprechende Widget an</em>
	#
	# Therefore the clicked entry is determined and the new Widget is shown. Changes to the previous shown Widget are saved before.
	# This function is attached to mouseclick-event in hierarchy tree in QtDesigner.
	# <br><em>Dazu wird der angeklickte Eintrag ermittelt und auf dessen Basis der das neue Widget angezeigt. Änderungen im
	# vorher angezeigten Widget werden zuvor gespeichert. Diese Funktion ist mit der MausKlick-Funktion des Hierarchie-Baumes im QtDesigner
	# verbungen.
	# </em>
	#
	# @param pItem - Clicked item<br><em>Angeklicktes Item</em>
	# @param iColum - Clicked Column(uninteresting)<br><em>Angeklickte Spalte(uninteressant)</em>
	#
	# @sa __iCurBladeRowIndex, __iCurProfileIndex, iSelectionType, __saveChanges()
	def itemClicked(self, pItem, iColum):
		#Typ des Ausgewählten Element noch nicht bestimmt
		self.iSelectionType = -1
		#Prüfen was das Item ist(Profil, BladeRow, TurboMaschine)
		#Auf Toplevel gibt es nur die Turbomaschine
		if (self.TWHierarchie.indexOfTopLevelItem(pItem) != -1):
			#Also falls es da vorkommt, ist es eine
			self.iSelectionType = QBladeDesigner.__TurboMachine
		#Darunter gibt es nur BladeRows
		elif (self.TWHierarchie.indexOfTopLevelItem(pItem.parent()) != -1):
			#Also falls der Vater des Items die Turbomaschine ist, ist es eine Bladerow
			self.iSelectionType = QBladeDesigner.__BladeRow
		#Ganz unten gibt es nur Profile
		elif (self.TWHierarchie.indexOfTopLevelItem(pItem.parent().parent()) != -1):
			#Falls also das Item ein Profil ist, ist dessen Vater eine BladeRow und dessen Vater wiederum die Turbomaschine
			self.iSelectionType = QBladeDesigner.__Profile
		
		#Schauen ob gültig
		if(self.iSelectionType == -1):
			#Nanu?
			return

		#Ist es die Turbomaschine?
		if(self.iSelectionType == QBladeDesigner.__TurboMachine):
			#ZentralWidget ändern
			#self.setCentralWidget(self.__pTurboMachineWidget)
			self.__pTurboMachineWidget.setVisible(True)
			self.WidgetLayout.addWidget(self.__pTurboMachineWidget)
			#Vor dem Wechseln aktuelles Widget sichern
			self.__saveChanges()
			#Als nächstes wird das TurboMaschinenWidget angezeigt, deshalb Indices ausnegieren
			self.__iCurProfileIndex = -1
			self.__iCurBladeRowIndex = -1
			#Fertig mit dem Wechsel
			return

		#Ist es eine BladeRow?
		if(self.iSelectionType == QBladeDesigner.__BladeRow):
			#BladeRowWidget anzeigen
			#BladeRowIndex ermitteln
			iNewBladeRowIndex = pItem.parent().indexOfChild(pItem)
			#Kein Profil wird angezeigt
			iNewProfileIndex = -1
			#Falls sich nichts geändert hat, können wir uns den wechsel sparen
			if (iNewProfileIndex == self.__iCurProfileIndex) and (iNewBladeRowIndex == self.__iCurBladeRowIndex):
				return
			
			#Entsprechendes Widget anzeigen
			#self.setCentralWidget(self.BladeRowWidgets[self.__iCurBladeRowIndex])
			self.__pTurboMachineWidget.getBladeRowWidget(iNewBladeRowIndex).setVisible(True)
			self.WidgetLayout.addWidget(self.__pTurboMachineWidget.getBladeRowWidget(iNewBladeRowIndex))
			#Vor den Wechsel aktuelles Widget zurückspeichern
			self.__saveChanges()

			#Ansonsten neue Werte übernehmen
			self.__iCurProfileIndex = iNewProfileIndex
			self.__iCurBladeRowIndex = iNewBladeRowIndex
			#Fertig mit dem Wechsel
			return

		#Ist es ein Profil?
		if(self.iSelectionType == QBladeDesigner.__Profile):

			#Profile2D anzeigen
			#Indexnummer des Profils ermitteln
			iNewProfileIndex = pItem.parent().indexOfChild(pItem)
			#IndexNummer der BladeRow ermitteln
			iNewBladeRowIndex = pItem.parent().parent().indexOfChild(pItem.parent())

			#Falls sich nichts geändert hat, können wir uns den wechsel sparen
			if (iNewProfileIndex == self.__iCurProfileIndex) and (iNewBladeRowIndex == self.__iCurBladeRowIndex):
				return
			
			#Entsprechendes Widget anzeigen
			self.__pTurboMachineWidget.getBladeRowWidget(iNewBladeRowIndex).getProfileWidget(iNewProfileIndex).setVisible(True)
			self.WidgetLayout.addWidget(self.__pTurboMachineWidget.getBladeRowWidget(iNewBladeRowIndex).getProfileWidget(iNewProfileIndex))

			#Vor dem Wechsel Änderungen zwischenspeichern
			self.__saveChanges()

			#Ansonsten neue Werte übernehmen
			self.__iCurProfileIndex = iNewProfileIndex
			self.__iCurBladeRowIndex = iNewBladeRowIndex





	## @brief Detaches the currently shown widget<br><em>Hängt das aktuell angezeigte Widget aus</em>
	#
	# Auxiliary function - In the past here the values of the widgets were saved, because the widgets were deleted. Meantime the way of detachment has prove of value.
	# So a created widget can be used optimal and don't need to be deleted.
	# <br><em>Hilfsfunktion - Früher wurden hier auch die Werte der Widgets gespeichert, da diese gelöscht wurden. Mittlerweile hat sich der Weg des aushängens
	# bewährt. So kann ein erstelltes Widget optimal genutzt werden und muß nichtmehr gelöscht werden.
	# </em>
	#
	# @sa itemClicked, __iCurBladeRowIndex, __iCurProfileIndex
	def __saveChanges(self):
		#Wurde die Turbomaschine angezeigt?
		if (self.__iCurBladeRowIndex == -1) and (self.__iCurProfileIndex == -1):
			#Objekt ausklinken
			self.__pTurboMachineWidget.setParent(None)
			self.__pTurboMachineWidget.setVisible(False)
			return

		#Wurde eine BladeRow angezeigt?
		if (self.__iCurProfileIndex == -1) and (self.__iCurBladeRowIndex != -1):
			#Objekt ausklinken
			self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex).setParent(None)
			self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex).setVisible(False)
			#Fertig
			return

		#Objekt ausklinken
		self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex).getProfileWidget(self.__iCurProfileIndex).setParent(None)
		self.__pTurboMachineWidget.getBladeRowWidget(self.__iCurBladeRowIndex).getProfileWidget(self.__iCurProfileIndex).setVisible(False)





	## @brief Generates a random color<br><em>Generiert eine zufällige Farbe</em>
	#
	# Auxiliary function - is used to generate color for icons of bladeRows and profiles.
	# <br><em>Hilfsfunktion - Wird benutzt um Farben für die Icons von BladeRow und Profil zu generieren.
	# </em>
	#
	# @return Random generated color of type QColor<br><em>Zufällig generierte Farbe vom Typ QColor</em>
	#
	# @sa __colorImage()
	def __generateColor(self):
		return QColor(int(random.randint(0, 4)*63.75), int(random.randint(0, 4)*63.75), int(random.randint(0, 4)*63.75), 200)




	## @brief Is called if something has changed in the contained widgets<br><em>Wird aufgerufen falls sich an den enthaltenen Widgets etwas ändert</em>
	#
	# Is in charge of making changes visible and realised. Is in charge of changing the title of BladeDesigner.
	# Is attached to contentChanged() in constructor of QTurboMachineWidget.
	# <br><em>Sorgt dafür, dass eine Änderung sichtbar gemacht und realisiert wird. Sorgt dafür, dass sich der Titel des BladeDesigners entsprechend ändert.
	# Ist im Konstruktor an contentChanged() von QTurboMachineWidget gehangen wurden.
	# </em>
	#
	# @sa __init__()
	def OnContentChanged(self):
		#Etwas hat sich verändert
		self.__bIsChanged = True
		#Namen der Datei schreiben
		strDateiname = self.__strSaveTo
		#Noch kein Name ausgewählt?
		if (strDateiname == ""):
			#Noch unbenannt
			strDateiname = self.tr("Unbenannt", "Dateistatus")

		#Status auf geändert setzen
		strNewTitle = basename(str(strDateiname)) + " " + self.tr("[Geändert]", "Dateistatus") + " - " + self.tr("BladeDesigner", "Name der Applikation")
		#Title setzen
		self.setWindowTitle(strNewTitle)


		


	## @brief Renames all labels as the corresponding language<br><em>Bennent alle Label in die entsprechende Sprache um</em>
	#
	#  Reimplemented from QBaseWidget::retranslateStrings(). Used for localisation purpose. The texts inputed in QtDesigner will be overwritten.
	#  <br><em>Diese Funktion ist reimplementiert aus QBaseWidget::retranslateStrings(). Wird zur Lokalisation verwendet. Die im QtDesigner eingetragenen Texte für
	#  Steuerelemente werden überschrieben.</em>
	#
	#  @param bPassDown - tells whether to pass down the retranslation to underlying Widgets<br><em>Gibt an, ob die Übersetzung an tiefer liegende Widgets übergeben werden soll.</em>
	def retranslateStrings(self, bPassDown = True):
		#Menüs
		self.MDatei.setTitle(self.tr("&Datei", "Titel des Dateimenü"))
		self.MBearbeiten.setTitle(self.tr("&Bearbeiten", "Titel des Bearbeitenmenü"))
		self.MAnsicht.setTitle(self.tr("&Ansicht", "Titel des Ansichtmenü"))
		self.MEinstellungen.setTitle(self.tr("&Einstellungen", "Titel des Einstellungmenü"))
		self.MHilfe.setTitle(self.tr("&Hilfe", "Titel des Hilfemenü"))
		self.MSprache.setTitle(self.tr("&Sprache", "Titel des Sprachenmenü im Menü Einstellungen"))

		#Aktionen
		self.ANeu.setText(self.tr("&Neu", "Text einer Aktion im Dateimenü"))
		self.ALaden.setText(self.tr("&Öffnen", "Text einer Aktion im Dateimenü"))
		self.ASpeichern.setText(self.tr("&Speichern", "Text einer Aktion im Dateimenü"))
		self.ASpeichernUnter.setText(self.tr("Speichern &unter ...", "Text einer Aktion im Dateimenü"))
		self.ABeenden.setText(self.tr("&Beenden", "Text einer Aktion im Dateimenü"))

		self.ACopy.setText(self.tr("&Kopieren", "Text einer Aktion im Bearbeitenmenü"))
		self.APaste.setText(self.tr("&Einfügen", "Text einer Aktion im Bearbeitenmenü"))
		self.ACut.setText(self.tr("&Ausschneiden", "Text einer Aktion im Bearbeitenmenü"))
		self.AUp.setText(self.tr("&Hoch", "Text einer Aktion im Bearbeitenmenü"))
		self.ADown.setText(self.tr("&Runter", "Text einer Aktion im Bearbeitenmenü"))

		self.AObjektHierarchie.setText(self.tr("&Objekt-Hierarchie", "Text einer Aktion im Ansichtmenü"))
		self.AToolbar.setText(self.tr("&Toolbar", "Text einer Aktion im Ansichtmenü"))

		self.ADeutsch.setText(self.tr("&Deutsch", "Text einer Aktion im Sprachenmenü"))
		self.AEnglish.setText(self.tr("&English", "Text einer Aktion im Sprachenmenü"))

		self.AAbout.setText(self.tr("&Über BladeDesigner", "Text einer Aktion im Sprachenmenü"))

		self.AAddProfile.setText(self.tr("Profile hinzufügen", "Text einer Aktion im Kontextmenü"))
		self.AAddBladeRow.setText(self.tr("Bladerow hinzufügen", "Text einer Aktion im Kontextmenü"))
		self.ARemove.setText(self.tr("Entfernen", "Text einer Aktion im Kontextmenü"))

		#DockingWindow
		self.DWHierarchie.setWindowTitle(self.tr("Objekt-Hierarchie", "Titel des Docking-Window mit der Objekthierarchie"))
		self.TWHierarchie.setHeaderLabel(self.tr("Objekte", "Titel der Spalte der Objekthierarchie im Docking-Window"))

		#Hierarchie-Einträge ändern
		self.TWHierarchie.topLevelItem(0).setText(0, self.tr("TurboMachine", "Bezeichnung der Turbomachine in der Objekthierarchie"))
		#Durch alle BladeRows gehen
		for iCurBladeRow in range(self.TWHierarchie.topLevelItem(0).childCount()):
			#BladeRow umbenennen
			self.TWHierarchie.topLevelItem(0).child(iCurBladeRow).setText(0, self.tr("Bladerow ", "Bezeichnung der BladeRow in der Objekthierarchie") + str(iCurBladeRow+1))
			#Durch alle Profile gehen
			for iCurProfile in range(self.TWHierarchie.topLevelItem(0).child(iCurBladeRow).childCount()):
				#Profil umbenennen
				self.TWHierarchie.topLevelItem(0).child(iCurBladeRow).child(iCurProfile).setText(0, self.tr("Profile ", "Bezeichner des Profiles in der Objekthierarchie") + str(iCurProfile+1))

		#Titel ändern
		strDateiname = self.__strSaveTo
		#Noch kein Name ausgewählt?
		if (strDateiname == ""):
			#Noch unbenannt
			strDateiname = self.tr("Unbenannt")

		strChanged = ""
		#Hat sich garnichts geändert?
		if (self.__bIsChanged):
			strChanged = self.tr("[Geändert]")

		#Status auf geändert setzen
		strNewTitle = basename(str(strDateiname)) + " " + strChanged + " - " + self.tr("BladeDesigner", "Name der Applikation")
		#Title setzen
		self.setWindowTitle(strNewTitle)

		#Weitergeben?
		if (bPassDown):
			#An Turbomachinenwidget weitergeben
			self.__pTurboMachineWidget.retranslateStrings()




	## @brief Is called on hiding<br><em>Wird beim Verstecken des Fensters aufgerufen</em>
	#
	#  Reimplemented from QWidget. Saves the state of visibility of the hierarchie-widget.
	#  <br><em>Diese Funktion ist reimplementiert aus QWidget. Speichert den Anzeigestatus der Hierarchiewidget.</em>
	#
	#  @param pEvent - Parameter of the event<br><em>Parameter des Events</em>
	#
	# @sa showEvent()
	def hideEvent(self, pEvent):
		self._isHierarchieVisible = self.DWHierarchie.isVisible()




	## @brief Is called on showing<br><em>Wird beim Anzeigen des Fensters aufgerufen</em>
	#
	#  Reimplemented from QWidget. Restores the state of visibility of the hierarchie-widget.
	#  <br><em>Diese Funktion ist reimplementiert aus QWidget. Stellt den Anzeigestatus des Hierarchiewidgets wieder her.</em>
	#
	#  @param pEvent - Parameter of the event<br><em>Parameter des Events</em>
	#
	# @sa hideEvent()
	def showEvent(self, pEvent):
		self.DWHierarchie.setVisible(self._isHierarchieVisible)

		
		


## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell Entfernen
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Hauptfenster erstellen
	pBladeDesigner = QBladeDesigner()
	#Anzeigen
	pBladeDesigner.show()
	#Ausführen
	sys.exit(pApp.exec_())