#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009

import sys
#Importpfad für übergeordnete Verzeichnisse setzen
sys.path.append('..')
#System-Werte zum Laden entfernen
#TODO: Entfernen, wenn alle Tests abgeschlossen
temp = sys.argv
sys.argv = []

from bladeRow import bladeRow
from turboMachine import dataStructure
from numpy import zeros
from PyQt4.QtGui import QWidget, QLineEdit, QDoubleValidator, QIntValidator, QApplication, QDialog, QVBoxLayout, QPushButton
from PyQt4.QtCore import SIGNAL,  QVariant, Qt
from Ui_BladeRowWidget import Ui_BladeRowWidget as BladeRowWidget
from QCoordinatesInputWidget import QCoordinatesInputWidget
from QStatusDialog import QStatusDialog
from QProfile2DWidget import QProfile2DWidget
from os.path import abspath
from QBaseWidget import QBaseWidget
from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB

import locale

#System-Werte wiederherstellen
#TODO: Entfernen, wenn alle Tests abgeschlossen
sys.argv = temp





## @brief Inputwidget for @link src::bladeRow::bladeRow bladeRow @endlink class<br><em>Eingabewidget für die @link src::bladeRow::bladeRow bladeRow @endlink Klasse</em>
#
# This widget holds all data necessary to do the calculation of @link src::bladeRow::bladeRow bladeRow @endlink.Thus it is able to do a stand-alone
# @link src::bladeRow::bladeRow bladeRow @endlink calculation. It can also be integrated into
# a tree strcutre to provide data for superior objects(e.g. QTurboMachineWidget). For managment and calculation purpose there is an array of QProfile2DWidget s.
# It offers an own QOGLPlotWidget for visualization.<br>
# The layout of this widget in QtDesignerform is contained in ./UI/BladeRowWidget.ui, which can be converted to UI_BladeRowWidget by ./CreateGUI.sh script using pyuic4
# and loaded afterwards.
# <br><em>Diese Widget nimmt alle für die Berechnung von @link src::bladeRow::bladeRow bladeRow @endlink nötige Daten inklusive QProfile2DWidget auf. Es ist deshalb im Stande
# selbstständige @link src::bladeRow::bladeRow bladeRow @endlink Berechnungen
# durchzuführen. Es lässt sich allerdings auch in eine Baumsturuktur integrieren. Zur Verwaltung und Berechnung besitzt es seinen eigenen Array von QProfile2DWidget s.
# Für die Visualisierung besitzt es sein eigenes QOGLPlotWidget.<br>
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/BladeRowWidget.ui, welche von ./CreateGUI.sh script mittels pyuic4 in UI_BladeRowWidget konvertiert
# und dann geladen werden kann.</em>
#
# @sa @link src::bladeRow::bladeRow bladeRow @endlink, QBaseWidget::QBaseWidget, QProfile2DWidget::QProfile2DWidget
class QBladeRowWidget(QWidget, BladeRowWidget, QBaseWidget):

	## @brief Constructor of QBladeRowWidget<br><em>Konstruktor des QBladeRowWidget</em>
	#
	# Initializes and configurates underlying widgets. Attaches to QBaseWidget provided interfaces(#_OnPerformCalc, #_OnCopyFrom, #_OnGetOutput,
	# #_OnSetCalculated, #_OnSetSaved). Also creates the list of profiles.
	# <br><em>Initialisiert und konfiguriert grundlegende Widgets. Hängt die von QBaseWidget bereitgestellten Schnittstellen ein(#_OnPerformCalc, #_OnCopyFrom, #_OnGetOutput,
	# #_OnSetCalculated, #_OnSetSaved). Erstellt die Profilliste.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		QWidget.__init__(self, pParent)
		QBaseWidget.__init__(self, pParent)
		self.setupUi(self)
		self.hookChangeNotify()

		#Namen setzen
		self.retranslateStrings(False)
		
		#Dimensionen setzen
		self.CIShroud.setDimensions(['z','r'])
		self.CIHub.setDimensions(['z','r'])
		self.CICoordinatesStackingLine.setDimensions(['z','r'])
		self.CICoordinatesStackingPoints.setDimensions(['z','r'])
		self.CICoordinatesTE.setDimensions(['z','r'])
		self.CICoordinatesLE.setDimensions(['z','r'])
		#Wertebereich für den Span setzen
		self.CISpan.setRange(0, 0.0, 1.0)
		#Sonderfälle
		self.CISpan.setAddAvailable(False)
		self.CIPeriphalOffset.setAddAvailable(False)

		#Validatoren anhängen
		for LELineEdit in self.findChildren(QLineEdit):
			LELineEdit.setValidator(QDoubleValidator(self))

		#Sonderfälle per Hand mit IntValidator versehen
		lIntValidNeeded = ('RowNumber','NumberOfBlades')
		for strLineEditName in lIntValidNeeded:
			getattr(self, 'LE' + strLineEditName).setValidator(QIntValidator(self))

		#Nur das nötige Anzeigen
		self.optionChanged()

		#Events für die Grafikanzeige
		#Tab Index so setzen, dass das Grafikfenster angezeigt wird
		self.tabWidget.setCurrentIndex(1)
		#Widget anzeigen, sonst können die Grafiktreiber nicht initialisiert werden
		self.show()
		#Driver initialisieren
		self.Plot.InitDriver()
		#Ansicht auf Isometrisch setzen
		self.Plot._display.View_Iso()
		#Passform
		self.Plot._display.FitAll()
		#Testmodus wechseln
		self.Plot._display.Test()
		#Widget wieder ausblenden
		self.hide()
		#Tab auf Ursprungstab zurücksetzen
		self.tabWidget.setCurrentIndex(0)

		#Auswahl der Ausgabe deaktivieren
		self.CBPlot.setEnabled(False)
		

		#Zeichenfenster verstecken
		#self.OGLPlot.setVisible(False)

		#Funktionen binden

		## @brief Functionpointer from QBaseWidget::_OnPerformCalc<br><em>Funktionszeiger von QBaseWidget::_OnPerformCalc</em>
		#
		#  Attached to OnPerformCalc()
		# <br><em>Verbunden mit OnPerformCalc()
		# </em>
		#
		# @sa OnPerformCalc()
		self._OnPerformCalc = self.OnPerformCalc

		## @brief Functionpointer from QBaseWidget::_OnCopyFrom<br><em>Funktionszeiger von QBaseWidget::_OnCopyFrom</em>
		#
		#  Attached to OnCopyFrom()
		# <br><em>Verbunden mit OnCopyFrom()
		# </em>
		#
		# @sa OnCopyFrom()
		self._OnCopyFrom = self.OnCopyFrom

		## @brief Functionpointer from QBaseWidget::_OnGetOutput<br><em>Funktionszeiger von QBaseWidget::_OnGetOutput</em>
		#
		# Attached to OnGetOutput()
		# <br><em>Verbunden mit OnGetOutput()
		# </em>
		#
		# @sa OnGetOutput()
		self._OnGetOutput = self.OnGetOutput

		## @brief Functionpointer from QBaseWidget::_OnSetCalculated<br><em>Funktionszeiger von QBaseWidget::_OnSetCalculated</em>
		#
		# Attached to OnSetCalculated()
		# <br><em>Verbunden mit OnSetCalculated()
		# </em>
		#
		# @sa OnSetCalculated()
		self._OnSetCalculated = self.OnSetCalculated

		## @brief Functionpointer from QBaseWidget::_OnSetSaved<br><em>Funktionszeiger von QBaseWidget::_OnSetSaved</em>
		#
		# Attached to OnSetSaved()
		# <br><em>Verbunden mit OnSetSaved()
		# </em>
		#
		# @sa OnSetSaved()
		self._OnSetSaved = self.OnSetSaved

		#Debug und ReadXML-Gruppenboxen ebenfalls verstecken

		#Interne Variablen zur Leichteren Berechnung
		#self.__resetFlags()

		#ProfileWidgets
		
		## @brief List of QProfile2DWidget s<br><em>List der QProfile2DWidget s</em>
		#
		# This list is for calculation of the @link src::bladeRow::bladeRow bladeRow @endlink and managment of QProfile2DWidget s. Also it is used to summarize the widgets to an hierarchy.
		# <br><em>Diese Liste dient  dazu die zur Berechnung benötigten QProfile2DWidget s zuverwalten. Des Weiteren wird sie dazu verwendet, dass die Widgets in einer
		# Hierarchie zusammengefasst werden.
		# </em>
		#
		# @sa QBladeDesigner
		self.__lProfileWidgets = []

		#TK-inter String zu Fließkomma-bug für Debugmodus umgehen
		locale.setlocale(locale.LC_NUMERIC, 'C')

		#Fortschrittsfunktion
		## @brief Stepfunction for calculation<br><em>Schrittfunktionszeiger für Berechnung</em>
		#
		# To this pointer a function should be attached, which is called after each step of calculation. It is used for visualization of progress. In case of
		# QBladeRowWidget, this pointer is either attached to QStatusDialog::step() on standalone calculation or assigned from superior strcutre.
		# <br><em>An diesen Zeiger sollte die Funktion gehangen werden, die aufgerufen wird, falls ein Berechnungsschritt vergangen ist. Sie dient
		# der Fortschrittsvisualisierung. Im Fall von QBladeRowWidget wird dieser Zeiger entweder auf QStatusDialog::step() bei selbstständiger Berechnung gehangen
		# oder von der übergeordneten Struktur( QTurboMachineWidget ) gesetzt
		# </em>
		#
		# @sa QTurboMachineWidget, QStatusDialog::step(), OnPerformCalc()
		self.step = lambda: None





	## @brief Marks the QBladeRowWidget as calculated<br><em>Markiert das QBladeRowWidget als berechnet</em>
	#
	# This function is called by QBaseWidget::setCalculated(). It takes care of marking the QProfile2DWidget s in the list of profiles also as calculated.
	# <br><em>Diese Funktion wird durch QBaseWidget::setCalculated() aufgerufen. Sie sorgt dafür, dass auch die QProfile2DWidget s in der Profilliste als berechnet markiert werden.
	# </em>
	#
	# @sa QBaseWidget::setCalculated()
	def OnSetCalculated(self):
		#An Profile weiterreichen
		for pProfile in self.__lProfileWidgets:
			pProfile.setCalculated()





	## @brief Marks the QBladeRowWidget as saved<br><em>Makriert die QBladeRowWidget als gespeichert</em>
	#
	# This function is called by QBaseWidget::setSaved(). It takes care of marking the QProfile2DWidget in the list of profiles also as saved.
	# <br><em>Diese Funktion wird durch QBaseWidget::setSaved() aufgerufen. Sie sorgt dafür, dass auch die QProfile2DWidget in der Profilliste als berechnet markiert werden.
	# </em>
	#
	# @sa QBaseWidget::setSaved()
	def OnSetSaved(self):
		#An Profile weiterreichen
		for pProfile in self.__lProfileWidgets:
			pProfile.setSaved()





	## @brief Sets the data of the contained widgets according to a @link src::bladeRow::bladeRow bladeRow @endlink object<br><em>Setzt die Daten der Steuerelemente anhand eines @link src::bladeRow::bladeRow bladeRow @endlink Objektes</em>
	#
	# Overloaded from QBaseWidget::setFromObject(). Loads the definitions from @link src::bladeRow::bladeRow bladeRow @endlink object, adjusts the widgets layout using optionChanged() and fills all needed data
	# from @link src::bladeRow::bladeRow bladeRow @endlink object into the QBladeRowWidget.
	# <br><em>Überladen von QBaseWidget::setFromObject(). Es läd die Definitionen aus den @link src::bladeRow::bladeRow bladeRow @endlink Objekt, passt dann mittels optionChanged() das Layout an und füllt
	# die betreffenden Daten aus dem @link src::bladeRow::bladeRow bladeRow @endlink Objekt in das QBladeRowWidget
	# </em>
	#
	# @param pBladeRow - @link src::bladeRow::bladeRow BladeRow @endlink object from which the data is to read<br><em>@link src::bladeRow::bladeRow BladeRow @endlink Objekt aus dem die Daten gelesen werden sollen</em>
	#
	# @sa QBaseWidget::setFromObject()
	def setFromObject(self, pBladeRow):
		#Prüfen ob die BladeRow den richtigen Typ hat
		if (pBladeRow.__class__ != bladeRow):
			return

		#Grundlegen Flags setzen
		self.CBDefinition.setCurrentIndex(pBladeRow.definition)
		self.CBSpanDistr.setCurrentIndex(pBladeRow.spanDistr)
		self.CBAngleFlag.setCurrentIndex(pBladeRow.angleFlag)
		#self.CBOffsetFlag.setCurrentIndex(pBladeRow.offsetFlag)
		#StandardLineEdits füllen
		self.LENumberOfBlades.setText(str(int(pBladeRow.numberBlades)))
		self.LEOmega.setText(str(pBladeRow.omega))

		#Ansicht aktualisieren
		self.optionChanged()

		#Gesetze Werte eintragen
		#StackingLine
		if (len(pBladeRow.coordinatesStackingLine) != 0):
			self.CICoordinatesStackingLine.setCoordinatesFromNumpy(pBladeRow.coordinatesStackingLine)

		#StackingPoints
		if (len(pBladeRow.coordinatesStackingPoints) != 0):
			self.CICoordinatesStackingPoints.setCoordinatesFromNumpy(pBladeRow.coordinatesStackingPoints)

		#Leading Edge
		if (len(pBladeRow.coordinatesLE) != 0):
			self.CICoordinatesLE.setCoordinatesFromNumpy(pBladeRow.coordinatesLE)

		#Trailing Edge
		if (len(pBladeRow.coordinatesTE) != 0):
			self.CICoordinatesTE.setCoordinatesFromNumpy(pBladeRow.coordinatesTE)

		#Hub
		if (len(pBladeRow.coordinatesHub) != 0):
			self.CIHub.setCoordinatesFromNumpy(pBladeRow.coordinatesHub)

		#Shroud
		if (len(pBladeRow.coordinatesShroud) != 0):
			self.CIShroud.setCoordinatesFromNumpy(pBladeRow.coordinatesShroud)

		#Daten der Kaskade laden
		self.LEBladeDist.setText(str(pBladeRow.bladeDist))
		self.LEBladeSpan.setText(str(pBladeRow.bladeSpan))
		self.LEDomainLength.setText(str(pBladeRow.domainLength))
		self.LEStackingLine.setText(str(pBladeRow.stackingLine))

		#Span vorhanden?
		if (len(pBladeRow.span) != 0):
			#Span in einen Numpy-Array packen
			lTemp = zeros([len(pBladeRow.span), 1])
			#Werte übertragen
			for iCurEntry in range(len(pBladeRow.span)):
				lTemp[iCurEntry][0] = pBladeRow.span[iCurEntry]

			#Werte übertragen
			self.CISpan.setCoordinatesFromNumpy(lTemp)

		#Profile setzen
		for iCurProfile in range(self.anzProfileWidgets()):
			#Profile setzen
			self.getProfileWidget(iCurProfile).setFromObject(pBladeRow.listProfiles[iCurProfile])
			

		
		

	## @brief Resets all displayflags<br><em>Setzt alle Anzeigeflags zurück</em>
	#
	# This is a auxiliary function, which helps optionChanged() to determine the current display condition of the contained widgets. For internal use only.
	# <br><em>Dies ist eine Hilfsfunktion, die optionChanged() dabei hilft die derzeitigen Anzeigezustände der Steuerelemente zu ermitteln. Nur interne Verwendung.</em>
	#
	# @sa optionChanged()
	def __resetFlags(self):
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowLeadTrailingEdge = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowCascade = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowStackingLine = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowSpanDistr = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowPeriphalOffset = False





	## @brief Performs the calculation of @link src::bladeRow::bladeRow bladeRow @endlink object<br><em>Führt die Berechnung für das @link src::bladeRow::bladeRow bladeRow @endlink Objekt aus</em>
	#
	# This function is attached to QBaseWidget::_OnPerformCalc to use the provided automatisms.
	# It is in charge of single calculation and preview. At first a status dialog is created showing the progress of the calculation.
	# Afterwards the calculation is passed to the @link src::bladeRow::bladeRow bladeRow @endlink object. The results of the calculation can be used to display the @link src::bladeRow::bladeRow bladeRow @endlink in QOGLPlotWidget.
	# Therefore visualization data is gathered previously. This function is attached to Calculate-Button in QtDesigner.
	# <br><em>Diese Funktion wird an QBaseWidget::_OnPerformCalc gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie ist für eine Einzelberechnung/Vorschau zuständig. Zu erst wird ein Statusdialog erstellt, der den Fortschritt der Berechnung anzeigt.
	#  Im Anschluß wird die Berechnung an das @link src::bladeRow::bladeRow bladeRow @endlink Objekt übergeben. Die Ergebnisse der Berechnung können dann zur Darstellung der @link src::bladeRow::bladeRow bladeRow @endlink
	#  im QOGLPlotWidget benutzt werden. Dazu werden im Vorfeld Darstellungsdaten gesammelt.Diese Funktion ist im QtDesigner mit dem Berechnen-Button verbunden.</em>
	#
	# @sa QBaseWidget::_OnPerformCalc, _pOutput, QStatusDialog, QOGLPlotWidget
	def OnPerformCalc(self):
		#Statusdialog anzeigen
		pStatusDialog = QStatusDialog(self)
		#Dialog Modal anzeigen
		pStatusDialog.setModal(True)
		#Dialog anzeigen
		pStatusDialog.show()
		#br = QBladeRowWidget(self)
		#Für jedes Profil mit 3 Schritten +1 Schritt für die BladeRow an sich rechnen
		#Schritte setzen
		self._pOutput.step = pStatusDialog.step
		#Anzahl Schritte setzen
		pStatusDialog.setProgressSteps(self.anzProfileWidgets()*3+1)

		#Arbeitsverzeichnis setzen
		self._pOutput.dataStruct = dataStructure(abspath('temp'))

		#Ausgabevektor anpassen -> BladeRow in die Mitte verschieben
		vPosition = [0.0, 0.0, 0.0]
		#StackingLine
		if (self._pOutput.definition == 0):
			#Ersten Wert holen
			fStart = self._pOutput.coordinatesStackingLine[0][1]
			fPosX = (self._pOutput.coordinatesStackingLine[len(self._pOutput.coordinatesStackingLine)-1][1] - fStart)*0.5 + fStart
			#Angepasste X-Position und Z-Position
			vPosition = [-fPosX, 0.0, -self._pOutput.coordinatesStackingLine[len(self._pOutput.coordinatesStackingLine)/2][0]]
		#Kaskade
		if (self._pOutput.definition == 2):
			#BladeSpan = Breite, sprich X-Korrektur
			#StackingLine = Fädellinie, sprich Mittelpunkt Z-Achse
			vPosition = [-(self._pOutput.bladeSpan/2.0), 0.0, -self._pOutput.stackingLine]

		try:
			QApplication.processEvents()
			#Berechnung durchführen
			self._pOutput()
			#Daten in die Ausgabe füttern
			#self.OGLPlot.setNurbsSurface(self._pOutput.bladeSurface, vPosition, [self.color().redF(), self.color().greenF(), self.color().blueF()])
			#Material = Graphic3d_MaterialAspect()
			#Material.SetColor(Quantity_Color(0, 1, 0.5, 0))
			self.plot(0)
			#Auswahl der Ausgabe aktivieren
			self.CBPlot.setEnabled(True)
		except Exception, eException:
			#Fehler ausgeben
			print "Fehler", str(eException)
			del self._pOutput
			self._pOutput = None
		
		#Dialog freigeben
		pStatusDialog.outputFinished()





	## @brief Plots the result of the calculation of @link src::bladeRow::bladeRow bladeRow @endlink object<br><em>Zeichnet das Ergebnis der Berechnung des @link src::bladeRow::bladeRow bladeRow @endlink Objekts</em>
	#
	# Hides all contained widgets and shows the drawing window.It is attached to Plot-Button in QtDesigner.
	# <br><em>Verbirgt alle Steuerelemente und zeigt das Zeichenfenster an. Diese Funktion ist im QtDesigner mit dem Zeichnen-Button verbunden.
	# </em>
	#
	# @sa OnPerformCalc(), _pOutput, QOGLPlotWidget
	def plot(self, iPlotType):
		#Gibt es einen Output? Dann gibt es sicher auch ein Ergebnis
		if(self._pOutput == None):
			return

		#Basismaterial erstellen
		Material = Graphic3d_MaterialAspect(Graphic3d_NOM_SILVER)
		#Diffusestärke setzen
		Material.SetDiffuse(0.8)

		#Farben erstellen
		Specular = Quantity_Color(0.4, 0.4, 0.4, Quantity_TOC_RGB)
		Ambiente = Quantity_Color(0.1, 0.1, 0.1, Quantity_TOC_RGB)
		#Farbe erstellen
		Diffuse = Quantity_Color(self._pColor.redF(), self._pColor.greenF(), self._pColor.blueF(), Quantity_TOC_RGB)
		#Farbe setzen
		Material.SetDiffuseColor(Diffuse)


		#Auswahl prüfen
		if(iPlotType == 0):
			#BladeSurface
			self._pOutput.drawBladeSurface(self.Plot._display, Material)
		if(iPlotType == 1):
			Colors = []
			#Materialien sammeln
			for Profile in self.__lProfileWidgets:
				#Profilfarbe holen
				Color = Profile.color()
				#Diffusefarbe erstellen
				Diffuse = Quantity_Color(Color.redF(), Color.greenF(), Color.blueF(), Quantity_TOC_RGB)
				#Zur Liste hinzufügen
				Colors.append(Diffuse)
				
			#NurbsCurves
			self._pOutput.drawNurbsCurves(self.Plot._display, Colors)
		if(iPlotType == 2):
			#BladeRow
			self._pOutput.drawRow(self.Plot._display, True, Material)
		
		
		





	## @brief Updates the layout of the widget according to the choosen Settings<br><em>Aktualisiert das Layout des Widgets abhängig von den gewählten Einstellungen</em>
	#
	# At first this function resets the internal display flags and determines the state of all contained widgets on the basis of striking widgets.
	# This function is attached to always visible and striking contained widgets. It is important for the visibility of contained wigets and the appearance of
	# @link src::bladeRow::bladeRow bladeRow @endlink object.
	# <br><em>Die Funktion setzt zuerst die internen Anzeigeflags zurück und ermittelt dann anhand markanter Steuerelemente den Zustand
	# aller anderen Steuerelemente. Diese Funktion ist im QtDesigner an markante immer sichtbare Steuerelemente gebunden.
	# Sie ist entscheidend für die Sichtbarkeit der Steuerelemente und das Erscheinungsbild der @link src::bladeRow::bladeRow bladeRow @endlink.</em>
	#
	# @sa QBaseWidget::optionChanged(), __resetFlags()
	def optionChanged(self):
		#Werte zurücksetzen
		self.__resetFlags()

		#DefinitionsIndex holen
		iDefIndex = self.CBDefinition.currentIndex()

		#StackingLine anzeigen?
		if (iDefIndex == 0):
			self.__bShowStackingLine = True

		#Leading/TrailingEgde anzeigen
		if (iDefIndex == 1):
			self.__bShowLeadTrailingEdge = True

		#Cascade anzeigen?
		if (iDefIndex == 2):
			self.__bShowCascade = True

		#Spanmodus prüfen
		if (self.CBSpanDistr.currentIndex() == 1):
			self.__bShowSpanDistr = True

		#Pheripheren Abstand anzeigen?
		if (iDefIndex <= 1):
			self.__bShowPeriphalOffset = True

		#Optionen weitergeben
		#Gruppenboxen
		self.CICoordinatesStackingLine.setVisible(self.__bShowStackingLine)
		self.CICoordinatesStackingPoints.setVisible(self.__bShowStackingLine)

		self.CICoordinatesLE.setVisible(self.__bShowLeadTrailingEdge)
		self.CICoordinatesTE.setVisible(self.__bShowLeadTrailingEdge)

		self.GBCascade.setVisible(self.__bShowCascade)

		self.CIShroud.setVisible(not self.__bShowCascade)
		self.CIHub.setVisible(not self.__bShowCascade)


		#Labels, Edits & Comboboxen
		#self.LOffsetFlag.setVisible(self.__bShowLeadTrailingEdge)
		#self.CBOffsetFlag.setVisible(self.__bShowLeadTrailingEdge)
		
		self.LAngleFlag.setVisible(not self.__bShowCascade)
		self.CBAngleFlag.setVisible(not self.__bShowCascade)

		self.LSpanDistr.setVisible(not self.__bShowCascade)
		self.CBSpanDistr.setVisible(not self.__bShowCascade)

		self.CISpan.setVisible(not self.__bShowCascade and self.__bShowSpanDistr)
		self.CIPeriphalOffset.setVisible(self.__bShowPeriphalOffset)

		self.LOmega.setVisible(not self.__bShowCascade)
		self.LEOmega.setVisible(not self.__bShowCascade)





	## @brief Collects all data from the widget and fills it into @link src::bladeRow::bladeRow bladeRow @endlink<br><em>Sammelt alle Daten der Steuerelemente in die @link src::bladeRow::bladeRow bladeRow @endlink</em>
	#
	#  This function is attached to QBaseWidget::_OnGetOutput to use the provided automatisms.
	#  It determines via internal display parameters(#__bShowStackingLine, #__bShowLeadTrailingEdge, ...) which data is inputed/activated and writes it to
	#  the (given) @link src::bladeRow::bladeRow bladeRow @endlink object.
	#  Reads the needed @link src::profile2D::profile2D profile2D @endlink s as well.
	#  <br><em>Diese Funktion wird an QBaseWidget::_OnGetOutput gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie ermittelt mit Hilfe der internen Anzeigeparamter(#__bShowStackingLine, #__bShowLeadTrailingEdge, ...) welche Daten eingegeben/aktiv sind und
	#  schreibt diese dann in das (übergebene) @link src::bladeRow::bladeRow bladeRow @endlink Objekt.
	#  Liest auch die benötigten @link src::profile2D::profile2D profile2D @endlink s ein.</em>
	#
	#  @param pBladeRow - @link src::bladeRow::bladeRow BladeRow @endlink object in which the data is to write(optional)<br><em>@link src::bladeRow::bladeRow BladeRow @endlink object in das die Daten geschrieben werden sollen (optional)</em>
	#
	#  @return @link src::bladeRow::bladeRow BladeRow @endlink object in which the data was written<br><em>@link src::bladeRow::bladeRow BladeRow @endlink Objekt in das die Daten gefüllt wurden</em>
	#
	# @sa QBaseWidget._OnGetOutput, optionChanged()
	def OnGetOutput(self, pBladeRow = None):
		#Hat sich nichts verändert, aber der Output ist schon vorhanden?
		if ((not self._bIsChangedCalc) and (self._pOutput != None)):
			#Dann brauch hier nicht groß rumgerechnet werden
			return self._pOutput
	
		#Zu füllende BladeRow übergeben?
		if (pBladeRow != None):
			#pBladeRow Object übernehmen

			#Fortschrittsfunktion
			## @brief outputobject of the calculation<br><em>Ausgabeobjekt der Berechnung</em>
			#
			# Overwritten from QBaseWidget::_pOutput. In this widget #_pOutput is of @link src::bladeRow::bladeRow bladeRow @endlink type.
			# It contains all @link src::profile2D::profile2D profile2D @endlink s neccessary.
			# <br><em>Überschrieben von QBaseWidget::_pOutput. In diesem Widget ist #_pOutput vom Typ @link src::bladeRow::bladeRow bladeRow @endlink.
			# Es enthält alle für die Berechnung benötigten @link src::profile2D::profile2D profile2D @endlink s.
			# </em>
			#
			# @sa QBaseWidget::_pOutput, OnGetOutput(), OnPerformCalc(), @link src::profile2D::profile2D profile2D @endlink
			self._pOutput = pBladeRow
		else:
			#Neues BladeRowObject anlegen
			self._pOutput = bladeRow()

		#Fortschrittsfunktion setzen
		self._pOutput.step = self.step
		self._pOutput.isCalculated = False

		#Allgemeine Werte übernehmen
		#Definition
		self._pOutput.definition = self.CBDefinition.currentIndex()
		#rowNumber
		self._pOutput.rowNumber = int(self.LERowNumber.text())
		#NumberOfBlades
		self._pOutput.numberBlades = int(self.LENumberOfBlades.text())
		#Omega
		self._pOutput.omega = float(self.LEOmega.text())
		#Debug-Optionen
		self._pOutput.debug = self.CBDebug.currentIndex()


		#Spezifisches
		#Stacking Line oder Leading-TrailingEdge gewählt?
		if (self.__bShowStackingLine or self.__bShowLeadTrailingEdge):
			#Hub und Shroud auslesen
			self._pOutput.coordinatesHub = self.CIHub.coordinatesAsNumpy()
			self._pOutput.coordinatesShroud = self.CIShroud.coordinatesAsNumpy()

			#Winkelflag auslesen
			self._pOutput.angleFlag = self.CBAngleFlag.currentIndex()

			#Benutzerdefinierte Spanverteilung?
			if self.__bShowSpanDistr:
				#Ja, benutzerdefiniert
				self._pOutput.spanDistr = 1
				#Temp-Struktur anlegen
				lTemp = []
				#Werte umfüllen
				for iCurEntry in self.CISpan.coordinatesAsNumpy():
					lTemp.extend(iCurEntry)
				#Daten holen
				self._pOutput.span = lTemp
			else:
				#Nein, nicht benutzerdefiniert
				self._pOutput.spanDistr = 0

			#Stacking Line spezifische Sachen:
			if self.__bShowStackingLine:
				#Stacking Line auslesen
				self._pOutput.coordinatesStackingLine = self.CICoordinatesStackingLine.coordinatesAsNumpy()
				#Nur schreiben, falls auch wirklich Werte drin stehen
				if (self.CICoordinatesStackingPoints.TWCoordinates.rowCount() != 0):
					#Stacking Points auslesen
					self._pOutput.coordinatesStackingPoints = self.CICoordinatesStackingPoints.coordinatesAsNumpy()


			#Leading-TrailingEdge spezifische Sachen:
			if self.__bShowLeadTrailingEdge:
				#Leading Edge auslesen
				self._pOutput.coordinatesLE = self.CICoordinatesLE.coordinatesAsNumpy()
				#Trailing Edge auslesen
				self._pOutput.coordinatesTE = self.CICoordinatesTE.coordinatesAsNumpy()
				#Vorder- oder Hinterkante?
				#self._pOutput.offsetFlag = self.CBOffsetFlag.currentIndex()

			
			#Peripahler Offset auslesen
			if self.__bShowPeriphalOffset:
				self._pOutput.periphalOffset = self.CIPeriphalOffset.coordinatesAsNumpy()
				

		#Cascade anzeigen?
		if self.__bShowCascade:
			#BladeDistanz auslesen
			self._pOutput.bladeDist = float(self.LEBladeDist.text())
			#Domain Length auslesen
			self._pOutput.domainLength = float(self.LEDomainLength.text())
			#Stacking Line auslesen
			self._pOutput.stackingLine = float(self.LEStackingLine.text())
			#BladeSpan auslesen
			self._pOutput.bladeSpan = float(self.LEBladeSpan.text())

		#Profile in die Bladerow füttern
		for iWidget in range(len(self.__lProfileWidgets)):
			#Falls ein Vorgebenes Objekt?
			if (pBladeRow != None):

				#Dann sind Profile sicher auch vorgegeben
				#Hat sich das Profil geändert?
				if (self.__lProfileWidgets[iWidget].isChangedCalc()):
					#Profil hinzufügen
					#TODO: ggf Prüfen, ob es Probleme gibt, bei bereits vorhandenen Profilen
					self._pOutput.listProfiles[iWidget] = self.__lProfileWidgets[iWidget].getOutput(self._pOutput.listProfiles[iWidget])
			else:
				#Hat sich das Profil geändert?
				if (self.__lProfileWidgets[iWidget].isChangedCalc()):
					#Profil hinzufügen
					self._pOutput.listProfiles.append(self.__lProfileWidgets[iWidget].getOutput())

		#Nummer an Profilen setzen
		self._pOutput.numberProfiles = len(self.__lProfileWidgets)




	## @brief Copies additional values from a QBladeRowWidget to this one<br><em>Kopiert zusätzliche Werte von einem QBladeRowWidget in dieses</em>
	#
	# Is attached to QBaseWidget::_OnCopyFrom. This function is in charge of copying the corresponding QProfile2DWidget s and the list of profiles.
	# <br><em>Wird an QBaseWidget::_OnCopyFrom gehangen. Diese Funktion kümmert sich darum, dass die Profilliste und die entsprechenden QProfile2DWidget s mitkopiert werden.
	# </em>
	#
	# @param pBladeRow - QBladeRowWidget from which data is copied<br><em>QBladeRowWidget von dem die Daten kopiert werden sollen</em>
	#
	# @sa QBaseWidget::_OnCopyFrom
	def OnCopyFrom(self, pBladeRow):
		#Alle pProfile übernehmen
		for pProfile in pBladeRow.__lProfileWidgets:
			#Neues Profil erstellen
			pNewProfile = QProfile2DWidget()
			#Kopieren
			pNewProfile.copyFrom(pProfile)
			#Profil übernehmen
			self.__lProfileWidgets.append(pNewProfile)

		print self.__lProfileWidgets
		




	## @brief Returns the number of QProfile2DWidget s contained in the list of profiles<br><em>Gibt die Anzahl der QProfile2DWidget s enthalten in der Profilliste wieder</em>
	#
	# Auxiliary function for superior classes and visualization.
	# <br><em>Hilfsfunktion für übergeordnete Klassen und Darstellung.
	# </em>
	#
	# @return Number of contained QProfile2DWidget s<br><em>Anzahl der enthaltenen QProfile2DWidget s</em>
	#
	# @sa __lProfileWidgets, QTurboMachineWidget::QTurboMachineWidget, QTurboMachineWidget::QTurboMachineWidget::OnPerformCalc(), QBladeDesigner
	def anzProfileWidgets(self):
		#Anzahl an Einträgen zurückliefern
		return len(self.__lProfileWidgets)





	## @brief Adds a new QProfile2DWidget to the list of profiles<br><em>Fügt ein neues QProfile2DWidget in die Profilliste ein</em>
	#
	# Auxiliary function for superior classes. Updates the number of span entrys as well.
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Aktualisiert gleichzeitig die Anzahl der Einträge für Span.
	# </em>
	#
	# @param iIndex - Index of list, where the new QProfile2DWidget is added<br><em>Stelle in der Liste an der das neue QProfile2DWidget eingefügt werden soll</em>
	# @param pProfileWidget - The QProfile2DWidget to add<br><em>Das einzufügende QProfile2DWidget</em>
	#
	# @sa __lProfileWidgets, QBladeDesigner::QBladeDesigner::addProfile(), QBladeDesigner::QBladeDesigner
	def insertProfileWidget(self, iIndex, pProfileWidget):
		#Nachrichtensignale übernehmen
		self.connect(pProfileWidget, SIGNAL("contentChanged()"), self.contentChanged)
		#Neues Profil anhängen
		self.__lProfileWidgets.insert(iIndex, pProfileWidget)
		#Neues Profil in PheriphalOffset und Span einfügen
		#Neue Zeile dafür einfügen
		self.CIPeriphalOffset.TWCoordinates.model().insertRow(iIndex)
		#iIndex der neuen Zeile holen
		pEntry = self.CIPeriphalOffset.TWCoordinates.model().index(iIndex, 0)
		#Startwert setzen
		self.CIPeriphalOffset.TWCoordinates.model().setData(pEntry, QVariant(self.tr("0.0", "Startwert für neue Perihpahl Offset Einträge")), Qt.EditRole)

		#Das selbe für Span
		#Neue Zeile dafür einfügen
		self.CISpan.TWCoordinates.model().insertRow(iIndex)
		#iIndex der neuen Zeile holen
		pEntry = self.CISpan.TWCoordinates.model().index(iIndex, 0)
		#Startwert setzen
		self.CISpan.TWCoordinates.model().setData(pEntry, QVariant(self.tr("0.0", "Startwert für neue Span Einträge")), Qt.EditRole)
		




	## @brief Removes a QProfile2DWidget from the list of profiles<br><em>Entfernt ein QProfile2DWidget aus der Profilliste</em>
	#
	# Auxiliary function for superior classes. Updates the number of span entrys as well
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Aktualisiert gleichzeitig die Anzahl der Einträge für Span.
	# </em>
	#
	# @param iIndex - Index of the QProfile2DWidget to remove<br><em>Index des zu entfernenden QProfile2DWidget s</em>
	#
	# @sa __lProfileWidgets, QBladeDesigner::QBladeDesigner::delProfileWidget(),
	def delProfileWidget(self, iIndex):
		#iIndex vorhanden?
		if (iIndex < 0) or (iIndex >= self.anzProfileWidgets()):
			return

		#Signal kappen
		self.disconnect(self.__lProfileWidgets[iIndex], SIGNAL("contentChanged()"), self.contentChanged)
		
		#Entsprechendes Widget löschen
		del self.__lProfileWidgets[iIndex]

		#Anzahl an Spalten in PheriphalOffset und Span anpassen
		#Eintrag holen
		pEntry = self.CIPeriphalOffset.TWCoordinates.model().index(iIndex, 0)
		#Eintrag löschen
		self.CIPeriphalOffset.TWCoordinates.model().removeRow(iIndex, pEntry.parent())
		#Das selbe für Span
		pEntry = self.CISpan.TWCoordinates.model().index(iIndex, 0)
		self.CISpan.TWCoordinates.model().removeRow(iIndex, pEntry.parent())





	## @brief Gets a certain QProfile2DWidget from the list of profiles<br><em>Holt ein bestimmtes QProfile2DWidget aus der Profilliste</em>
	#
	# Auxiliary function for superior classes. The QProfile2DWidget is not removed from list.
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Das QProfile2DWidget wird nicht aus der Liste gelöscht.
	# </em>
	#
	# @param iIndex - Index of the QProfile2DWidget to return<br><em>Index des QProfile2DWidget, welches zurückgegeben werden soll</em>
	#
	# @sa __lProfileWidgets, QBladeDesigner::QBladeDesigner::copy(), QBladeDesigner::QBladeDesigner::addBladeRow(), QBladeDesigner::QBladeDesigner::itemClicked(), setFromObject()
	def getProfileWidget(self, iIndex):
		#iIndex vorhanden?
		if (iIndex < 0) or (iIndex >= self.anzProfileWidgets()):
			return None
		
		#Profil zurückgeben
		return self.__lProfileWidgets[iIndex]





	## @brief Overwrites a QProfile2DWidget with an other<br><em>Überschreibt ein QProfile2DWidget mit einem Anderen</em>
	#
	# Auxiliary function for superior classes.
	# <br><em>Hilfsfunktion für übergeordnete Klassen.
	# </em>
	#
	# @param iIndex - Index of the QProfile2DWidget to overwrite<br><em>Index des QProfile2DWidget s in der Liste das überschrieben werden soll</em>
	# @param pProfileWidget - the new QProfile2DWidget<br><em>Das neue QProfile2DWidget</em>
	#
	# @sa __lProfileWidgets
	def setProfileWidget(self, iIndex, pProfileWidget):
		#iIndex vorhanden?
		if (iIndex < 0) or (iIndex >= self.anzProfileWidgets()):
			return None
		
		#Profil schreiben
		self.__lProfileWidgets[iIndex] = pProfile





	## @brief Swaps 2 QProfile2DWidget s in the list of profiles<br><em>Vertauscht 2 QProfile2DWidget s in der Profilliste</em>
	#
	# Auxiliary function for superior classes.
	# <br><em>Hilfsfunktion für übergeordnete Klassen.
	# </em>
	#
	# @param iIndexProfile1 - Index of the first QProfile2DWidget to swap<br><em>Index des ersten QProfile2DWidget s das getauscht werden soll</em>
	# @param iIndexProfile2 - Index of the second QProfile2DWidget to swap<br><em>Index des zweiten QProfile2DWidget s das getauscht werden soll</em>
	#
	# @sa __lProfileWidgets, QBladeDesigner::QBladeDesigner::up(), QBladeDesigner::QBladeDesigner::down()
	def swapProfileWidgets(self, iIndexProfile1, iIndexProfile2):
		#Gleiche Indices?
		if (iIndexProfile2 == iIndexProfile1):
			#Wechsel nicht nötig
			return
		
		#Klassischer Wechsel
		#Profile1 zwischenspeichern
		pTemp = self.__lProfileWidgets[iIndexProfile1]
		#Profile2 nach Profile1
		self.__lProfileWidgets[iIndexProfile1] = self.__lProfileWidgets[iIndexProfile2]
		#Profile2 mit durch Profile2 ersetzen
		self.__lProfileWidgets[iIndexProfile2] = pTemp

		#Span und PeriphalOffset mitwechseln
		#Entsprechende Einträge selektieren
		self.CIPeriphalOffset.TWCoordinates.setCurrentItem(self.CIPeriphalOffset.TWCoordinates.item(iIndexProfile1, 0))
		self.CISpan.TWCoordinates.setCurrentItem(self.CISpan.TWCoordinates.item(iIndexProfile1, 0))

		#Nach oben oder nach unten verschieben?
		if (iIndexProfile1 < iIndexProfile2):
			self.CIPeriphalOffset.down()
			self.CISpan.down()
		else:
			#Nach oben
			self.CIPeriphalOffset.up()
			self.CISpan.up()





	## @brief Renames all labels as the corresponding language<br><em>Bennent alle Label in die entsprechende Sprache um</em>
	#
	#  Reimplemented from QBaseWidget::retranslateStrings(). Used for localisation purpose. The texts inputed in QtDesigner will be overwritten.
	#  <br><em>Diese Funktion ist reimplementiert aus QBaseWidget::retranslateStrings(). Wird zur Lokalisation verwendet. Die im QtDesigner eingetragenen Texte für
	#  Steuerelemente werden überschrieben.</em>
	#
	#  @param bPassDown - tells whether to pass down the retranslation to underlying Widgets<br><em>Gibt an, ob die Übersetzung an tiefer liegende Widgets übergeben werden soll.</em>
	def retranslateStrings(self, bPassDown = True):
		#Gruppenboxen
		self.GBGeneral.setTitle(self.tr("General", "Titel der Generalgruppenbox"))
		self.GBFlags.setTitle(self.tr("Flags", "Titel der Flagsgruppenbox"))
		self.GBCascade.setTitle(self.tr("Cascade", "Titel der Cascadengruppenbox"))

		#Labels
		self.LRowNumber.setText(self.tr("Row &Number", "Text eines Labels in Generalgruppenbox"))
		self.LNumberOfBlades.setText(self.tr("&Number Of Blades", "Text eines Labels in Generalgruppenbox"))
		self.LOmega.setText(self.tr("&Omega", "Text eines Labels in Generalgruppenbox"))
		
		self.LDefinition.setText(self.tr("D&efinition", "Text eines Labels in Flagsgruppenbox"))
		self.LDebug.setText(self.tr("D&ebug", "Text eines Labels in Flagsgruppenbox"))
		self.LSpanDistr.setText(self.tr("&Span Distr", "Text eines Labels in Flagsgruppenbox"))
		self.LAngleFlag.setText(self.tr("Angle &Flag", "Text eines Labels in Flagsgruppenbox"))
		#self.LOffsetFlag.setText(self.tr("Offset &Flag", "Text eines Labels in Flagsgruppenbox"))

		self.LBladeDist.setText(self.tr("&Blade Dist", "Text eines Labels in Cascadegruppenbox"))
		self.LDomainLength.setText(self.tr("&Domain Length", "Text eines Labels in Cascadegruppenbox"))
		self.LStackingLine.setText(self.tr("&Stacking Line", "Text eines Labels in Cascadegruppenbox"))
		self.LBladeSpan.setText(self.tr("&Blade Span", "Text eines Labels in Cascadegruppenbox"))

		#ComboBoxen
		#CBDefinition
		lLabels = (
			self.tr("stacking axis, stacking point, chord length", "Text des 1. Eintrags in CBDefinition"),
			self.tr("leading edge, trailing edge, circumferential offset", "Text des 2. Eintrags in CBDefinition"),
			self.tr("caskade of blades", "Text des 3. Eintrags in CBDefinition")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBDefinition.count()):
			#Namen übernehmen
			self.CBDefinition.setItemText(iCurrent, lLabels[iCurrent])

		#CBDebug
		lLabels = (
			self.tr("de...what?",  "Text des 1. Eintrags in CBDebug"),
			self.tr("gimme some",  "Text des 2. Eintrags in CBDebug"),
			self.tr("spam me!",  "Text des 3. Eintrags in CBDebug")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBDebug.count()):
			#Namen übernehmen
			self.CBDebug.setItemText(iCurrent, lLabels[iCurrent])

		#CBSpanDistr
		lLabels = (
			self.tr("Isosurface", "Text des 1. Eintrags in CBSpanDistr"),
			self.tr("Userdefined", "Text des 2. Eintrags in CBSpanDistr")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBSpanDistr.count()):
			#Namen übernehmen
			self.CBSpanDistr.setItemText(iCurrent, lLabels[iCurrent])

		#CBAngleFlag
		lLabels = (
			self.tr("keine winkelkonforme Abbildung", "Text des 1. Eintrags in CBAngleFlag"),
			self.tr("winkelkonforme Abbildung", "Text des 2. Eintrags in CBAngleFlag"),
			self.tr("2D-Dickenverteilung auf 3D-Skelettlinie", "Text des 3. Eintrags in CBAngleFlag")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBAngleFlag.count()):
			#Namen übernehmen
			self.CBAngleFlag.setItemText(iCurrent, lLabels[iCurrent])

		#CBOffsetFlag
		#lLabels = (
			#self.tr("Vorderkante", "Text des 1. Eintrags in CBOffsetFlag"),
			#self.tr("Hinterkante", "Text des 2. Eintrags in CBOffsetFlag")
		#)

		##Durch die ComboBox gehen und ändern
		#for iCurrent in range(self.CBOffsetFlag.count()):
			##Namen übernehmen
			#self.CBOffsetFlag.setItemText(iCurrent, lLabels[iCurrent])

		#Buttons
		self.BCalculate.setText(self.tr("&Calculate", "Aufschrift des Berechnen-Buttons"))
		#self.BPlot.setText(self.tr("&Plot", "Aufschrift des Plotten-Buttons"))

		#CoordinateInputWidgets
		self.CIShroud.setText(self.tr('Shroud', "Titel des CoordinatenInput der Shroud"))
		self.CIHub.setText(self.tr('Hub', "Titel des CoordinatenInput der Hub"))
		self.CICoordinatesStackingLine.setText(self.tr('Stacking Line', "Titel des CoordinatenInput der Stacking Line"))
		self.CICoordinatesStackingPoints.setText(self.tr('Stacking Points', "Titel des CoordinatenInput der Stacking Points"))
		self.CICoordinatesTE.setText(self.tr('Leading Edge', "Titel des CoordinatenInput der Leading Edge"))
		self.CICoordinatesLE.setText(self.tr('Trailing Edge', "Titel des CoordinatenInput der Trailing Edge"))
		self.CISpan.setText(self.tr('Span', "Titel des CoordinatenInput des Span"))
		self.CIPeriphalOffset.setText(self.tr('Periphal Offset', "Titel des CoordinatenInput des Perhiphal Offset"))
		self.CISpan.setDimensions([self.tr('Werte', "Bezeichnung der Wertespalte von CISpan")])
		self.CIPeriphalOffset.setDimensions([self.tr('Werte', "Bezeichnung der Wertespalte von Periphal Offset")])

		#Texte in den Widgets anpassen
		for pCoordinatesInput in self.findChildren(QCoordinatesInputWidget):
			pCoordinatesInput.retranslateStrings()
		
		#An Profile hinunterreichen?
		if (bPassDown):
			#Durch alle Profile gehen
			for pProfile in self.__lProfileWidgets:
				#Anstoßen
				pProfile.retranslateStrings()




## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell Entfernen
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pBladeRowWidget = QBladeRowWidget()
	#pWidget = qtViewer3d(pDlg)
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#pDlg.layout().addWidget(pWidget)
	#pWidget.resize(800,600)
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pBladeRowWidget)
	#Dialog anzeigen
	pDlg.show()

	#pWidget.InitDriver()
	#pBladeRowWidget.OGLPlot.InitDriver()
	pDlg.resize(pDlg.sizeHint())
	#Ausführen
	sys.exit(pApp.exec_())

if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowCopyTest'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pBladeRowWidget = QBladeRowWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pBladeRowWidget)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	pApp.exec_()
	pDlg2 = QDialog()
	#Widget erstellen
	pBladeRowWidget2 = QBladeRowWidget(pDlg2)
	pBladeRowWidget2.copyFrom(pBladeRowWidget)
	#Nochmal das ganze nun aber laden
	#Dummy Dialog zur Anzeige des Widgets erstellen

	#Layout für den Dialog erstellen
	pDlg2.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg2.layout().addWidget(pBladeRowWidget2)
	#Dialog anzeigen
	pDlg2.show()
	#Ausführen
	pApp.exec_()