#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************


## @package BladeDesigner3D
# @author Steve Walter
# @date 08.12.2009
import sys
sys.path.append("..")
from PyQt4.QtGui import QApplication, QDialog, QVBoxLayout
from PyQt4.QtCore import Qt
from PyQt4.QtOpenGL import QGLWidget

from OpenGL.GL import 	glShadeModel, glClearDepth, glEnable, glDepthFunc, glBlendFunc, glClearColor, glClear, glLightfv, glViewport, glMatrixMode, glLoadIdentity, glRotate, \
			glPushMatrix, glScale, glPopMatrix, glTranslate, glColor3fv, glDisable, glRasterPos3f, glGetFloatv, glBegin, glColor3f, glVertex3f, glEnd, glColor4fv, \
			GL_SMOOTH, GL_DEPTH_TEST, GL_LINE_SMOOTH, GL_POINT_SMOOTH, GL_POLYGON_SMOOTH, GL_BLEND, GL_LEQUAL, GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA, GL_PERSPECTIVE_CORRECTION_HINT, \
			GL_LINE_SMOOTH_HINT, GL_POLYGON_SMOOTH_HINT, GL_NICEST, GL_COLOR_BUFFER_BIT, GL_DEPTH_BUFFER_BIT, GL_AUTO_NORMAL, GL_LIGHTING, GL_LIGHT0, GL_POSITION, \
			GL_PROJECTION, GL_MODELVIEW, GL_COLOR_MATERIAL, GL_FLAT, GL_MAP2_VERTEX_3, GL_CURRENT_COLOR, glHint, GL_LINES
from OpenGL.GLU import 	gluNewNurbsRenderer, gluPerspective, gluLookAt, gluBeginSurface, gluNurbsProperty, gluNurbsSurface, gluEndSurface, gluNewQuadric, gluCylinder, \
			gluDeleteQuadric, GLU_SAMPLING_METHOD, GLU_OBJECT_PARAMETRIC_ERROR, GLU_PARAMETRIC_TOLERANCE, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON, GLU_FILL
from OpenGL.GLUT import glutInit, glutBitmapCharacter, GLUT_BITMAP_8_BY_13

from math import tan, pi
from ctypes import c_float

import sys





## @brief Class showing 3D graphics using OpenGL and Qt<br><em>Klasse die 3D Grafiken mittels OpenGL und Qt anzeigt</em>
#
# This class is used to show a blade. Also it implements functionality for rotating, zooming and translating. For Visualization
# it uses the nurbsinterpolation with a primitiv level of detail provided by OpenGL to save computing time. There may be possible stutter when
# zooming in depending on the computer.
# <br><em>Diese Klasse wird dazu benutzt eine Blade anzuzeigen. Des Weiteren implementiert es eine
# Rotations-, Zoom- und Translationsfunktion. Zur Darstellung wird die von OpenGL bereitsgestellte
# NurbsInterpolation mit einem primitiven Level of Detail Ansatz genutzt um Rechenzeit zusparen. Bei näheren Heranzoomen kann
# es allerdings abhängig vom Rechner zu ruckeln kommen.
# </em>
#
# @sa QBladeRowWidget::QBladeRowWidget
class QOGLPlotWidget(QGLWidget):





	## @brief Constructor of QOGLPlotWidget<br><em>Konstruktor von QOGLPlotWidget</em>
	#
	# Creates the drawing window and sets initial values
	# <br><em>Erstellt das Zeichenfenster und setzt Startwerte.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		#Standardkonstruktor verwenden
		QGLWidget.__init__(self, pParent)
		self.setMinimumSize(320,240)
		self.setFocusPolicy(Qt.StrongFocus)
		#Transformationsdaten
		#Rotation um X- und Y-Achse, zu Anfangs keine
		## Rotation around the X-Axis in radiants<br><em>Rotation um die X-Achse in Radiant</em>
		# @sa __fRotY, __fRotSpeed, rotate()
		self.__fRotX = 0.0
		## Rotation around the Y-Axis in radiants<br><em>Rotation um die Y-Achse in Radiant</em>
		# @sa __fRotX, __fRotSpeed, rotate()
		self.__fRotY = 0.0

		#Distanz zum Mittelpunkt
		## Distance to the point of origin<br><em>Distanz zum Koordinatenursprung</em>
		#
		# Is used to zoom.
		# <br><em> Wird zum zoomen benötigt.
		# </em>
		#
		# @sa zoom(), __fZoomSpeed
		self.__fDistanz = 100.0

		#Eingabegerätdaten
		#Mausposition für Rotation
		## Position of the mousecursor on the screen<br><em>Position der Maus auf dem Bildschrim</em>
		#
		# Is needed to determine velocity(rotation, translation, zoom).
		# <br><em>Wird zur Ermittlung von Geschwindigkeiten(Rotation, Translation, Zoom) benötigt.
		# </em>
		#
		# @sa mousePressEvent(), mouseMoveEvent(), mouseReleaseEvent()
		self.__pMousePos = None
		#Welcher Mausbutton gedrückt
		## Holds which mousebutton was pressed<br><em>Gibt an, welcher Mausbutton gedrückt wurde</em>
		#
		# Is used to determine the type of movement(rotation, translation, zoom).
		# <br><em>Wird zur Ermittlung der Bewegungsart(Rotation, Translation, Zoom) benötigt.</em>
		#
		# @sa mousePressEvent(), mouseMoveEvent(), mouseReleaseEvent()
		self.__pMouseButton = Qt.NoButton

		#Rotationgeschwindigkeit
		## Velocity of rotation in radiants<br><em>Rotationsgeschwindigkeit in Radiant</em>
		# @sa __fRotX, __fRotY, rotate()
		self.__fRotSpeed = 0.3
		## Velocity of zoom<br><em>Zoomgeschwindigkeit</em>
		# @sa zoom(), __fDistanz
		self.__fZoomSpeed = 0.1

		#NurbsDaten
		#Daten für das Nurbs-Surface
		## Knots in V-direction of nurbsdata<br><em>Knoten in V-Richtung der Nurbsdaten</em>
		# @sa __pKnotU, __pControlPoints, __pNurbs
		self.__pKnotV = None
		## Knots in U-direction of nurbsdata<br><em>Knoten in U-Richtung der Nurbsdaten</em>
		# @sa __pKnotV, __pControlPoints, __pNurbs
		self.__pKnotU = None
		## Controlpoints of nurbsdata<br><em>Kontrollpunkte der Nurbsdaten</em>
		# @sa __pKnotU, __pKnotV, __pNurbs
		self.__pControlPoints = None
		#Farbe des Nurbssurface
		## Color of the nurbs surface<br><em>Farbe der Nurbsfläche</em>
		#
		# List of format: [R: float, G: float, B:float].
		# <br><em>Liste mit dem Format: [R: float, G: float, B:float].
		# </em>
		#
		# @sa __pNurbs, paintGL(), setNurbsSurface()
		self.__pColor = None
		#Position des Surfaces
		## Position of nurbs surface in space<br><em>Position der Nurbsfläche im Raum</em>
		#
		# List of format: [X: float, Y:float, Z:float].
		# <br><em>Liste mit dem Format: [X: float, Y:float, Z:float].
		# </em>
		#
		# @sa __pNurbs, setNurbsSurface(), paintGL()
		self.__pPosition = None
		#Nurbsrenderobjekt
		## Nurbs surface as OpenGL-object<br><em>Nurbsfläche als OpenGL-Objekt</em>
		# @sa __pKnotU, __pKnotV, __pControlPoints, __pPosition, __pColor, setNurbsSurface(), paintGL()
		self.__pNurbs = gluNewNurbsRenderer()

		#Tesselationdaten
		#Level of Detail Parameter
		## Parameter for tesselation<br><em>Parameter für Tesselation</em>
		self.__fParametricTolerance = c_float(1.0)
		#Distanz zum Mittelpunkt ab dem das Model am detailarmsten dargestellt wird
		## Distanc to the center from which the model is displayed with the least detail.<br><em>Distanz zum Mittelpunkt ab der das Modell am detailarmsten dargestellt wird.</em>
		# @sa __fMinLoDDist, __fLoDDist
		self.__fMaxLoDDist = 250.0
		#Distanz zum Mittelpunkt ab dem das Model am detailreichsen dargestellt wird
		## Distance to the center from whicht the model is displayed with the most detail.<br><em>Distanz zum Mittelpunkt ab dem das Modell am detailreichsten dargestellt wird</em>
		# @sa __fMaxLoDDist, __fLoDDist
		self.__fMinLoDDist = 95.0
		#Gesamtgröße des Bereichs in dem das LoD angepasst wird
		## Fullsize of the range where Level of Detail takes place<br><em>Gesamtgröße des Bereichs in dem das Level of Detail angepasst wird</em>
		# @sa __fMaxLoDDist, __fMinLoDDist
		self.__fLoDDist = self.__fMaxLoDDist - self.__fMinLoDDist
		#Detailgrad des Models wenn man nächsten ist
		## Degree of detail, when the model has the most detail<br><em>Detailgerad des Models, wenn es am detailreichsten ist</em>
		# @sa __fMinLoD, __fLoDDelta
		self.__fMaxLoD = 0.09
		#Detailgrad des Models wenn man am weitesten davon entfernt ist
		## Degree of detail, when the model has the least detail<br><em>Detailgrad des Models, wenn es am detailärmsten dargestellt wird</em>
		# @sa __fMaxLoD, __fLoDDelta
		self.__fMinLoD = 5.7
		#Größe des Korrekturbereichs
		## Size of the range to correct<br><em>Größe des Korrekturbereichs</em>
		# @sa __fMinLoD, __fMaxLoD
		self.__fLoDDelta = self.__fMinLoD - self.__fMaxLoD

		#Blickmittelpunkt der Kamera
		## Center of view of the camera<br><em>Blickmittelpunkt der Kamera</em>
		# @sa paintGL(), move()
		self.__vCenterPos = [0.0, 0.0, 0.0]

		#Drahtgitter oder Solid rendern?
		## Show as wireframe or solid mesh?<br><em>Als Drahtgittermodell oder als Festkörper zeichnen?</em>
		#
		# False = solid mesh, true = wireframe. Default: False. Changeable using F1.
		# <br><em> False = Festkörper, True = Drahtgittermodell. Standard: False. Umstellbar mittels F1.
		# </em>
		#
		# @sa keyPressEvent(), paintGL()
		self.__bRenderWireframe = False
		#Smooth oder Flat-Shading
		## Flatshading or Smoothshading<br><em>Flatshading oder Smoothshading</em>
		#
		# 0 = Smoothshading, 1 = Flatshading. Default: 0. Changeable using F2.
		# <br><em> 0 = Smoothshading, 1 = Flatshading. Standard: 0. Umstellbar mittels F2.
		# </em>
		#
		# @sa keyPressEvent(), paintGL()
		self.__iShadeModel = 0





	## @brief Sets the data shown in the 3D window<br><em>Setzt die Daten, die im 3D-Fenster angezeigt werden sollen</em>
	#
	# Therefore data is copied into the buffer und shown every frame.
	# <br><em> Dazu werden die Daten in den Zwischenspeicher kopiert und dann in jedem Frame angezeigt.
	# </em>
	#
	# @param pSurface - Nurbs surface containing all data<br><em>Nurbsfläche die alle Daten beinhaltet</em>
	# @param pPosition - List of coordinates containing the position of nurbs surface<br><em>Liste mit den Positionskoordinaten der Nurbsfläche</em>
	# @param pColor - Color of the nurbs surface<br><em>Farbe der Nurbsfläche</em>
	#
	# @sa __pKnotU, __pKnotV, __pControlPoints, __pColor, __pPosition, paintGL()
	def setNurbsSurface(self, pSurface, pPosition = None, pColor = None):
		#Surface holen
		#surface = turboMachineExample.turbo.listRows[0].bladeSurface
		#Startwerte setzen
		self.__pKnotV = []
		self.__pKnotU = []
		self.__pControlPoints = []
		#KnotV und KnotU hinzufügen
		for i in range(pSurface.knotV().size()):
			self.__pKnotV.append(pSurface.knotV()[i])
		for i in range(pSurface.knotU().size()):
			self.__pKnotU.append(pSurface.knotU()[i])

		#Kontrollpunkte anhängen
		# Zeile auskommentieren, falls die Beleuchtungsnormalen verkehrt herum sind
		#for u in range(pSurface.ctrlPnts().rows()):
		for u in range(pSurface.ctrlPnts().rows()-1,-1,-1):
			lv = []
			for v in range(pSurface.ctrlPnts().cols()):
				lu = []
				lu.append(pSurface.ctrlPnts().getelem(u, v).getx())
				lu.append(pSurface.ctrlPnts().getelem(u, v).gety())
				lu.append(pSurface.ctrlPnts().getelem(u, v).getz())
				lv.append(lu)
			self.__pControlPoints.append(lv)

		#Farbe übernehmen
		self.__pColor = pColor
		#Position übernehmen
		self.__pPosition = pPosition
		
		


		
	## @brief Initializes OpenGL in the Widget<br><em>Initialisiert OpenGL im Widget</em>
	#
	# Therfore the shadingmodel und some parameters(z-buffer, smooth parameter, blend function) are set.
	# Also the z-buffer is cleared, lighting activated and a parallel light source installed.
	# <br><em>Dazu wird das Shadingmodel und einige Parameter (Z-Buffer, Smooth Parameter, Blendfunktion) gesetzt.
	# Ebenso wird der Z-Buffer gelöscht, die Beleuchtung aktiviert und eine parrallele Lichtquelle erstellt.
	# </em>
	def initializeGL(self):
		#GLUT initialisieren
		glutInit(sys.argv)
		#Weiche Farbverläufe, nicht Facebeleuchtung nutzen
		glShadeModel(GL_SMOOTH)
		#Z-Buffer-Tiefe auf 1.0 setzen
		glClearDepth(1.0)
		# Einige Parameter anschalten - Z-Buffer ...
		for option in [GL_DEPTH_TEST,GL_LINE_SMOOTH, GL_POINT_SMOOTH, \
			GL_POLYGON_SMOOTH,GL_BLEND]:
			glEnable(option)

		#Z-Buffer-Funktion setzen
		glDepthFunc(GL_LEQUAL);
		##Alphablending anschalten
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)

		#Hinweise für OpenGL-Setzen
		for hint in [GL_PERSPECTIVE_CORRECTION_HINT,GL_LINE_SMOOTH_HINT, \
			GL_POLYGON_SMOOTH_HINT]:
			glHint(hint,GL_NICEST)

		#Framebuffer löschfarbe setzen
		glClearColor(0.9, 0.9, 0.9, 0)
		#Z- und Framebuffer löschen
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		#Automatische Normalenberechnung aktivieren
		glEnable(GL_AUTO_NORMAL)
		#Beleuchtung aktivieren
		glEnable(GL_LIGHTING)
		#Parrallele Lichtquelle erstellen
		glLightfv(GL_LIGHT0, GL_POSITION, [0.0, 0.0, 1.0, 0.0])
		#Punktlicht aktivieren
		glEnable(GL_LIGHT0)





	## @brief Is called on resize of the widget<br><em>Wird aufgerufen wenn sich die Größe des Widgets ändert</em>
	#
	# Is used to adjust the OpenGL to the new size of the widget.
	# Therefore the viewport is expanded and the projection matrix adjusted.
	# <br><em>Wird dazu benutzt um OpenGL an die neue Größe des Widgets anzupassen.
	# Dazu wird der Viewport erweitert und die Projektionsmatrix angepasst.
	# </em>
	#
	# @param iWidth - New width of th widget<br><em>Neue Breite des Widgets</em>
	# @param iHeight - New height of the widget<br><em>Neue Höhe des Widgets</em>
	def resizeGL(self, iWidth, iHeight):
		#Viewport wieder auf gesamte Widgetgröße anpassen
		glViewport(0, 0, iWidth, iHeight)
		#Projektionsmatrix einstellen
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		gluPerspective(45.0, float(iWidth)/float(iHeight), 0.1, 1000.0)




	
	## @brief Drawing routine, draws the figure<br><em>Zeichenroutine, zeichnet die Form</em>
	#
	# First the back- and z-buffer is cleard and the view matrix is adjusted.
	# Afterwards the nurbs surface und the coordinate system is drawn.
	# <br><em>Dazu wird zuerst der Back- und Z-Buffer gelöscht und die ViewMatrix entsprechend angepasst.
	# Anschließend wird Nurbsfläche und das Koordinatensystem gezeichnet
	# </em>
	#
	# @sa QOGLPlotWidget
	def paintGL(self):
		#Sichbar?
		if (not self.isVisible()):
			#Falls nicht sparen wir uns ne Menge Rechenzeit
			return

		#Frame- und Z-Buffer löschen
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		#Worldmatrizen bearbeiten
		glMatrixMode(GL_MODELVIEW)

		#Weltmatrizen zurücksetzen
		glLoadIdentity()
		#Kamera korrekt Positionieren
		gluLookAt(self.__vCenterPos[0], self.__vCenterPos[1], self.__vCenterPos[2]+self.__fDistanz, self.__vCenterPos[0], self.__vCenterPos[1], self.__vCenterPos[2], 0.0, 1.0, 0.0)
		#Kamerawinkel anpassen
		glRotate(self.__fRotY, 0.0, 1.0, 0.0)
		glRotate(self.__fRotX, 1.0, 0.0, 0.0)
		
		#Koordinatensystem in die Welt zeichnen
		#Matrix für Koordinatensystem auf den Stack pushen
		glPushMatrix()
		#Koordinatensystem stark vergrößern
		glScale(10.0, 10.0, 10.0)
		#Koordinatensystem zeichnen
		self.drawCoordinateSystem()
		#Matrix für Koordinatensytem wird nichtmehr gebraucht
		glPopMatrix()

		#Prüfen ob Daten vorhanden
		if((self.__pKnotU != None) and (self.__pKnotV != None) and (self.__pControlPoints != None)):
			#Farbgebung durch Materialien ausschalten, Farbe des Objekts wird somit nur durch glColor bestimmt
			glEnable(GL_COLOR_MATERIAL)
			#Shadingmodell anpassen
			glShadeModel((GL_SMOOTH, GL_FLAT)[self.__iShadeModel])
			#Matrix für Nurbsfläche auf den Stack pushen
			glPushMatrix()
			#Position angegeben?
			if((self.__pPosition != None) or (len(self.__pPosition) != 3)):
				#Position übergeben
				#glTranslate(-300.0, 0.0, -50.0)
				glTranslate(self.__pPosition[0], self.__pPosition[1], self.__pPosition[2])
			#Farbe vorhanden?
			if((self.__pColor != None) or (len(self.__pColor) != 3)):
				##Farbe setzen
				glColor3fv(self.__pColor)
			#Start des Zeichenvorgangs
			gluBeginSurface(self.__pNurbs)
			#Parametrisches LoD einstellen
			gluNurbsProperty(self.__pNurbs, GLU_SAMPLING_METHOD, GLU_OBJECT_PARAMETRIC_ERROR)
			#LoD setzen
			gluNurbsProperty(self.__pNurbs, GLU_PARAMETRIC_TOLERANCE, self.__fParametricTolerance)
			#Drahtgittermodus?
			if(self.__bRenderWireframe):
				#Falls ja, aktivieren
				gluNurbsProperty(self.__pNurbs, GLU_DISPLAY_MODE, GLU_OUTLINE_POLYGON)
			else:
				#Ansonsten normale Polygone
				gluNurbsProperty(self.__pNurbs, GLU_DISPLAY_MODE, GLU_FILL)

			#Nurbsfläche zeichnen
			gluNurbsSurface(self.__pNurbs, self.__pKnotU, self.__pKnotV, self.__pControlPoints , GL_MAP2_VERTEX_3)
			#Zeichenvorgang beenden
			gluEndSurface(self.__pNurbs)
			#Nurbsmatrix vom Stack schmeißen
			glPopMatrix()
			#Farbgebung durch Materialien einschalten
			glDisable(GL_COLOR_MATERIAL)
			#Shadingmodell zurücksetzen
			glShadeModel(GL_SMOOTH)

		#Koordinatensystem zeichnen
		#Z-Buffer löschen
		glClear(GL_DEPTH_BUFFER_BIT)
		#Nach dem kleineren von Höhe/Breite richten
		fValue = self.width()
		#Prüfen ob Höhe kleiner
		if (fValue > self.height()):
			#dann Höhe nehmen
			fValue = self.height()
		#Größe berechen, 1/4 der Entsprechend Größe
		fValue /= 4
		#Viewport auf linke untere Ecke setzen
		glViewport(0, 0, fValue, fValue)
		#Projectionsmatrix ändern
		glMatrixMode(GL_PROJECTION)
		#Aktuelle Matrix zwischenspeichern -> dazu neue auf den Stack
		glPushMatrix()
		#Einheitsmatrix laden
		glLoadIdentity()
		#Perspektivische Projektion auf Quadrat stellen
		gluPerspective(45.0, 1.0, 0.1, 300.0)
		#Den Rest im Model-Space erledigen
		glMatrixMode(GL_MODELVIEW)
		#Weltmatrizen zurücksetzen
		glLoadIdentity()
		#Kamera auf Koordinatensystem schauen lassen
		gluLookAt(0.0, 0.0, 10.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)
		#Winkel anpassen
		glRotate(self.__fRotX, 1.0, 0.0, 0.0)
		glRotate(self.__fRotY, 0.0, 1.0, 0.0)
		#Koordinatensystem vergrößern
		glScale(2.5, 2.5, 2.5)
		#Koordinatensystem zeichnen
		self.drawCoordinateSystem(True)
		#Viewport wieder herstellen
		glViewport(0, 0, self.width(), self.height())
		#Projektionsmatrix wieder zurücksetzen
		glMatrixMode(GL_PROJECTION)
		glPopMatrix()





	## @brief Input routine, is called when a key is pressed<br><em>Eingaberoutine, wird immer aufgerufen, wenn eine Taste gedrückt wurde</em>
	#
	# Takes care of the change between Shadingmodels and type of grid.
	# F1 changes the type of grid(wireframe, solid mesh) and F2 changes the shadingmodel(smooth, flat).
	# <br><em>Ist für den Wechsel der ShadingModelle und der Gitterart verantwortlich.
	# F1 wechselt die Gitterart(Drahtgitter, Festkörper) und F2 das Shadingmodell(Smooth, Flat).
	# </em>
	#
	# @param pEvent - Event, containing the pressed key<br><em>Event, dass die gedrückten Tasten enthält</em>
	#
	# @sa __bRenderWireframe, __iShadeModel
	def keyPressEvent(self, pEvent):
		#Taste kontrollieren
		#F1 -> Drahtgitter
		if(pEvent.key() == Qt.Key_F1):
			#Drahtgittermodus invertieren
			self.__bRenderWireframe = not self.__bRenderWireframe

		#F2 -> Shadingmodus ändern
		if(pEvent.key() == Qt.Key_F2):
			#Shadingmodell ändern
			self.__iShadeModel = (self.__iShadeModel + 1) % 2

		#Neuzeichnen erzwingen
		self.updateGL()



		

	## @brief Input routine, is called, when a mousebutton is pressed<br><em>Eingaberoutine, wird aufgerufen, wenn ein Mausbutton gedrückt wurde</em>
	#
	# Simply determines which button is pressed at which position.
	# <br><em>Ermittelt lediglich welcher Knopf und an welcher Stelle
	# </em>
	#
	# @param pEvent - Event, containing the pressed mousebutton and mouseposition<br><em>Event, dass den gedrückten Mausbutton und die Mausposition enthält</em>
	#
	# @sa __pMouseButton, __pMousePos, mouseMoveEvent(), mouseReleaseEvent()
	def mousePressEvent(self, pEvent):
		#Mausbutton setzen
		self.__pMouseButton = pEvent.button()
		#Mausposition setzen
		self.__pMousePos = pEvent.posF()





	## @brief Input routine, is called when the mouse is moved<br><em>Eingaberoutine, wird aufgerufen, wenn sich die Maus bewegt</em>
	#
	# Determines how far the mouse has moved and calls the corresponding functionality(rotation, translation, zoom) depending on
	# the keystroke.
	# <br><em>Ermittelt wie weit sich die Maus bewegt hat und ruft dann je nach Tastendruck die entsprechende
	# Funktionalität(Rotieren, Translation, Zoom) auf.
	# </em>
	#
	# @param pEvent - Event, containing the new mouseposition<br><em>Event, dass die neue Mausposition enthält</em>
	#
	# @sa __pMouseButton, __pMousePos, mousePressEvent(), mouseReleaseEvent()
	def mouseMoveEvent(self, pEvent):
		#Ist ein Button gedrückt?
		if (self.__pMouseButton == Qt.NoButton):
			#Dann ist es nicht von interesse
			return

		#Neue Position auslesen
		pNewPos = pEvent.posF()
		#Unterschied berechnen
		fDeltaX = pNewPos.x() - self.__pMousePos.x()
		fDeltaY = pNewPos.y() - self.__pMousePos.y()
		#Neue Mausposition setzen
		self.__pMousePos = pNewPos

		#Ist rechter Button gedrückt?
		if (self.__pMouseButton == Qt.RightButton):
			#Zoomen
			self.zoom(fDeltaY*self.__fZoomSpeed)

		#Ist linker Button gedrückt?
		if (self.__pMouseButton == Qt.LeftButton):
			#Rotieren
			self.rotate(fDeltaY*(self.__fRotSpeed), fDeltaX*self.__fRotSpeed)

		#Ist mittlerer Button gedrückt?
		if (self.__pMouseButton == Qt.MidButton):
			#Bewegen
			self.move(fDeltaX, fDeltaY)

		#Neuzeichnen erzwingen
		self.updateGL()





	## @brief Input routine, called when a mousebutton is released<br><em>Eingaberoutine, wird aufgerufen, wenn ein Mausbutton losgelassen wird</em>
	#
	# Resets the pressed mousebutton.
	# <br><em>Setzt lediglich den gedrückten Mausbutton zurück.
	# </em>
	#
	# @param pEvent - Event, containing the released mousebutton<br><em>Event, dass den los gelassenen Mausbutton enthält</em>
	#
	# @sa __pMouseButton, __pMousePos, mousePressEvent(), mousePressEvent()
	def mouseReleaseEvent(self, pEvent):
		#Mausbutton wieder freigeben
		self.__pMouseButton = Qt.NoButton





	## @brief Input routine, is called, when the mousewheel is turned<br><em>Eingaberoutine, wird aufgerufen, wenn das Mausrad gedreht wird</em>
	#
	# Adjustes the zoom.
	# <br><em>Passt den Zoom entsprechend an.
	# </em>
	#
	# @param pEvent - Event, containing how far the mousewheel has turned<br><em>Event, dass enthält, wie weit sich das Mausrad gedreht hat</em>
	#
	# @sa __fZoomSpeed, zoom()
	def wheelEvent(self, pEvent):
		#Mausrad rotiert, zoomen
		#Korrekturfaktor berechnen um Zoomspeed gleich lassen zukönnen
		self.zoom(float(pEvent.delta()/8)*self.__fZoomSpeed)





	## @brief Writes a text in the 3D window<br><em>Schreibt einen Text ins 3D-Fenster</em>
	#
	# @param x - X-coordinate of the text<br><em>X-Koordinate des Texts</em>
	# @param y - Y-coordinate of the text<br><em>Y-Koordinate des Texts</em>
	# @param z - Z-coordinate of the text<br><em>Z-Koordinate des Texts</em>
	# @param text - Text to write<br><em>Text der geschrieben werden soll</em>
	#
	# @sa paintGL(), drawCoordinateSystem()
	def writeText(self,x,y,z,text) :
		#3D-Punkt in die 2D-Ebene projezieren
		glRasterPos3f(x,y,z)
		#Für jedes Zeichen des Textes
		for i in range(len(text)) :
			#Zeichen mappen
			glutBitmapCharacter(GLUT_BITMAP_8_BY_13, ord(text[i]));
		




	## @brief Rotates a figure<br><em>Rotiert die Figur</em>
	#
	# Is called by mouseMoveEvent(), when the left mousebutton is pressed.
	# <br><em>Wird von mouseMoveEvent() aufgerufen, wenn die linke Maustaste gedrückt wird.
	# </em>
	#
	# @param fRotX - Rotation around the x-axis in radiants<br><em>Rotation um die X-Achse in Radiant</em>
	# @param fRotY - Rotation around the y-axis in radiants<br><em>Rotation um die Y-Achse in Radiant</em>
	#
	# @todo Correct the error of alignment while rotating<br><em>Ausrichtungsfehler bei Rotation beheben</em>
	#
	# @sa mouseMoveEvent(), zoom(), move()
	def rotate(self, fRotX, fRotY):
		#Entsprechend rotieren
		self.__fRotX += fRotX
		self.__fRotY += fRotY

		#neu zeichnen
		self.updateGL()





	## @brief Zooms in the figure<br><em>Zoomt auf die Figur</em>
	#
	# Is called by mouseMoveEvent(), when the right mousebutton is pressed or the mousewheel is turned.
	# <br><em>Wird von mouseMoveEvent() aufgerufen, wenn die rechte Maustaste gedrückt oder das Mausrad gedreht wird.
	# </em>
	#
	# @param fZoomfactor - Factor to zoom<br><em>Faktor um den gezoomt wird</em>
	#
	# @sa  mouseMoveEvent(), rotate(), move()
	def zoom(self, fZoomfactor):
		#Distanz anpassen
		self.__fDistanz -= fZoomfactor

		#Parametrische Distanz neu berechnen
		#--------------------|----------------------------------|----------------------->
		#	 	fMaxLoDDist			   fMinLoDDist			0
		#   außerhalb			innerhalb			vor		LoD-Gleitbereichs
		#außerhalb des LoD-Gleitbereichs?
		if(self.__fDistanz > self.__fMaxLoDDist):
			#dann Maximalwert nutzen
			self.__fParametricTolerance = self.__fMinLoD
		#vor LoD-Gleitbereichs?
		elif(self.__fDistanz < self.__fMinLoDDist):
			#dann minimalwert setzen
			self.__fParametricTolerance = self.__fMaxLoD
		#Ansonsten innerhalb des LoD-Gleitbereichs
		else:
			#Relative Distanz berechnen
			self.__fParametricTolerance = ((self.__fDistanz - self.__fMinLoDDist) / self.__fLoDDist) * self.__fLoDDelta + self.__fMaxLoD

		#neu zeichnen
		self.updateGL()





	## @brief Moves the figure<br><em>Bewegt die Figur</em>
	#
	# Is called by mouseMoveEvent(), when the mousewheel is pressed.
	# <br><em>Wird von mouseMoveEvent() aufgerufen, wenn das Mausrad gedrückt wird.
	# </em>
	#
	# @param fDeltaX - Distance, which the figure is moved in x-direction<br><em>Distanz, welche die Figur in X-Richtung verschoben werden soll</em>
	# @param fDeltaY - Distance, which the figure is moved in y-direction<br><em>Distanz, welche die Figur in Y-Richtung verschoben werden soll</em>
	#
	# @todo Correct the error of alignment while rotating<br><em>Ausrichtungsfehler bei Rotation beheben</em>
	#
	# @sa mouseMoveEvent(), rotate(), zoom()
	def move(self, fDeltaX, fDeltaY):
		#Blickdreieck berechnen um die Breite der Ebene zuermitteln in der bewegt wird
		ratio = (tan(pi/8)*self.__fDistanz)/(float(self.width())*0.5)

		#Neue Position berechnen
		self.__vCenterPos[0] -= ratio*fDeltaX
		self.__vCenterPos[1] += ratio*fDeltaY

		#neu zeichnen
		self.updateGL()





	## @brief Draws a coordinate system<br><em>Zeichnet ein Koordinatensystem</em>
	#
	# Can be drawn with labeling and arrows optinally.
	# <br><em> Lässt sich optional mit oder ohne Beschriftung und Pfeile zeichnen.
	# </em>
	#
	# @param bDrawArrows - True = with arrows and labeling, False = without<br><em>True = mit Pfeilen und Beschriftung, False = ohne</em>
	#
	# @sa paintGL()
	def drawCoordinateSystem(self, bDrawArrows = False):
		#Beleuchtung ausschalten
		glDisable(GL_LIGHTING)
		#Farbe zwischenspeichern
		pOldColor = glGetFloatv(GL_CURRENT_COLOR)
		
		#Koordinatensystem zeichnen
		glBegin(GL_LINES)
		glColor3f( 1.0, 0.0, 0.0)
		#X-Achse
		glVertex3f(-1.0, 0.0, 0.0)
		glVertex3f( 1.0, 0.0, 0.0)

		glColor3f( 1.0, 1.0, 0.0)
		#Y-Achse
		glVertex3f( 0.0, -1.0, 0.0)
		glVertex3f( 0.0,  1.0, 0.0)

		glColor3f( 0.0, 1.0, 0.0)
		#Z-Achse
		glVertex3f( 0.0, 0.0, -1.0)
		glVertex3f( 0.0, 0.0,  1.0)
		glEnd()

		#Falls keine Pfeile gezeichnet werden sollen hier abbrechen
		if(not bDrawArrows):
			#Farbe wiederherstellen
			glColor4fv(pOldColor)
			#Beleuchtung wieder einschalten
			glEnable(GL_LIGHTING)
			return

		#Werte für die Kegel
		#Kegel schaut zeigt standardmäßig richtung Z-Achse
		lKegel = (
			#Rotation X-Achse, Rotation Y-Achse, Farbe
			#Z-Kegel, keine XRot, keine YRot, Farbe blau
			(0.0, 0.0, [0.0, 1.0, 0.0]),
			#X-Kegel, kein XRot, 90° YRot, Farbe grün
			(0.0, 90.0, [1.0, 0.0, 0.0]),
			#Y-Kegel, 90° XRot, keine YRot, Farbe gelb
			(-90.0, 0.0, [1.0, 1.0, 0.0])
			)

		#Kegel für Pfeile erstellen
		pKegel = gluNewQuadric()
		
		for pEntry in lKegel:
			#Kegel Z-Achse erstellen
			glPushMatrix()
			#Farbe setzen
			glColor3fv(pEntry[2])
			#An die entsprechende Stelle schieben
			glRotate(pEntry[0], 1.0, 0.0, 0.0)
			glRotate(pEntry[1], 0.0, 1.0, 0.0)
			glTranslate(0.0, 0.0, 1.0)
			gluCylinder(pKegel, 0.08, 0.0, 0.3, 8, 1)
			glPopMatrix()


		#Achsen Beschriften
		glColor3f(0.0, 0.0, 0.0)
		self.writeText(0.0, 1.4, 0.0, "y")
		self.writeText(1.4, 0.0, 0.0, "x")
		self.writeText(0.0, 0.0, 1.4, "z")
			
		#Kegel freigeben
		gluDeleteQuadric(pKegel)
		#Farbe wiederherstellen
		glColor4fv(pOldColor)
		#Beleuchtung wieder einschalten
		glEnable(GL_LIGHTING)





#Testcase starten
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Applikation erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pProfile2DWidget = QOGLPlotWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pProfile2DWidget)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	sys.exit(pApp.exec_())


