#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009
from PyQt4.QtCore import QSize, SIGNAL, SLOT
from PyQt4.QtGui import QWidget, QVBoxLayout, QComboBox, QSizePolicy, QHBoxLayout, QIcon, QPixmap
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4 import NavigationToolbar2QT as NavigationToolbar

from matplotlib.figure import Figure

import numpy as N

## @brief Helperclass fpr QPlotwitget, which forms the graphical window<br><em>Hilfsklasse für QPlotWidget die das Grafikfenster darstellt</em>
#
# Therefor the functionality provided by matplotlib is used. It includes a graphical window compatible to Qt called FigureCanvas.
# <br><em>Dafür wird die von matplotlib bereitgestellte Funktionalität genutzt.Diese beinhaltet ein
# mit Qt kompatibles Zeichenfenster FigureCanvas.
# </em>
#
# @sa QPlotWidget
class MyMplCanvas(FigureCanvas):

	## @brief Constructor of MyMplCanvas<br><em>Konstruktor des MyMplCanvas</em>
	#
	# Creates the FigureCanvas and with it the drawing area. Initializes some texts and settings
	# of the coordinate system.
	# <br><em>Erstellt das FigureCanvas und damit die Zeichenfläche. Initialisiert
	# einige Texte und Einstellungen des Koordiantensystems.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	# @param width - Width of the figure in inches<br><em>Breite der Figur in inches</em>
	# @param height - Height of the figure in inches<br><em>Höhe der Figur in inches</em>
	# @param dpi - Resolution<br><em>Auflösung</em>
	# @param sharex - <br><em></em>
	# @param sharey - <br><em></em>
	#
	# @sa QPlotWidget
	def __init__(self, pParent=None, width = 10, height = 12, dpi = 100, sharex = None, sharey = None):
		## @brief Graphic to be shown<br><em>Grafik die angezeigt wird</em>
		self.fig = Figure(figsize = (width, height), dpi=dpi, facecolor = '#FFFFFF')
		## @brief Subgraphic<br><em>Untergrafik</em>
		self.ax = self.fig.add_subplot(111, sharex = sharex, sharey = sharey)
		self.fig.subplots_adjust(left=0.1, bottom=0.1, right=0.9, top=0.9)
		## @brief Labeling of x-axis<br><em>Beschriftungen der X-Achse</em>
		self.xtitle="x-Axis"
		## @brief Labeling of y-axis<br><em>Beschriftungen der Y-Achse</em>
		self.ytitle="y-Axis"
		## @brief Header of the graphical window<br><em>Titel des Zeichenfenster</em>
		self.PlotTitle = "Calculation"
		## @brief State of the grid<br><em>Status des Gitters</em>
		self.grid_status = True
		## @brief Type of x-axis<br><em>Art der X-Achse</em>
		self.xaxis_style = 'linear'
		## @brief Type of y-axis<br><em>Art der Y-Achse</em>
		self.yaxis_style = 'linear'
		self.format_labels()
		self.ax.hold(True)
		FigureCanvas.__init__(self, self.fig)
		#self.fc = FigureCanvas(self.fig)
		FigureCanvas.setSizePolicy(self,
			QSizePolicy.Expanding,
			QSizePolicy.Expanding)
		FigureCanvas.updateGeometry(self)





	## @brief Scales the labeling to the corresponding size<br><em>Skaliert die Beschriftungen entsprechend</em>
	def format_labels(self):
		self.ax.set_title(self.PlotTitle)
		self.ax.title.set_fontsize(10)
		self.ax.set_xlabel(self.xtitle, fontsize = 9)
		self.ax.set_ylabel(self.ytitle, fontsize = 9)
		labels_x = self.ax.get_xticklabels()
		labels_y = self.ax.get_yticklabels()

		for xlabel in labels_x:
			xlabel.set_fontsize(8)
		for ylabel in labels_y:
			ylabel.set_fontsize(8)
			ylabel.set_color('b')





	## @brief Recommended sizehint for Qt<br><em>Empfohlener Größenhinweis für Qt</em>
	#
	# Gets the recommended size from FigureCanvas.
	# This function is demanded by Qt.
	# <br><em> Holt die Empfohlene Größe vom FigureCanvas.
	# Von Qt verlange Funktion.
	# </em>
	#
	# @sa minimumSizeHint()
	def sizeHint(self):
		w, h = self.get_width_height()
		return QSize(w, h)





	## @brief Minimal sizehint for Qt<br><em>Minimalster Größehinweis für Qt</em>
	#
	# Should be used to actually not shrink the window to an invalid size, but it doesn't work like expected.
	# It becomes obsoleted with the change in presentation style.
	# <br><em>Sollte eigentlich dafür benutzt werden damit das Fenster nicht unnötig klein gemacht wird, funktioniert aber nicht so wie erwartet.
	# Durch die Umstellung der Anzeigeoptionen ist es auch nichtmehr so von nöten.
	# </em>
	#
	# @sa sizeHint()
	def minimumSizeHint(self):
		return QSize(100 , 100)





## @brief Widget to draw 2D graphs<br><em>Widget zum zeichnen von 2D Graphen</em>
#
# This class is used to integrate drawing functions of matplotlib into Qt.
# Therefore all functionality is passed to the integrated figureCanvas, which cares for execution.
# This class is only a kind of wrapper.
# <br><em>Diese Klasse dient zur Integration von matplotlib Zeichenfunktionen in Qt.
# Alle Funktionalitäten werden dabei an das interne FigureCanvas weitergegeben, welches sich dann
# um die ausführung kümmert. Die Klasse an sich stellt nur eine Art Wrapper dar.
# </em>
#
# @sa MyMplCanvas, QProfile2DWidget::QProfile2DWidget
class QPlotWidget(QWidget):
	## @brief Available colors<br><em>Verfügbare Farben</em>
	#
	# <ul>
	# <li> b = blue/<em>blau</em> </li>
	# <li> g = green/<em>grün</em> </li>
	# <li> r = red/<em>rot</em> </li>
	# <li> c = cyan/<em>blaugrün</em> </li>
	# <li> m = magenta/<em>violett</em> </li>
	# <li> y = yellow/<em>gelb</em> </li>
	# <li> k = black/<em>schwarz</em> </li>
	# <li> w = white/<em>weiß</em> </li>
	# </ul>
	#
	# @sa PlotStyle
	PlotColor = ['b', 'g', 'r', 'c', 'm', 'y', 'k', 'w']
	## @brief Available linestyles<br><em>Verfügbare Linienstile</em>
	#
	# <ul>
	# <li> - = continuous line/<em>durchgezoge Linie</em> </li>
	# <li> -- = dashed line/<em>gestrichelte Linie</em> </li>
	# <li> -. = dash-dot-line/<em>Strich-Punkt-Linie</em> </li>
	# <li> : = dotted line/<em>gepunktete Linie</em> </li>
	# <li> . = big points/<em>große Punkte</em> </li>
	# <li> , = small points/<em>kleine Punkte</em> </li>
	# <li> o = circle points/<em>Kreispunkte</em> </li>
	# <li> s = square points/<em>Quadratpunkte</em> </li>
	# <li> + = cross points/<em>Kreuzpunkte</em> </li>
	# <li> x = x points/<em>x Punkte</em> </li>
	#
	# @sa PlotColor
	PlotStyle = ['-', '--', '-.', ':', '.', ',', 'o', 's', '+', 'x']




	
	## @brief Constructor of QPlotWidget<br><em>Konstruktor von QPlotWidget</em>
	#
	# Creates the MyMplCanvas and adds an additional navigation toolbar.
	# <br><em>Erstellt das MyMplCanvas und hängt eine zusätzliche Navigationsleiste an.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	#
	# @sa MyMplCanvas
	def __init__(self, pParent = None):
		QWidget.__init__(self, pParent)
		#GUI-Elemente erstellen
		self.__setupUi()


		## @brief Lines to draw<br><em>Zu zeichnende Linien</em>
		#
		# The lines to draw are stored within through a call to plot(). They are drawn with a call to endDrawing().
		# The list of lines can be cleared with a call to beginDrawing(). This is useful in case of changing linestyle or
		# the plot color.
		# <br><em>Die zu zeichnenden Linien werden durch einen Aufruf von plot() hier zwischengespeichert und
		# erst nach einem Aufruf von endDrawing() gezeichnet. Gelöscht werden kann die Liste mittels beginDrawing().
		# Dies ist von Nutzen bei der Änderung von Farbe oder Linienstil</em>
		#
		# @sa plot(), beginDrawing(), endDrawing()
		self.__PlotEntrys = []





	## @brief Creates the GUI-Elements contained by the Widget<br><em>Erstellt die im Widget enthaltenen GUI-Elemente</em>
	#
	# This is not a function created by QtDesigner. Its is handmade to reduce overhead.
	# <br><em>Diese Funktion ist nicht mit QtDesigner erstellt. Um Overhead zu verhindern ist sie handgeschrieben</em>
	def __setupUi(self):
		## @brief Drawing window<br><em>Zeichenfenster</em>
		# @sa MyMplCanvas
		self.canvas = MyMplCanvas()
		## @brief Navigation toolbar<br><em>Navigationsleiste</em>
		self.toolbar = NavigationToolbar(self.canvas, self.canvas)
		## @brief Layout within the widget<br><em>Layout innerhalb des Widgets</em>
		self.vbox = QVBoxLayout()
		self.vbox.addWidget(self.canvas)

		## @brief Layout for toolbar, color- and style-selection<br><em>Layout für die Toolbar, sowie Farb- und Linienstilauswahl</em>
		self.hbox = QHBoxLayout()

		self.hbox.addWidget(self.toolbar)

		## @brief ComboBox to select the style of the lines<br><em>ComboBox zur Auswahl des Linienstils</em>
		self.CBColor = QComboBox()
		#Zeichenarten hinzufügen
		for pEntry in QPlotWidget.PlotColor:
			self.CBColor.addItem(QIcon(QPixmap(":/Plot/res/PlotStyles/" + pEntry +  ".png")), "")

		## @brief ComboBox to select the color<br><em>ComboBox zur Farbauswahl</em>
		self.CBLineStyle = QComboBox()
		#Farben hinzufügen
		for pEntry in QPlotWidget.PlotStyle:
			self.CBLineStyle.addItem(QIcon(QPixmap(":/Plot/res/PlotStyles/" + pEntry +  ".png")), "")

		# Farbauswahl und Linienstil hinzufügen
		self.hbox.addWidget(self.CBLineStyle)
		self.hbox.addWidget(self.CBColor)

		#Navigationsleiste, Farbauswahl und Linienstil hinzufügen
		self.vbox.addLayout(self.hbox)

		#Ins Widget einfügen
		self.setLayout(self.vbox)

		#Einträge verbinden
		self.connect(self.CBColor, SIGNAL("currentIndexChanged(int)"), self.redraw)
		self.connect(self.CBLineStyle, SIGNAL("currentIndexChanged(int)"), self.redraw)
		




	## @brief Draws a point<br><em>Zeichnet einen Punkt</em>
	#
	# This function adds a point to the list of points to draw.
	# The points are drawn or joined to lines with a call of endDrawing() or redraw().
	# <br><em> Diese Funktion fügt einen Punkt in der Liste der zuzeichnenden Punkte hinzu.
	# Die Punkte werden bei den Aufruf von endDrawing() oder redraw() entsprechend gezeichnet oder
	# zu Linien verbunden.
	# </em>
	#
	# @param PlotX - X-Coordinate of the point<br><em>X-Koordinate des Punktes</em>
	# @param PlotY - Y-Coordinate of the point<br><em>Y-Koordinate des Punktes</em>
	#
	# @sa beginDrawing(), redraw(), plot()
	def plot(self, PlotX, PlotY):
		#In zu plottende Einträge übernehmen
		self.__PlotEntrys.append([PlotX, PlotY])





	## @brief Ends the process of drawing<br><em>Beendet den Zeichenvorgang</em>
	#
	# This function draws the created graphics with the choosen color and linestyle.
	# <br><em> Diese Funktion zeichnet die erstellte Grafik mit der gewählten Farbe und Linienstil.
	# </em>
	#
	# @sa beginDrawing(), redraw(), plot()
	def endDrawing(self):
		#Farbe ermitteln
		strColor = QPlotWidget.PlotColor[self.CBColor.currentIndex()] + QPlotWidget.PlotStyle[self.CBLineStyle.currentIndex()]
		
		#Durch alle Einträge gehen
		for pPlotEntry in self.__PlotEntrys:
			#Linie zur Zeichenroutine hinzufügen
			self.canvas.ax.plot(pPlotEntry[0], pPlotEntry[1], strColor)

		#Alle angegebenen Linien zeichnen
		self.canvas.draw()





	## @brief Starts the process of drawing<br><em>Beginnt den Zeichenvorgang</em>
	#
	# This function clears the canvas and initializes the List of drawing entrys.
	# <br><em> Diese Funktion leert die Leinwand und initialisiert die Liste der Zeicheneinträge.
	# </em>
	#
	# @sa endDrawing(), redraw(), plot()
	def beginDrawing(self):
		self.canvas.ax.clear()
		self.canvas.ax.axis('equal')
		self.canvas.format_labels()
		self.__PlotEntrys = []





	## @brief Renews the graphic<br><em>Erneuert die Grafik</em>
	#
	# This function clears the canvas and redraws the graphic.
	# It can be used to activated new colors and linestyles.
	# <br><em> Diese Funktion leert die Leinwand und zeichnet dann erneut die Grafik.
	# Sie kann dazu benutzt werden um eine neue Farbe oder Linienstil zu aktivieren.
	# </em>
	#
	# @sa beginDrawing(), endDrawing(), plot()
	def redraw(self):
		self.canvas.ax.clear()
		self.canvas.format_labels()
		self.endDrawing()

