#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 15.05.2009

import sys
#Importpfad für übergeordnete Verzeichnisse setzen
sys.path.append('..')
#System-Werte zum Laden entfernen
#TODO: Entfernen, wenn alle Tests abgeschlossen
strTemp = sys.argv
sys.argv = []

from profile2D import profile2D
from numpy import zeros, radians, degrees
from PyQt4.QtGui import QWidget, QLineEdit, QDoubleValidator, QIntValidator, QApplication, QDialog, QVBoxLayout, QMessageBox, QCursor, QLabel, QGroupBox, QComboBox, QCheckBox
from PyQt4.QtCore import Qt
from Ui_Profile2DWidget import Ui_Profile2DWidget as Profile2DWidget
from QCoordinatesInputWidget import QCoordinatesInputWidget
from QStatusDialog import QStatusDialog
from QBaseWidget import QBaseWidget

#System-Werte wiederherstellen
#TODO: Entfernen, wenn alle Tests abgeschlossen
sys.argv = strTemp

			
			


## @brief Inputwidget for @link src::profile2D::profile2D profile2D @endlink class<br><em>Eingabewidget für die @link src::profile2D::profile2D profile2D @endlink Klasse</em>
#
# This widget holds all data necessary to do the calculation of @link src::profile2D::profile2D profile2D @endlink. It is able to do a stand-alone profile-calculation,
# but can also be integrated into a tree strcutre to provide data for superior objects(e.g. QBladeRowWidget). Results can be displayed via the integrated QPlotWidget.<br>
# The layout of this widget in QtDesignerform is contained in ./UI/Profile2DWidget.ui, which can be converted to UI_Profile2DWidget by ./CreateGUI.sh script using pyuic4
# and loaded afterwards.
# <br><em>Dieses Widget nimmt alle für die Berechnung von @link src::profile2D::profile2D profile2D @endlink nötigen Daten auf. Es ist im Stande selbständige
# Profile-Berechnungen durchzuführen, lässt sich allerdings auch in eine Baumstruktur integrieren um übergeordneten
# Objekten (zBsp QBladeRowWidget) Daten zuliefern. Ergebnisse lassen sich durch das integrierte QPlotWidget anzeigen.<br>	
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/Profile2DWidget.ui, welche vom ./CreateGUI.sh Script mittels pyuic4 in UI_Profile2DWidget konvertiert
# und dann geladen wird.</em>
#
# @sa @link src::profile2D::profile2D profile2D @endlink, QBaseWidget::QBaseWidget
class QProfile2DWidget(QWidget, Profile2DWidget, QBaseWidget):

	## @brief Constructor of QProfile2DWidget<br><em>Konstruktor des QProfile2DWidget</em>
	#
	#  Initializes and configurates underlying widgets. Attaches to QBaseWidget provided interfaces(#_OnPerformCalc, #_OnGetOutput).
	#  <br><em>Initialisiert und konfiguriert grundlegende Widgets, hängt von QBaseWidget bereitsgestellte Schnittstellen ein(#_OnPerformCalc, #_OnGetOutput).</em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		QWidget.__init__(self, pParent)
		QBaseWidget.__init__(self, pParent)
		self.setupUi(self)
		self.hookChangeNotify()

		#Namen setzen
		self.retranslateStrings()

		#self.mouseMoveEvent = self.customMouseMove
		#self.mousePressEvent = self.customMousePress
		#self.mouseReleaseEvent = self.customMouseRelease

		#for pEntry in self.findChildren(QLabel):
			#pEntry.mouseMoveEvent = self.customMouseMove
			#pEntry.mousePressEvent = self.customMousePress
			#pEntry.mouseReleaseEvent = self.customMouseRelease

		#Dimensionen setzen
		self.CICamberLineNurbs.setDimensions(['X', 'Y'])
		self.CIThicknessNurbs.setDimensions(['X', 'Y'])
		#Wertebereich setzen
		self.CICamberLineNurbs.setRange(0, 0.0, 1.0)
		self.CIThicknessNurbs.setRange(1, 0.0, 1.0)
		#Validatoren anhängen
		for LELineEdit in self.findChildren(QLineEdit):
			LELineEdit.setValidator(QDoubleValidator(self))

		#Sonderfälle per Hand mit IntValidator versehen
		lIntValidNeeded = ('ProfileResolution', 'CamberResolution', 'BladeRowNumber', 'ProfileNumber', 'MagFact')
		for strLineEditName in lIntValidNeeded:
			getattr(self, 'LE' + strLineEditName).setValidator(QIntValidator(self))

		#Nur das nötige Anzeigen
		self.optionChanged()

		#Debug und ReadXML-Gruppenboxen ebenfalls verstecken
		self.GBDebug.hide()
		self.GBReadXML.hide()
		#Polynom Koeffizienten auf "approximiert" setzen
		self.CBPolyMethod.setCurrentIndex(1)

		## @brief XML-file to load for testing purpose<br><em>Zu ladende XML-Datei für Testzwecke</em>
		#
		#  @sa OnGetOutput()
		#  @todo Remove this variable and change it to dynamic loading<br><em>Variable entfernen und zum dynamischen Laden wechseln</em>
		self.__strProFile = 'Profil2D.xml'

		## @brief Functionpointer from QBaseWidget::_OnPerformCalc<br><em>Funktionszeiger von QBaseWidget::_OnPerformCalc</em>
		#
		#  Attached to OnPerformCalc()<br><em>Verbunden mit OnPerformCalc()</em>
		#
		# @sa OnPerformCalc(), QBaseWidget::_OnPerformCalc
		self._OnPerformCalc = self.OnPerformCalc
		
		## @brief Functionpointer from QBaseWidget::_OnGetOutput<br><em>Funktionszeiger von QBaseWidget::_OnGetOutput</em>
		#
		#  Attached to OnGetOutput()<br><em>Verbunden mit OnGetOutput()</em>
		#
		# @sa OnGetOutput(), QBaseWidget::_OnGetOutput
		self._OnGetOutput = self.OnGetOutput





	## @brief Resets all displayflags<br><em>Setzt alle Anzeigeflags zurück</em>
	#
	# This is a auxiliary function, which helps optionChanged() to determine the current display condition of the contained widgets. For internal use only.
	# <br><em>Dies ist eine Hilfsfunktion, die optionChanged() dabei hilft die derzeitigen Anzeigezustände der Steuerelemente zu ermitteln. Nur interne Verwendung.</em>
	#
	# @sa optionChanged()
	def __resetFlags(self):
		#Interne Variablen zur Leichteren Berechnung
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowWeights = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowCamberLineNurbs = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowThicknessNurbs = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowAngles = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowCamberResolution = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowCa0 = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowMaxCamberPosition = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowPolynom = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowWeightCondition = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowLeadingEdgeRadius = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowMaxThickness = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowThickMethod = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowAdditional = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowErrorMax = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowReadXML = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowResolution = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowPoly = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowApproxPoly = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowDirectPoly = False
		## @brief For internal uses only<br><em>Nur interne Zwecke</em>
		self.__bShowDebug = False
		




	## @brief Collects all data from the widget and fills it into profile2D<br><em>Sammelt alle Daten der Steuerelemente in das profile2D</em>
	#
	#  This function is attached to QBaseWidget::_OnGetOutput to use the provided automatisms.
	#  It determines via internal display parameters(#__bShowWeights, #__bShowCamberLineNurbs, ...) which data is inputed/activated and writes it to
	#  the (given) profile2D-object.
	#  <br><em>Diese Funktion wird an QBaseWidget::_OnGetOutput gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie ermittelt mit Hilfe der internen Anzeigeparamter(#__bShowWeights, #__bShowCamberLineNurbs, ...) welche Daten eingegeben/aktiv sind und
	#  schreibt diese dann in das (übergebene) profile2D-Objekt.</em>
	#
	#  @param pProfile - @link src::profile2D::profile2D Profile2D @endlink in which the data is to write(optional)<br><em>@link src::profile2D::profile2D Profile2D @endlink in das die Daten geschrieben werden sollen (optional)</em>
	#
	#  @return @link src::profile2D::profile2D Profile2D @endlink in which the data was written<br><em>@link src::profile2D::profile2D Profile2D @endlink in das die Daten gefüllt wurden</em>
	#
	# @sa QBaseWidget._OnGetOutput, optionChanged()
	def OnGetOutput(self, pProfile = None):
		#Ansonsten neu durchrechnen
		#Zuschreibendes Profil angegeben?
		if (pProfile != None):
			#Vorgegebenes Profil übernehmen

			## @brief outputobject of the calculation<br><em>Ausgabeobjekt der Berechnung</em>
			#
			# Overwritten from QBaseWidget::_pOutput. In this widget #_pOutput is of @link src::profile2D::profile2D profile2D @endlink type.
			# <br><em>Überschrieben von QBaseWidget::_pOutput. In diesem Widget ist #_pOutput vom Typ @link src::profile2D::profile2D profile2D @endlink.</em>
			#
			# @sa QBaseWidget::_pOutput, OnGetOutput(), OnPerformCalc()
			self._pOutput = pProfile
		else:
			#Output Struktur anlegen
			self._pOutput = profile2D()

		#Profil ist damit verändert wurden
		self._pOutput.isCalculated = False

		#Debug-Optionen aktivieren?
		if (self.__bShowDebug):
			#Debuglevel setzen
			self._pOutput.debug = self.CBDebugLevel.currentIndex()+1
			#ControlPoints?
			self._pOutput.cp = 0
			if (self.CBControlPoints.isChecked()):
				self._pOutput.cp = 1
			#MagFactor
			self._pOutput.magFact = int(self.LEMagFact.text())
			#Dash
			self._pOutput.dash = self.SBDash.value()

		#Aus XML Datei lesen?
		if self.__bShowReadXML:
			#Aus XML-Datei lesen
			self._pOutput.readXml = 1
			#XML-Werte eintrage
			self._pOutput.bladeRowNumber = int(self.LEBladeRowNumber.text())
			self._pOutput.profileNumber = int(self.LEProfileNumber.text())
			#XML einlesen
			#TODO: Testfile, muß noch geändert werden
			self._pOutput.readXmlFile(self.__strProFile)
			#Werte konvertieren
			self._pOutput.flagCamberLine = int(self._pOutput.flagCamberLine)
			self._pOutput.flagThickDistr = int(self._pOutput.flagThickDistr)
		else:
			#Auswahl übernehmen
			self._pOutput.flagCamberLine = self.CBCamberLine.currentIndex()
			#+1 da no profile rausgefallen ist
			self._pOutput.flagThickDistr = self.CBThickDistr.currentIndex()+1
			#Profil übernehmen
			self._pOutput.profileResolution = int(self.LEProfileResolution.text())
			#ChordLength Auslesen
			self._pOutput.chordLength= float(self.LEChordLength.text())
			#Gamma auslesen
			self._pOutput.gamma = radians(float(self.LEGamma.text()))

		#Camberline Resolution sichtbar?
		if self.__bShowCamberResolution:
			#Hinzufügen
			self._pOutput.camberResolution = int(self.LECamberResolution.text())

		#LeadingEdge Radius sichtbar?
		if self.__bShowLeadingEdgeRadius:
			#Hinzufügen
			self._pOutput.leadingEdgeRadius = float(self.LELeadingEdgeRadius.text())

		#MaxThickness anzeigen?
		if self.__bShowMaxThickness:
			#Hinzufügen
			self._pOutput.maxThickness = float(self.LEMaxThickness.text())

		#ThickMethod anzeigen
		if self.__bShowThickMethod:
			#Hinzufügen
			self._pOutput.thickMeth = self.CBThickMethod.currentIndex()

		#Polynom anzeigen?
		if self.__bShowPolynom:
			#Hinzufügen
			self._pOutput.polyCamberLineDegree = int(self.CBPolyCamberlineDegree.currentText())

		#MaxCamberPosition angezeigt?
		if self.__bShowMaxCamberPosition:
			#Hinzufügen
			self._pOutput.maxCamberPosition = float(self.LEMaxCambPosition.text())

		#Wichtungen anzeigen?
		if self.__bShowWeights:
			#Hinzufügen
			self._pOutput.weightCombination = self.CBWeightCombination.currentIndex()

		#Wichtungskombination angezeigt?
		if self.__bShowWeightCondition:
			#Hinzufügen
			self._pOutput.weightCondition = self.CBWeightCondition.currentIndex()

		#MaxIter und Error angezeigt?
		if self.__bShowErrorMax:
			#Error und Max-Wert übernehmen
			self._pOutput.error = float(self.LEError.text())
			self._pOutput.maxIter = int(self.LEMaxIterations.text())

		#Winkel angezeigt?
		if self.__bShowAngles:
			#Winkel übernehmen
			self._pOutput.angleOfAttack = float(self.LEAngleOfAttack.text())
			self._pOutput.outflowAngle = float(self.LEOutflowAngle.text())

		#Ca0 angezeigt?
		if self.__bShowCa0:
			#Hinzufügen
			self._pOutput.Ca0 = float(self.LECa0.text())

		#Camberline Nurbs per Hand eingeben?
		if self.__bShowCamberLineNurbs:
			#Daten aus QTableWidget holen
			lNurbs = self.CICamberLineNurbs.coordinatesAsNumpy()

			#Fehlerhafte Eingabe abfangen
			if len(lNurbs) == 0:
				QMessageBox.information(self, self.tr('Eingabefehler', 'Titel der Messagebox bei fehlenden CamberlineNurbs'), self.tr('Es wurden keine Punkte für die Camberline-Nurbskurve eingegeben\nBitte geben sie Punkte für die Nurbskurve ein', 'Nachricht der Messagebox bei fehlenden CamberlineNurbs'))
				return

			#In die Ausgabestruktur füttern
			self._pOutput.controlPointsCamb = lNurbs
			
		#ThicknessNurbs von Hand eingegeben?
		if self.__bShowThicknessNurbs:
			#Daten als Numpy holen
			lNurbs = self.CIThicknessNurbs.coordinatesAsNumpy()
			
			#Fehlerhafte Eingabe abfangen
			if len(lNurbs) == 0:
				QMessageBox.information(self, self.tr('Eingabefehler', 'Titel der Messagebox bei fehlenden ThicknessNurbs'), self.tr('Es wurden keine Punkte für die Thickness-Nurbskurve eingegeben\nBitte geben sie Punkte für die Nurbskurve ein', 'Nachricht der Messagebox bei fehlenden ThicknessNurbs'))
				return

			#In die Ausgabestruktur füttern
			self._pOutput.controlPointsThick = lNurbs
			#Approximierter Nurbsnasenradius auslesen
			self._pOutput.approxNurbsNoseRad = float(self.LEApproxNurbsNoseRadius.text())
			
		#Polynom eingegeben?
		if self.__bShowPoly:
			
			#Approximiertes Polynom Flag setzen
			self._pOutput.approxPolyCoeff = self.CBPolyMethod.currentIndex()
			
			fp0 = float(self.LEp0.text())	# Skalierungsfaktor fuer VK/HK-radius
			fp1 = float(self.LEp1.text())	# 1. exponent randwert-fkt
			fp2 = float(self.LEp2.text())	# 2. exponent randwert-fkt |=p1
			fp3 = float(self.LEp3.text())#1.35#1.65	# Exponent=1/p1; muss > 1 sein, ansonsten frei waehlbar#
			
			#approximierte Methode gewählt?
			if self._pOutput.approxPolyCoeff == 1:
				#coeff for thickness Distribution polynomial
				iBound = int(self.CBBound.currentText())	# Art zur Bestimmung der Koeffizienten [1-4]
				fb2 = float(self.LEb2.text())#0.35	# f'(b2) = 0.0		~x-wert des dicken-max
				fKruemmungsRadius = float(self.LEKrummungsRadius.text())
				self._pOutput.polyThickRBs= self._pOutput.computePolynomialThicknessCoefficients(iBound, fb2, fKruemmungsRadius, fp0, fp1, fp2, fp3)
				
			#directe Methode gewählt?
			if self._pOutput.approxPolyCoeff == 0:
				#Numpy Array erstellen
				lCoeffArray = zeros((9))
				#Mit werten füllen
				lCoeffArray[0]= float(self.LEa0.text())
				lCoeffArray[1]= float(self.LEa1.text())
				lCoeffArray[2]= float(self.LEa2.text())
				lCoeffArray[3]= float(self.LEa3.text())
				lCoeffArray[4]= float(self.LEa4.text())
				lCoeffArray[5]= fp0
				lCoeffArray[6]= fp1
				lCoeffArray[7]= fp2
				lCoeffArray[8]= fp3

				#Werte übernehmen
				self._pOutput.polyThickRBs = lCoeffArray

		



	## @brief Sets the data of the contained widgets according to a @link src::profile2D::profile2D profile2D @endlink object<br><em>Setzt die Daten in den Steuerelementen anhand eines @link src::profile2D::profile2D profile2D @endlink Objekts</em>
	#
	#  Overloaded from QBaseWidget::setFromObject(). Loads the definitions from @link src::profile2D::profile2D profile2D @endlink object, adjusts the widgets layout using optionChanged() and fills all needed data
	#  from @link src::profile2D::profile2D profile2D @endlink object into the widgets.
	#  <br><em>Ist von QBaseWidget::setFromObject() überladen. Es läd die Definition aus dem @link src::profile2D::profile2D profile2D @endlink Objekt, passt dann mit optionChanged() die Steuerelemente an und füllt alle
	#  nötigen mit den Daten aus dem @link src::profile2D::profile2D profile2D @endlink Objekt.</em>
	#
	#  @param pProfile - @link src::profile2D::profile2D Profile2D @endlink object from which the data is to read<br><em>@link src::profile2D::profile2D Profile2D @endlink aus dem die Daten gelesen werden</em>
	#
	#  @sa QBaseWidget::setFromObject()
	#
	#  @todo Tests using ReadXML, Tests of all kind<br><em>Tests mit ReadXML, Test mit allen Arten</em>
	def setFromObject(self, pProfile):
		#Prüfen ob das Profil auch wirklich eins ist
		if (pProfile.__class__ != profile2D):
			return
		
		#Neues Profil übernehmen
		self._pOutput = pProfile

		#Aus XML Datei lesen?
		if self._pOutput.readXml == 1:
			#Aus XML-Datei lesen
			self.CBReadXML.setChecked(True)
			#XML-Werte eintrage
			self.LEBladeRowNumber.setText(str(pProfile.bladeRowNumber))
			self.LEProfileNumber.setText(str(pProfile.profileNumber))
		else:
			#Auswahl übernehmen
			self.CBCamberLine.setCurrentIndex(pProfile.flagCamberLine)
			#-1 da no profile rausgefallen ist
			self.CBThickDistr.setCurrentIndex(pProfile.flagThickDistr-1)
			#Profil übernehmen
			self.LEProfileResolution.setText(str(int(pProfile.profileResolution)))
			#Auslesen
			self.LEChordLength.setText(str(pProfile.chordLength))
			#Gamma setzen
			self.LEGamma.setText(str(degrees(pProfile.gamma)))

		#Andere Combobox-Schalter aktivieren bevor der Rest angezeigt wird
		#Index herausziehen
		iCLIndex = self.CBCamberLine.currentIndex()
		iTDIndex = self.CBThickDistr.currentIndex()+1

		#WeightCombination anzeigen?
		if (iCLIndex > 0) or (iTDIndex == 1) or (iTDIndex == 2):
			self.CBWeightCombination.setCurrentIndex(pProfile.weightCombination)

		#PolyCamberlineDegree anzeigen?
		if(iCLIndex == 1):
			self.CBPolyCamberlineDegree.setCurrentIndex(self.CBPolyCamberlineDegree.findText(str(int(pProfile.polyCamberLineDegree))))

		#PolyMethod anzeigen?
		if not ((iCLIndex == 0) and (iTDIndex == 3)):
			self.CBPolyMethod.setCurrentIndex(pProfile.approxPolyCoeff)

		#Dialog aktualisieren und Schalter aktivieren
		self.optionChanged()

		#Camberline Resolution sichtbar?
		if self.__bShowCamberResolution:
			#Hinzufügen
			self.LECamberResolution.setText(str(int(self._pOutput.camberResolution)))

		#LeadingEdge Radius sichtbar?
		if self.__bShowLeadingEdgeRadius:
			#Hinzufügen
			self.LELeadingEdgeRadius.setText(str(pProfile.leadingEdgeRadius))

		#MaxThickness anzeigen?
		if self.__bShowMaxThickness:
			#Hinzufügen
			self.LEMaxThickness.setText(str(pProfile.maxThickness))

		#ThickMethod anzeigen
		if self.__bShowThickMethod:
			#Hinzufügen
			self.CBThickMethod.setCurrentIndex(pProfile.thickMeth)
			
		#MaxCamberPosition angezeigt?
		if self.__bShowMaxCamberPosition:
			#Hinzufügen
			self.LEMaxCambPosition.setText(str(pProfile.maxCamberPosition))

		#Wichtungskombination angezeigt?
		if self.__bShowWeightCondition:
			#Hinzufügen
			self.CBWeightCondition.setCurrentIndex(pProfile.weightCondition)

		#MaxIter und Error angezeigt?
		if self.__bShowErrorMax:
			#Error und Max-Wert übernehmen
			self.LEError.setText(str(pProfile.error))
			self.LEMaxIterations.setText(str(int(pProfile.maxIter)))

		#Winkel angezeigt?
		if self.__bShowAngles:
			#Winkel übernehmen
			self.LEAngleOfAttack.setText(str(pProfile.angleOfAttack))
			self.LEOutflowAngle.setText(str(pProfile.outflowAngle))

		#Ca0 angezeigt?
		if self.__bShowCa0:
			#Hinzufügen
			self.LECa0.setText(str(pProfile.Ca0))

		#Camberline Nurbs per Hand eingeben?
		if self.__bShowCamberLineNurbs:
			self.CICamberLineNurbs.setCoordinatesFromNumpy(pProfile.controlPointsCamb)

		#ThicknessNurbs von Hand eingegeben?
		if self.__bShowThicknessNurbs:
			#Daten als Numpy holen
			self.CIThicknessNurbs.setCoordinatesFromNumpy(pProfile.controlPointsThick)
			#Approximierter NurbsNasenradius holen
			self.LEApproxNurbsNoseRadius.setText(str(pProfile.approxNurbsNoseRad))

		#Polynom eingegeben?
		if self.__bShowPoly:
			#directe Methode gewählt?
			if self._pOutput.approxPolyCoeff == 0:
				#Mit werten füllen
				#self.LEa0.setText(str(pProfile.polyThickRBs[0][0]))
				#self.LEa1.setText(str(pProfile.polyThickRBs[0][1]))
				#self.LEa2.setText(str(pProfile.polyThickRBs[0][2]))
				#self.LEa3.setText(str(pProfile.polyThickRBs[0][3]))
				#self.LEa4.setText(str(pProfile.polyThickRBs[0][4]))
				#self.LEp0.setText(str(pProfile.polyThickRBs[0][5]))
				#self.LEp1.setText(str(pProfile.polyThickRBs[0][6]))
				#self.LEp2.setText(str(pProfile.polyThickRBs[0][7]))
				#self.LEp3.setText(str(pProfile.polyThickRBs[0][8]))
				self.LEa0.setText(str(pProfile.polyThickRBs[0]))
				self.LEa1.setText(str(pProfile.polyThickRBs[1]))
				self.LEa2.setText(str(pProfile.polyThickRBs[2]))
				self.LEa3.setText(str(pProfile.polyThickRBs[3]))
				self.LEa4.setText(str(pProfile.polyThickRBs[4]))
				self.LEp0.setText(str(pProfile.polyThickRBs[5]))
				self.LEp1.setText(str(pProfile.polyThickRBs[6]))
				self.LEp2.setText(str(pProfile.polyThickRBs[7]))
				self.LEp3.setText(str(pProfile.polyThickRBs[8]))






	## @brief Performs the calculation of @link src::profile2D::profile2D profile2D @endlink object<br><em>Führt die Berechnung für das @link src::profile2D::profile2D profile2D @endlink Objekt aus</em>
	#
	#  This function is attached to QBaseWidget::_OnPerformCalc to use the provided automatisms.
	#  It is in charge of single calculation and preview. At first a status dialog is created showing the progress of the calculation.
	#  Afterwards the calculation is passed to the @link src::profile2D::profile2D profile2D @endlink object. The results of the calculation can be used to display the profile in QPlotWidget.
	#  This function is attached to Calculate-Button in QtDesigner.
	#  <br><em>Diese Funktion wird an QBaseWidget::_OnPerformCalc gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie ist für eine Einzelberechnung/Vorschau zuständig. Zu erst wird ein Statusdialog erstellt, der den Fortschritt der Berechnung anzeigt.
	#  Im Anschluß wird die Berechnung an das @link src::profile2D::profile2D profile2D @endlink Objekt übergeben. Die Ergebnisse der Berechnung können dann zur Darstellung des profiles
	#  im QPlotWidget benutzt werden. Diese Funktion ist im QtDesigner mit dem Berechnen-Button verbunden.</em>
	#
	#  @sa QBaseWidget::_OnPerformCalc, _pOutput, QStatusDialog, QPlotWidget
	def OnPerformCalc(self):
		#Kontrolle auf Nötigkeit und auslesen der Daten wird von QBaseWidget.performCalc(...) durchgeführt
		#Statusdialog anzeigen
		pStatusDialog = QStatusDialog()
		#Dialog Modal anzeigen
		pStatusDialog.setModal(True)
		#Dialog anzeigen
		pStatusDialog.show()

		try:
			#Werte berechnen
			self._pOutput.callfunctions()
			#Anzeigen
			self.plot()
		except Exception, eException:
			print "Fehler", str(eException)

		#Dialog freigeben
		pStatusDialog.outputFinished()





	## @brief Plots the result of the calculation of @link src::profile2D::profile2D profile2D @endlink object<br><em>Zeichnet das Ergebnis der Berechnung des @link src::profile2D::profile2D profile2D @endlink Objekts</em>
	#
	#  This function shows the QPlotWidget and passes, if neccessary, the result of the calculation of @link src::profile2D::profile2D profile2D @endlink object.
	#  It is attached to Plot-Button in QtDesigner.
	#  <br><em>Diese Funktion zeigt das QPlotWidget an und übergibt, falls vorhanden, das Ergebnis der Berechnung des @link src::profile2D::profile2D profile2D @endlink Objekts.
	#  Sie ist im QtDesigner mit dem Zeichnen-Button verbunden.</em>
	#
	#  @sa OnPerformCalc(), _pOutput, QPlotWidget
	def plot(self):
		#keine Daten vorhanden?
		if self._pOutput == None:
			return

		#Keine Zeichendaten vorhanden?
		if (self._pOutput.gesarray == []):
			return

		self.Plot.beginDrawing()

		self.Plot.plot(self._pOutput.gesarray[0],self._pOutput.gesarray[1])	#plot camberline

		if len(self._pOutput.gesarray) > 2:
			self.Plot.plot(self._pOutput.gesarray[2],self._pOutput.gesarray[3])	#plot pressure site
			self.Plot.plot(self._pOutput.gesarray[4],self._pOutput.gesarray[5])	#plot suction site

			llowerProfile=zeros([2,len(self._pOutput.gesarray[2])], 'd')
			lupperProfile=zeros([2,len(self._pOutput.gesarray[4])], 'd')
			llowerProfile[0,:]=self._pOutput.gesarray[2]
			llowerProfile[1,:]=self._pOutput.gesarray[3]
			lupperProfile[0,:]=self._pOutput.gesarray[4]
			lupperProfile[1,:]=self._pOutput.gesarray[5]

		#Figur zeichnen
		self.Plot.endDrawing()
		
		





	## @brief Updates the layout of the widget according to the choosen Settings<br><em>Aktualisiert das Layout des Widgets abhängig von den gewählten Einstellungen</em>
	#
	# At first this function resets the internal display flags and determines the state of all contained widgets on the basis of striking widgets.
	# This function is attached to always visible and striking contained widgets. It is important for the visibility of contained wigets and the appearance of
	# @link src::profile2D::profile2D profile2D @endlink object (e.g. as circArc, Nurbs-based, polynomial, ...).
	# <br><em>Die Funktion setzt zuerst die internen Anzeigeflags zurück und ermittelt dann anhand markanter Steuerelemente den Zustand
	# aller anderen Steuerelemente. Diese Funktion ist im QtDesigner an markante immer sichtbare Steuerelemente gebunden.
	# Sie ist entscheidend für die Sichtbarkeit der Steuerelemente und das Auftreten des @link src::profile2D::profile2D profile2D @endlink Objekts (zBsp als Kreisbogen, auf Nurbsbasis, Polynomial, ...).</em>
	#
	# @sa QBaseWidget::optionChanged(), __resetFlags()
	def optionChanged(self):
		#Standardmäßig garnichts zeigen
		self.__resetFlags()

		#Aus XML-Datei auslesen?
		if self.CBReadXML.isChecked():
			self.__bShowReadXML = True
			self.__bShowErrorMax = True
			#Den Rest können wir uns dann sparen
		else:
			#Nicht aus der XML-Datei lesen

			#Profile anzeigen
			self.__bShowResolution = True

			#Index herausziehen
			iCLIndex = self.CBCamberLine.currentIndex()
			iTDIndex = self.CBThickDistr.currentIndex()+1

			#Camberline größer 0(=polynomial, Nurbs, NACA)?
			#Bei Null sind Thickness 1(=polynomial) und 2(=Nurbs) noch zugelassen
			if (iCLIndex > 0) or (iTDIndex == 1) or (iTDIndex == 2):
				#Weights anzeigen
				self.__bShowWeights = True
				#Bei Index 1 und 4 kann WeightCondition eingesetzt werten
				if (self.CBWeightCombination.currentIndex() == 2) or (self.CBWeightCombination.currentIndex() == 3):
					self.__bShowWeightCondition = True

			#ThickDistr 4(=DoubleCircArc) nur bei Camberline 0(=CircArc) möglich
			if (iCLIndex == 0):
				#Eintrag nicht vorhanden obwohl er vorhanden sein sollte?
				if self.CBThickDistr.findText(self.tr("double circArc Profile", "Text des 3. Eintrags in CBThickDistr")) == -1:
					#Eintrag hinzufügen
					self.CBThickDistr.addItem(self.tr("double circArc Profile", "Text des 3. Eintrags in CBThickDistr"))
			else:
				iIndex = self.CBThickDistr.findText(self.tr("double circArc Profile", "Text des 3. Eintrags in CBThickDistr"))
				#Eintrag vorhanden obwohl er nicht da sein darf?
				if iIndex != -1:
					#Ist der Eintrag ausgewählt?
					if iIndex == iTDIndex:
						#AuswahlIndex auf 0 setzen
						self.CBThickDistr.setCurrentIndex(0)

					#Eintrag löschen
					self.CBThickDistr.removeItem(iIndex)


			#PolynomGrad nur bei Camberline 1(=polynomial)
			if(iCLIndex == 1):
				self.__bShowPolynom = True
				#MaxCamberPosition nur wenn PolyReg auf 2(=4x)
				if(self.CBPolyCamberlineDegree.currentIndex() == 2):
					self.__bShowMaxCamberPosition = True

			#Camberline per Nurbs eingeben?
			if (iCLIndex == 2):
				self.__bShowCamberLineNurbs = True
				self.__bShowErrorMax = True

			#Ca0 nur zeigen, wenn Camberline 3(=NACA)
			if (iCLIndex == 3):
				self.__bShowCa0 = True

			#Winkel immer anzeigen
			if (True):
				self.__bShowAngles = True

			#Thickness also Polynom(=1)?
			if (iTDIndex == 1):
				self.__bShowThickMethod = True

			#Thicknes per Nurbs(=2) eingeben?
			if (iTDIndex == 2):
				self.__bShowThicknessNurbs = True
				self.__bShowErrorMax = True

			#Thickness als doubleCircArd(=3)?
			if(iTDIndex == 3):
				self.__bShowLeadingEdgeRadius = True

			#CamberResolution nur im Fall von 0(=CircArc)/0(=no profile) und 0(=CircArc)/3(=doubleCircArc) anzeigen
			if (iCLIndex == 0) and ((iTDIndex == 0) or (iTDIndex == 3)):
				self.__bShowCamberResolution = True

			#Falls Thickness 1(=poly) oder 3(=doubleCircArc)
			if ((iTDIndex == 1) and (iCLIndex != 1)) or (iTDIndex == 3):
				self.__bShowMaxThickness = True

			#Für alle außer DoublecircArc
			if not ((iCLIndex == 0) and (iTDIndex == 3)):
				#Polynom anzeigen
				self.__bShowPoly = True

				#Approximiert anzeigen?
				if self.CBPolyMethod.currentIndex() == 1:
					self.__bShowApproxPoly = True

				#Direkt anzeigen?
				if self.CBPolyMethod.currentIndex() == 0:
					self.__bShowDirectPoly = True

		#Prüfen ob zusätzliche Parameter angezeigt werden müßen
		lAdditionalContains = (self.__bShowCa0, self.__bShowLeadingEdgeRadius, self.__bShowMaxThickness, self.__bShowThickMethod, self.__bShowErrorMax)
		#Durch alle Parameter gehen
		for bTrigger in lAdditionalContains:
			#Wenn nur einer angezeigt werden muß -> AdditionalGruppenbox anzeigen
			if bTrigger:
				self.__bShowAdditional = True
				#Den Rest sparen wir uns
				break

		#Debugoptionen aktiviert?
		if (self.CBDebug.isChecked()):
			#Optionen aktiviert
			self.__bShowDebug = True

		#Optionen weitergeben
		#Gruppenboxen
		self.GBWeights.setVisible(self.__bShowWeights)
		self.CICamberLineNurbs.setVisible(self.__bShowCamberLineNurbs)
		self.CIThicknessNurbs.setVisible(self.__bShowThicknessNurbs)
		self.GBAngles.setVisible(self.__bShowAngles)
		self.GBPolynom.setVisible(self.__bShowPolynom)
		self.GBAdditional.setVisible(self.__bShowAdditional)
		self.GBReadXML.setVisible(self.__bShowReadXML)
		self.GBResolution.setVisible(self.__bShowResolution)

		#Edits und deren Labels
		self.LCamberResolution.setVisible(self.__bShowCamberResolution)
		self.LECamberResolution.setVisible(self.__bShowCamberResolution)

		self.LCa0.setVisible(self.__bShowCa0)
		self.LECa0.setVisible(self.__bShowCa0)

		self.LMaxCambPosition.setVisible(self.__bShowMaxCamberPosition)
		self.LEMaxCambPosition.setVisible(self.__bShowMaxCamberPosition)

		self.LWeightCondition.setVisible(self.__bShowWeightCondition)
		self.CBWeightCondition.setVisible(self.__bShowWeightCondition)

		self.LLeadingEdgeRadius.setVisible(self.__bShowLeadingEdgeRadius)
		self.LELeadingEdgeRadius.setVisible(self.__bShowLeadingEdgeRadius)

		self.LMaxThickness.setVisible(self.__bShowMaxThickness)
		self.LEMaxThickness.setVisible(self.__bShowMaxThickness)

		self.LThickMethod.setVisible(self.__bShowThickMethod)
		self.CBThickMethod.setVisible(self.__bShowThickMethod)

		self.LMaxIterations.setVisible(self.__bShowErrorMax)
		self.LEMaxIterations.setVisible(self.__bShowErrorMax)

		self.LError.setVisible(self.__bShowErrorMax)
		self.LEError.setVisible(self.__bShowErrorMax)

		self.__setWidgetsVisible(self.__bShowApproxPoly, [(0, 'KrummungsRadius'),(0, 'b2'),(1, 'Bound')])
		self.__setWidgetsVisible(self.__bShowDirectPoly, [(0, 'a0'), (0, 'a1'), (0, 'a2'), (0, 'a3'), (0, 'a4')])

		self.LApproxNurbsNoseRadius.setVisible(self.__bShowThicknessNurbs)
		self.LEApproxNurbsNoseRadius.setVisible(self.__bShowThicknessNurbs)

		self.GBPoly.setVisible(self.__bShowPoly and self.CBShowPolyKoeffizient.isChecked())





	## @brief Sets the visibility of certain widgets<br><em>Setzt die Sichtbarkeit bestimmter Steuerelemente</em>
	#
	#  Auxiliary function for comfortable setting of visibility of some widgets, which is called in optionChanged(). Effects the widget and its descriptive label.
	#  <br><em>Hilfsfunktion die für das bequeme setzen der Sichtbarkeit einiger Steuerelemente in optionChanged() aufgerufen wird. Wirkt sich dabei auf das
	#  Steuerelement und dessen beschreibendes Label aus.</em>
	#
	#  @param bVisible - True when visible, else false<br><em>True wenn sichtbar, sonst False</em>
	#  @param lEntrys - list of Widgetnames to set with format: [ iWidgetFlag, strWidgetName, iWidgetFlag, strWidgetName, ... ] <br>
	#		    where iWidgetFlag can be 0 - for LineEdits or 1 - for ComboBoxes<br>
	#			strWidgetName is the Name of the Widget without DescriptionTags
	#		    e.g. [0, 'Error', 1, 'Bound', ...] will effect LError-LineEdit, LError-Label, CBBound-ComboBox and LBound-Label and so on
	#		    <br><em>Liste der Steuerelementnamen die gesetzt werden sollen mit dem Format: [ iWidgetFlag, strWidgetName, iWidgetFlag, strWidgetName, ... ] <br>
	#		    wobei iWidgetFlag 0 - für LineEdits oder 1 - für ComboBoxen sein kann<br>
	#		    strWidgetName ist der Name des Widgets ohne Typtag<br>
	#		    zBsp: [0, 'Error', 1, 'Bound', ...] wirkt sich auf LError-LineEdit, LError-Label, CBBound-ComboBox und LBound-Label usw aus</em>
	#profile2D-object (e.g. as circArc, Nurbs-based, polynomial, ...)
	# @sa optionChanged()
	def __setWidgetsVisible(self, bVisible, lEntrys):
		#Durch alle Einträge gehen
		for (iFlag, strName) in lEntrys:
			#An Flag unterscheiden
			if iFlag == 0:
				strWidgetName = 'LE%s' % strName
			elif iFlag == 1:
				strWidgetName = 'CB%s' % strName

			#Widget holen und einstellen
			getattr(self, strWidgetName).setVisible(bVisible)
			#Label ebenfalls
			getattr(self, 'L'+strName).setVisible(bVisible)




	## @brief Activates actions connected to tabchange<br><em>Aktiviert mit Tabänderung verbundene Aktionen</em>
	#
	#  When the tab is changed to the widget with graphical output, the result is recalculated and redrawn.
	#  <br><em>Sollte auf das Widget mit der grafischen Ausgabe gewechselt worden sein, wird das Ergebnis neu berechnet und anschließend gezeichnet</em>
	#
	#  @param iIndex - Index of the tab choosen<br><em>Index des ausgewählten Tabs</em>´
	#
	# @sa performCalc(), plot()
	def tabChanged(self, iIndex):
		#2. Tab ausgewählt? -> Plottab
		if(iIndex == 1):
			#Neu berechnen, falls sich was geändert hat
			self.performCalc()
			self.plot()





	## @brief Renames all labels as the corresponding language<br><em>Bennent alle Label in die entsprechende Sprache um</em>
	#
	#  Reimplemented from QBaseWidget::retranslateStrings(). Used for localisation purpose. The texts inputed in QtDesigner will be overwritten.
	#  <br><em>Diese Funktion ist reimplementiert aus QBaseWidget::retranslateStrings(). Wird zur Lokalisation verwendet. Die im QtDesigner eingetragenen Texte für
	#  Steuerelemente werden überschrieben.</em>
	#
	#  @param bPassDown - tells whether to pass down the retranslation to underlying Widgets<br><em>Gibt an, ob die Übersetzung an tiefer liegende Widgets übergeben werden soll.</em>
	def retranslateStrings(self, bPassDown = True):
		#Gruppenboxen
		self.GBOptions.setTitle(self.tr("Options", "Titel der Optionengruppenbox bei Sprachenwechsel"))
		self.GBReadXML.setTitle(self.tr("XML Options", "Titel der XML-Gruppenbox bei Sprachenwechsel"))
		self.GBDebug.setTitle(self.tr("Debug", "Titel der Debuggruppenbox bei Sprachenwechsel"))
		self.GBWeights.setTitle(self.tr("Weights", "Titel der Weightsgruppenbox bei Sprachenwechsel"))
		self.GBPolynom.setTitle(self.tr("Polynom", "Titel der Polynomgruppenbox bei Sprachenwechsel"))
		self.GBResolution.setTitle(self.tr("Resolution", "Titel der Resolutiongruppenbox bei Sprachenwechsel"))
		self.GBAngles.setTitle(self.tr("Angels", "Titel der Angelsgruppenbox bei Sprachenwechsel"))
		self.GBAdditional.setTitle(self.tr("Additional", "Titel der Additionalgruppenbox bei Sprachenwechsel"))
		self.GBPoly.setTitle(self.tr("Polynom Koeffizient", "Titel der Polynomkoeffizientengruppenbox bei Sprachenwechsel"))

		#Label
		self.LCamberLine.setText(self.tr("Camber Line", "Name des Camberline-Flags bei Sprachenwechsel in Optionsgruppenbox"))
		self.LThickDistr.setText(self.tr("Thick Distribution", "Name des Thickness-Flags bei Sprachenwechsel in Optionsgruppenbox"))

		self.LBladeRowNumber.setText(self.tr("Blade Row Number", "Text eines Labels in XMLgruppenbox bei Sprachenwechsel"))
		self.LProfileNumber.setText(self.tr("Profile Number", "Text eines Labels in XMLgruppenbox bei Sprachenwechsel"))

		self.LMagFactor.setText(self.tr("Mag Factor", "Text eines Labels in Debuggruppenbox bei Sprachenwechsel"))
		self.LDash.setText(self.tr("Dash", "Text eines Labels in Debuggruppenbox bei Sprachenwechsel"))
		self.LDebugLevel.setText(self.tr("Debug Level", "Text eines Labels in Debuggruppenbox bei Sprachenwechsel"))

		self.LWeightCombination.setText(self.tr("Weight Combination", "Text eines Labels in Weightgruppenbox bei Sprachenwechsel"))
		self.LWeightCondition.setText(self.tr("Weight Condition", "Text eines Labels in Weightgruppenbox bei Sprachenwechsel"))

		self.LPolyCamberlineDegree.setText(self.tr("Poly Camberline Degree", "Text eines Labels in Polynomgruppenbox bei Sprachenwechsel"))
		self.LMaxCambPosition.setText(self.tr("Max Camber Position", "Text eines Labels in Polynomgruppenbox bei Sprachenwechsel"))

		self.LProfileResolution.setText(self.tr("Profile Resolution", "Text eines Labels in Resolutiongruppenbox bei Sprachenwechsel"))
		self.LCamberResolution.setText(self.tr("Camber Resolution", "Text eines Labels in Resolutiongruppenbox bei Sprachenwechsel"))

		self.LAngleOfAttack.setText(self.tr("Angle of Attack", "Text eines Labels in Angelsgruppenbox bei Sprachenwechsel"))
		self.LOutflowAngle.setText(self.tr("Outflow Angle", "Text eines Labels in Angelsgruppenbox bei Sprachenwechsel"))

		self.LCa0.setText(self.tr("Ca0", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LLeadingEdgeRadius.setText(self.tr("Lead-/Trailing Egde Radius", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LMaxThickness.setText(self.tr("Max Thickness", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LThickMethod.setText(self.tr("Thick Method", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LError.setText(self.tr("Error", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LMaxIterations.setText(self.tr("Max Iterations", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LChordLength.setText(self.tr("Chord Length", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LGamma.setText(self.tr("Gamma", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))
		self.LApproxNurbsNoseRadius.setText(self.tr("Aprroximated Nurbsnoseradius", "Text eines Labels in Additionalgruppenbox bei Sprachenwechsel"))

		self.LMethod.setText(self.tr("Methode", "Text eines Labels in Polynomkoeffizientengruppenbox bei Sprachenwechsel"))
		self.LBound.setText(self.tr("Bound", "Text eines Labels in Polynomkoeffizientengruppenbox bei Sprachenwechsel"))
		self.LKrummungsRadius.setText(self.tr("KrummungsRadius", "Text eines Labels in Polynomkoeffizientengruppenbox bei Sprachenwechsel"))

		#Checkboxen
		self.CBReadXML.setText(self.tr("Read XML", "Text einer Checkbox in Optionsgruppenbox"))
		self.CBDebug.setText(self.tr("Debug", "Text einer Checkbox in Optionsgruppenbox"))
		self.CBAppOverride.setText(self.tr("an XML anhängen?", "Text einer Checkbox in Optionsgruppenbox"))
		self.CBShowPolyKoeffizient.setText(self.tr("Polynom Koeffizient anzeigen", "Text einer Checkbox in Optionsgruppenbox"))

		self.CBControlPoints.setText(self.tr("Control Points", "Text einer Checkbox in Debuggruppenbox"))

		#ComboBoxen
		#CBCamberLine
		lLabels = (
			self.tr("circ Arc", "Text des 1. Eintrags in CBCamberline"),
			self.tr("polynomial", "Text des 2. Eintrags in CBCamberline"),
			self.tr("Nurbs Camber Line", "Text des 3. Eintrags in CBCamberline"),
			self.tr("NACA 65", "Text des 4. Eintrags in CBCamberline")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBCamberLine.count()):
			#Namen übernehmen
			self.CBCamberLine.setItemText(iCurrent, lLabels[iCurrent])

		#CBThickDistr
		lLabels = (
			self.tr("polynomial", "Text des 1. Eintrags in CBThickDistr"),
			self.tr("Nurbs", "Text des 2. Eintrags in CBThickDistr"),
   			self.tr("double circArc Profile", "Text des 3. Eintrags in CBThickDistr")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBThickDistr.count()):
			#Namen übernehmen
			self.CBThickDistr.setItemText(iCurrent, lLabels[iCurrent])

		#CBWeightCombination
		lLabels = (
			self.tr("curvature of polynom", "Text des 1. Eintrags in CBWeightCombination"),
			self.tr("curvature of polynomial part", "Text des 2. Eintrags in CBWeightCombination"),
			self.tr("div weight funct", "Text des 3. Eintrags in CBWeightCombination"),
			self.tr("combination of polynomial & other weight funct", "Text des 4. Eintrags in CBWeightCombination")
		)
		
		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBWeightCombination.count()):
			#Namen übernehmen
			self.CBWeightCombination.setItemText(iCurrent, lLabels[iCurrent])

		#CBWeightCondition
		lLabels = (
			self.tr("parabel", "Text des 1. Eintrags in CBWeightCondition"),
			self.tr("kub. parabel", "Text des 2. Eintrags in CBWeightCondition"),
			self.tr("tangens", "Text des 3. Eintrags in CBWeightCondition"),
			self.tr("arcsinus", "Text des 4. Eintrags in CBWeightCondition"),
			self.tr("hyperbel", "Text des 5. Eintrags in CBWeightCondition"),
			self.tr("gerade", "Text des 6. Eintrags in CBWeightCondition")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBWeightCondition.count()):
			#Namen übernehmen
			self.CBWeightCondition.setItemText(iCurrent, lLabels[iCurrent])

		#CBThickMethod
		lLabels = (
			self.tr("NACA assumed", "Text des 1. Eintrags in CBThickMethod"),
			self.tr("polynomial", "Text des 2. Eintrags in CBThickMethod"),
			self.tr("elliptic", "Text des 3. Eintrags in CBThickMethod")

		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBThickMethod.count()):
			#Namen übernehmen
			self.CBThickMethod.setItemText(iCurrent, lLabels[iCurrent])

		#CBPolyMethod
		lLabels = (
			self.tr("direct", "Text des 1. Eintrags in CBPolyMethod"),
			self.tr("approximated", "Text des 2. Eintrags in CBPolyMethod")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBPolyMethod.count()):
			#Namen übernehmen
			self.CBPolyMethod.setItemText(iCurrent, lLabels[iCurrent])

		#Buttons
		#self.BCalculation.setText(self.tr("&Calculate", "Aufschrift des Berechnen-Buttons"))

		#CoordinatesInputWidgets
		self.CICamberLineNurbs.setText(self.tr('Camberline Nurbs', 'Titel des CoordinatenInput der CamberlineNurbs'))
		self.CIThicknessNurbs.setText(self.tr('Thickness Nurbs', 'Titel des CoordinatenInput der Thickness Nurbs'))
		self.CICamberLineNurbs.retranslateStrings()
		self.CIThicknessNurbs.retranslateStrings()


	#def customMousePress(self, pEvent):
		##Taste prüfen
		#if(pEvent.buttons() != Qt.RightButton):
			#return
			
		#self.iMouseX = pEvent.globalX()
		#self.iMouseY = pEvent.globalY()
		#self.bScrolling = True
		#self.oldCursor = self.cursor()
		#self.setCursor(Qt.SizeAllCursor)
		

	#def customMouseMove(self, pEvent):
		#if(not self.bScrolling):
			#pEvent.ignore()

		##Position berechnen
		#self.scrollArea.verticalScrollBar().setValue(self.scrollArea.verticalScrollBar().value()-(pEvent.globalY()-self.iMouseY))
		#self.scrollArea.horizontalScrollBar().setValue(self.scrollArea.horizontalScrollBar().value()-(pEvent.globalX()-self.iMouseX))

		#self.iMouseX = pEvent.globalX()
		#self.iMouseY = pEvent.globalY()
		

	#def customMouseRelease(self, pEvent):
		#if(not self.bScrolling):
			#return
			
		#self.bScrolling = False
		#self.setCursor(self.oldCursor)
		



		
## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell Entfernen
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pProfile2DWidget = QProfile2DWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())	
	
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pProfile2DWidget)
	pProfile2DWidget.scrollArea.scrollContentsBy(10,10)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	sys.exit(pApp.exec_())

if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowGetSetTest'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pProfile2DWidget = QProfile2DWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pProfile2DWidget)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	pApp.exec_()
	#Backup speichern
	pProfile2D = pProfile2DWidget.getpProfile2D()
	#Profil vorhanden?
	if pProfile2D == None:
		sys.exit(0)

	#Ansonsten Nochmal das ganze nun aber laden
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pProfile2DWidget2 = QProfile2DWidget()
	pProfile2DWidget2.setpProfile2D(pProfile2D)
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pProfile2DWidget2)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	pApp.exec_()

if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowCopyTest'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pProfile2DWidget = QProfile2DWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pProfile2DWidget)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	pApp.exec_()
	pDlg2 = QDialog()
	#Widget erstellen
	pProfile2DWidget2 = QProfile2DWidget(pDlg2)
	pProfile2DWidget2.copyFrom(pProfile2DWidget)
	#Nochmal das ganze nun aber laden
	#Dummy Dialog zur Anzeige des Widgets erstellen

	#Layout für den Dialog erstellen
	pDlg2.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg2.layout().addWidget(pProfile2DWidget2)
	#Dialog anzeigen
	pDlg2.show()
	#Ausführen
	pApp.exec_()