#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Steve Walter	                           *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Steve Walter
# @date 08.09.2009
 
import sys
#Importpfad für übergeordnete Verzeichnisse setzen
sys.path.append('..')
#System-Werte zum Laden entfernen
#TODO: Entfernen, wenn alle Tests abgeschlossen
strTemp = sys.argv
sys.argv = []

from turboMachine import turboMachine, dataStructure
from numpy import zeros
from PyQt4.QtGui import QWidget, QIntValidator, QFileDialog, QApplication, QDialog, QVBoxLayout
from PyQt4.QtCore import SIGNAL
from Ui_TurboMachineWidget import Ui_TurboMachineWidget as TurboMachineWidget
from QStatusDialog import QStatusDialog
from QBaseWidget import QBaseWidget
from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB

#System-Werte wiederherstellen
#TODO: Entfernen, wenn alle Tests abgeschlossen
sys.argv = strTemp

## @brief Inputwidget for @link src::turboMachine::turboMachine turboMachine @endlink class<br><em>Eingabewidget für die @link src::turboMachine::turboMachine turboMachine @endlink Klasse</em>
#
# This widget holds all data including QBladeRowWidget s, which again contain QProfile2DWidget s, neccessary for the calculated of @link src::turboMachine::turboMachine turboMachine @endlink. It is able to do
# stand-alone @link src::turboMachine::turboMachine turboMachine @endlink calculations. Therefore it holds its own array of QBladeRowWidget s. It is the heart of the calculation of a full turboMachine and
# for this reason is called by the baseGUI ( QBladeDesigner ).<br>
# The layout of this widget in QtDesignerform is contained in ./UI/TurboMachineWidget.ui, which can be converted to UI_TurboMachineWidget by ./CreateGUI.sh script using pyuic4
# and loaded afterwards.
# <br><em>Dieses Widget nimmt alle für die Berechnung von @link src::turboMachine::turboMachine turboMachine @endlink nötigen Daten inklusive der QBladeRowWidget s, welche wiederum die QProfile2DWidget s
# enthalten, auf. Es ist im Stande selbständige @link src::turboMachine::turboMachine turboMachine @endlink-Berechnungen durchzuführen. Dafür bestizt es seinen eigenen Array von QBladeRowWidget s.
# Es bildet das Herzstück zur Berechnung einer kompletten TurboMachine und wird von der BasisGUI ( QBladeDesigner ) für die Berechnung angesprochen.<br>
# Das Layout dieses Widgets befindet sich in der QtDesignerform in ./UI/TurboMachineWidget.ui, welche vom ./CreateGUI.sh Script mittels pyuic4 in UI_TurboMachineWidget konvertiert
# und dann geladen wird.
# </em>
#
# @sa @link src::turboMachine::turboMachine turboMachine @endlink, QBaseWidget, QBladeRowWidget
class QTurboMachineWidget(QWidget, TurboMachineWidget, QBaseWidget):
	
	## @brief Constructor of QTurboMachineWidget<br><em>Konstruktor des QTurboMachineWidget</em>
	#
	# Initializes and configurates underlying widgets. Attaches to QBaseWidget provided interfaces(#_OnPerformCalc, #_OnCopyFrom, #_OnGetOutput,
	# #_OnSetCalculated, #_OnSetSaved). Also creates the list of bladeRows.
	# <br><em>Initialisiert und konfiguriert grundlegende Widgets. Hängt die von QBaseWidget bereitgestellten Schnittstellen ein(#_OnPerformCalc, #_OnCopyFrom, #_OnGetOutput,
	# #_OnSetCalculated, #_OnSetSaved). Erstellt die Bladerowliste.
	# </em>
	#
	# @param pParent - Parentwidget, the inputwidget belongs to<br><em>Elternwidget zu dem das Eingabewidget gehört</em>
	def __init__(self, pParent=None):
		QWidget.__init__(self, pParent)
		QBaseWidget.__init__(self, pParent)
		self.setupUi(self)
		self.hookChangeNotify()

		#Namen setzen
		self.retranslateStrings(False)

		#Int-Validator für das Zahlenfeld
		self.LEAnzBladeRows.setValidator(QIntValidator(self))

		#Events für die Grafikanzeige
		#Tab Index so setzen, dass das Grafikfenster angezeigt wird
		self.tabWidget.setCurrentIndex(1)
		#Widget anzeigen, sonst können die Grafiktreiber nicht initialisiert werden
		self.show()
		#Driver initialisieren
		self.Plot.InitDriver()
		#Ansicht auf Isometrisch setzen
		self.Plot._display.View_Iso()
		#Passform
		self.Plot._display.FitAll()
		#Testmodus wechseln
		self.Plot._display.Test()
		#Widget wieder ausblenden
		self.hide()
		#Tab auf Ursprungstab zurücksetzen
		self.tabWidget.setCurrentIndex(0)

		#BladeRows
		
		## @brief List of QBladeRowWidget s<br><em>List der QBladeRowWidget s</em>
		#
		# This list is for calculation of the turboMachine and managment of QBladeRowWidget s. Also it is used to summarize the widgets to an hierarchy.
		# <br><em>Diese Liste dient  dazu die zur Berechnung benötigten QBladeRowWidget s zuverwalten. Des Weiteren wird sie dazu verwendet, dass die Widgets in einer
		# Hierarchie zusammengefasst werden.
		# </em>
		#
		# @sa QBladeDesigner::QBladeDesigner
		self.__lBladeRowWidgets = []


		#Fortschrittsfunktion

		## @brief Stepfunction for calculation<br><em>Schrittfunktionszeiger für Berechnung</em>
		#
		# To this pointer a function should be attached, which is called after each step of calculation. It is used for visualization of progress. In case of
		# QTurboMachineWidget, this pointer is attached to QStatusDialog::step().
		# <br><em>An diesen Zeiger sollte die Funktion gehangen werden, die aufgerufen wird, falls ein Berechnungsschritt vergangen ist. Sie dient
		# der Fortschrittsvisualisierung. Im Fall von QTurboMachineWidget wird dieser Zeiger auf QStatusDialog::step() gesetzt.
		# </em>
		#
		# @sa QStatusDialog::step(), OnPerformCalc()
		self.step = lambda: None

		#Funktionen binden
		
		## @brief Functionpointer from QBaseWidget::_OnPerformCalc<br><em>Funktionszeiger von QBaseWidget::_OnPerformCalc</em>
		#
		#  Attached to OnPerformCalc()
		# <br><em>Verbunden mit OnPerformCalc()
		# </em>
		#
		# @sa OnPerformCalc()
		self._OnPerformCalc = self.OnPerformCalc


		## @brief Functionpointer from QBaseWidget::_OnCopyFrom<br><em>Funktionszeiger von QBaseWidget::_OnCopyFrom</em>
		#
		#  Attached to OnCopyFrom()
		# <br><em>Verbunden mit OnCopyFrom()
		# </em>
		#
		# @sa OnCopyFrom()
		self._OnCopyFrom = self.OnCopyFrom


		## @brief Functionpointer from QBaseWidget::_OnGetOutput<br><em>Funktionszeiger von QBaseWidget::_OnGetOutput</em>
		#
		# Attached to OnGetOutput()
		# <br><em>Verbunden mit OnGetOutput()
		# </em>
		#
		# @sa OnGetOutput()
		self._OnGetOutput = self.OnGetOutput


		## @brief Functionpointer from QBaseWidget::_OnSetCalculated<br><em>Funktionszeiger von QBaseWidget::_OnSetCalculated</em>
		#
		# Attached to OnSetCalculated()
		# <br><em>Verbunden mit OnSetCalculated()
		# </em>
		#
		# @sa OnSetCalculated()
		self._OnSetCalculated = self.OnSetCalculated


		## @brief Functionpointer from QBaseWidget::_OnSetSaved<br><em>Funktionszeiger von QBaseWidget::_OnSetSaved</em>
		#
		# Attached to OnSetSaved()
		# <br><em>Verbunden mit OnSetSaved()
		# </em>
		#
		# @sa OnSetSaved()
		self._OnSetSaved = self.OnSetSaved





	## @brief Marks the QTurboMachineWidget as calculated<br><em>Markiert das QTurboMachineWidget als berechnet</em>
	#
	# This function is called by QBaseWidget::setCalculated(). It takes care of marking the QBladeRowWidget s in the list of bladeRows also as calculated.
	# <br><em>Diese Funktion wird durch QBaseWidget::setCalculated() aufgerufen. Sie sorgt dafür, dass auch die QBladeRowWidget s in der Bladerowliste als berechnet markiert werden.
	# </em>
	#
	# @sa QBaseWidget::setCalculated()
	def OnSetCalculated(self):
		#An BladeRows weiterreichen
		for pBladeRow in self.__lBladeRowWidgets:
			pBladeRow.setCalculated()




			
	## @brief Marks the QTurboMachineWidget as saved<br><em>Makriert die QTurboMachineWidget als gespeichert</em>
	#
	# This function is called by QBaseWidget::setSaved(). It takes care of marking the QBladeRowWidget in the list of bladeRows also as saved.
	# <br><em>Diese Funktion wird durch QBaseWidget::setSaved() aufgerufen. Sie sorgt dafür, dass auch die QBladeRowWidget in der Bladerowliste als berechnet markiert werden.
	# </em>
	#
	# @sa QBaseWidget::setSaved()
	def OnSetSaved(self):
		#An BladeRows weiterreichen
		for pBladeRow in self.__lBladeRowWidgets:
			pBladeRow.setSaved()




	## @brief Performs the calculation of @link src::turboMachine::turboMachine turboMachine @endlink object<br><em>Führt die Berechnung für das @link src::turboMachine::turboMachine turboMachine @endlink Objekt aus</em>
	#
	# This function is attached to QBaseWidget::_OnPerformCalc to use the provided automatisms.
	# It performs the calculation of the whole @link src::turboMachine::turboMachine turboMachine @endlink. At first a status dialog is created showing the progress of the calculation.
	# Afterwards the calculation is passed to the @link src::turboMachine::turboMachine turboMachine @endlink object. In the end the result ist written to the corresponding file in the working directory.
	# This function is attached to Calculate-Button in QtDesigner.
	# <br><em>Diese Funktion wird an QBaseWidget::_OnPerformCalc gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie führt die vollständige Berechnung der @link src::turboMachine::turboMachine turboMachine @endlink aus. Zu erst wird ein Statusdialog erstellt, der den Fortschritt der Berechnung anzeigt.
	#  Im Anschluß wird die Berechnung an das @link src::turboMachine::turboMachine turboMachine @endlink Objekt übergeben. Am Ende wird das Ergebnis in die entsprechenden Dateien im Arbeitsverzeichnis geschrieben.
	#  Diese Funktion ist im QtDesigner mit dem Berechnen-Button verbunden.</em>
	#
	# @sa QBaseWidget::_OnPerformCalc, _pOutput, QStatusDialog::QStatusDialog, @link src::bladeRow::bladeRow::writeVRML bladeRow::writeVRML @endlink
	def OnPerformCalc(self):
		#Statusdialog anzeigen
		pStatusDialog = QStatusDialog(self)
		#Dialog Modal anzeigen
		pStatusDialog.setModal(True)
		#Dialog anzeigen
		pStatusDialog.show()
		#Fortschrittsbalken setzen
		#2 Schritte für die Turbomaschine
		iSteps = 2
		#Durch alle BladeRows gehen
		for pBladeRowWidget in self.__lBladeRowWidgets:
			#Für jedes Profil 3 Schritte hinzufügen + 1 Schritt für die BladeRow an sich
			iSteps += pBladeRowWidget.anzProfileWidgets()*3 + 1
			#Fortschrittsfunktion setzen
			pBladeRowWidget.step = pStatusDialog.step
		#Für jedes BladeRowWidget es eine Schritt und in den BladeRows gibt es 3 Schritte pro Profil
		pStatusDialog.setProgressSteps(iSteps)

		self.step = pStatusDialog.step
		
		#Turbomachine holenself._pOutput.drawTurboMachine(self.Plot._display)
		self.getOutput()
		try:
			#Ausführen
			self._pOutput()
			#Nach der Berechnung zeichnen
			self.plot()
		except Exception, eException:
					#Fehler ausgeben
					print "Fehler", str(eException)
					
		#Als berechnet markieren
		self.setCalculated()
		#Dialog freigeben
		pStatusDialog.outputFinished()
		#Falls nötig VRML-Datei schreiben
		if(self.CBSchreibeVRML.isChecked()):
			#Durch alle BladeRows gehen
			for pCurBladeRow in self._pOutput.listRows:
				#VRML-Datei schreiben
				pCurBladeRow.writeVRML()




	## @brief Plots the result of the calculation of @link src::turboMachine::turboMachine turboMachine @endlink object<br><em>Zeichnet das Ergebnis der Berechnung des @link src::turboMachine::turboMachine turboMachine @endlink Objekts</em>
	#
	# Here could be your code. - This function is to be implemented. It is attached to Plot-Button in QtDesigner
	# <br><em>Hier könnte ihr Code stehen. - Diese Funktion muß noch implementiert werden. Sie ist mit dem Zeichnen-Button im QtDesigner verbunden.
	# </em>
	#
	# @todo drawing function<br><em>Zeichenfunktion</em>
	def plot(self):
		#Gibt es einen Output? Dann gibt es sicher auch ein Ergebnis
		if(self._pOutput == None):
			return

		#Liste der BladeRowMaterialien
		Materials = []
		
		#Durch alle BladeRows gehen
		for BladeRow in self.__lBladeRowWidgets:
			#Basismaterial erstellen
			Material = Graphic3d_MaterialAspect(Graphic3d_NOM_SILVER)
			#Diffusestärke setzen
			Material.SetDiffuse(0.8)

			#Farben erstellen
			Specular = Quantity_Color(0.4, 0.4, 0.4, Quantity_TOC_RGB)
			Ambiente = Quantity_Color(0.1, 0.1, 0.1, Quantity_TOC_RGB)
			#Farbe holen
			Color = BladeRow.color()
			
			#Farbe erstellen
			Diffuse = Quantity_Color(Color.redF(), Color.greenF(), Color.blueF(), Quantity_TOC_RGB)
			#Farbe setzen
			Material.SetDiffuseColor(Diffuse)
			#Zur Materialliste hinzufügen
			Materials.append(Material)

		#Turbomachine zeichnen
		self._pOutput.drawTurboMachine(self.Plot._display, Materials)


	## @brief Sets the data of the contained widgets according to a @link src::turboMachine::turboMachine turboMachine @endlink object<br><em>Setzt die Daten der Steuerelemente anhand eines @link src::turboMachine::turboMachine turboMachine @endlink Objektes</em>
	#
	# Overloaded from QBaseWidget::setFromObject(). Sets data of contained widgets and transforms the data of the bladeRow s to the corresponding QBladeRowWidget s.
	# <br><em>Überladen von QBaseWidget::setFromObject().Setzt die Werte der Steuerelemente und transformiert dann die bladeRow s in die entsprechenden QBladeRowWidget s.
	# </em>
	#
	# @param pTurboMachine - @link src::turboMachine::turboMachine TurboMachine @endlink object from which the data is to read<br><em>@link src::turboMachine::turboMachine TurboMachine @endlink Objekt aus dem die Daten gelesen werden sollen</em>
	#
	# @sa QBaseWidget::setFromObject()
	def setFromObject(self, pTurboMachine):
		#Typ prüfen
		if (pTurboMachine.__class__ != turboMachine):
			return

		#Anzahl von BladeRows setzen
		self.LEAnzBladeRows.setText(str(self._pOutput.numberRows))

		#An BladeRows verteilen
		for iCurBladeRow in range(self.anzBladeRowWidgets()):
			#BladeRows setzen
			self.getBladeRowWidget(iCurBladeRow).setFromObject(pTurboMachine.listRows[iCurBladeRow])
		




	## @brief Collects all data from the widget and fills it into @link src::turboMachine::turboMachine turboMachine @endlink<br><em>Sammelt alle Daten der Steuerelemente in die @link src::turboMachine::turboMachine turboMachine @endlink</em>
	#
	#  This function is attached to QBaseWidget::_OnGetOutput to use the provided automatisms.
	#  It collects all available data, creates a new @link src::turboMachine::turboMachine turboMachine @endlink object and configuration if neccessary. Afterwards it sets the directorys for the bladeRow s and
	#  reads their data from QBladeRowWidget s. Reads also the neccessary @link src::bladeRow::bladeRow bladeRow @endlink s and  @link src::profile2D::profile2D profile2D @endlink s
	#  <br><em>Diese Funktion wird an QBaseWidget::_OnGetOutput gehangen um die bereitsgestellten Automatismen zunutzen.
	#  Sie sammelt alle vorhandenen Daten, legt falls notwendig ein neues @link src::turboMachine::turboMachine turboMachine @endlink Objekt an und erstellt eine Configuration. Im Anschluß setzt es die
	#  Verzeichnisse der bladeRow s und liest deren Daten aus dem QBladeRowWidget s ein.
	#  Liest auch die benötigten @link src::bladeRow::bladeRow bladeRow @endlink s und  @link src::profile2D::profile2D profile2D @endlink s ein.
	#  </em>
	#
	#  @param pBase - @link src::turboMachine::turboMachine TurboMachine @endlink object in which the data is to write(optional)<br><em>@link src::turboMachine::turboMachine TurboMachine @endlink Objekt in das die Daten geschrieben werden sollen (optional)</em>
	#
	#  @return @link src::turboMachine::turboMachine TurboMachine @endlink object in which the data was written<br><em>@link src::turboMachine::turboMachine TurboMachine @endlink Objekt in das die Daten gefüllt wurden</em>
	#
	# @sa QBaseWidget._OnGetOutput, optionChanged()
	def OnGetOutput(self, pBase = None):
		#Arbeitsverzeichnis holen
		strWorkDir = self.LEWorkDir.text()
		#Liste mit BladeRows und Profiles erstellen
		lConfiguration = []
		#Durch die BladeRows gehen
		for pBladeRowWidget in self.__lBladeRowWidgets:
			#Anzahl an Profilen anhängen
			lConfiguration.append(pBladeRowWidget.anzProfileWidgets())
		#TODO: Neuerstellung verhindern, alles auf eine Turbomachine auslegen
		#Noch kein Output vorhanden?
		if(self._pOutput == None):
			#Turbomachine erstellen
			## @brief outputobject of the calculation<br><em>Ausgabeobjekt der Berechnung</em>
			#
			# Overwritten from QBaseWidget::_pOutput. In this widget #_pOutput is of turboMachine type.
			# It contains all @link src::bladeRow::bladeRow bladeRow @endlink s and @link src::profile2D::profile2D profile2D @endlink s neccessary.
			# <br><em>Überschrieben von QBaseWidget::_pOutput. In diesem Widget ist #_pOutput vom Typ turboMachine.
			# Es enthält alle für die Berechnung benötigten @link src::bladeRow::bladeRow bladeRow @endlink s und @link src::profile2D::profile2D profile2D @endlink s.
			# </em>
			#
			# @sa QBaseWidget::_pOutput, OnGetOutput(), OnPerformCalc()
			self._pOutput = turboMachine(str(strWorkDir), lConfiguration)
		#else:
			#del self._pOutput
			#self._pOutput = turboMachine(str(strWorkDir), lConfiguration)
			
		#Debugmodus übernehmen
		self._pOutput.debug = self.CBDebug.currentIndex()
		#Fortschrittsfunktion setzen
		self._pOutput.step = self.step

		#Schritte anhängen in den BladeRow
		for pBladeRow in self._pOutput.listRows:
			pBladeRow.step = self.step
		
		#BladeRows anhängen
		for iCurrentBladeRow in range(self.anzBladeRowWidgets()):
			#Hat sich was geändert?
			#if (self.__lBladeRowWidgets[iCurrentBladeRow].isChangedCalc()):
				#BladeRow neu holen
				#self._pOutput.listRows[iCurrentBladeRow] = self.__lBladeRowWidgets[iCurrentBladeRow].getBladeRow(self._pOutput.listRows[iCurrentBladeRow])
			self._pOutput.listRows[iCurrentBladeRow] = self.__lBladeRowWidgets[iCurrentBladeRow].getOutput(self._pOutput.listRows[iCurrentBladeRow])
			#Arbeitsverzeichnis hinunterreichen
			self._pOutput.listRows[iCurrentBladeRow].dataStruct = self._pOutput.dataStruct





	## @brief Opens a file dialog to select a working directory<br><em>Öffnet einen Dateidialog um ein Arbeitsverzeichnis auszuwählen</em>
	#
	# Is used for comfortable selection of a working directory. Is attached to ...-Button.
	# <br><em>Dient zur komfortableren Auswahl eines Arbeitsverzeichnis. Ist mit dem ...-Button verbunden.
	# </em>
	def selectWorkDir(self):
		#Neues Verzeichnis wählen lassen
		strNewDir = QFileDialog.getExistingDirectory(self, self.tr("Arbeitsverzeichnis wählen"))
		#Kein Verzeichnis gewählt?
		if (strNewDir == ''):
			return

		#Ansonsten Verzeichnis übernehmen
		self.LEWorkDir.setText(strNewDir)





	## @brief Copies additional values from a QTurboMachineWidget to this one<br><em>Kopiert zusätzliche Eingaben von einer anderen QTurboMachineWidget in diese</em>
	#
	# There is no additional Data to copy. Not yet.
	# <br><em>Es gibt keine zusätzliche Eingaben zukopieren. Jedenfalls noch nicht.
	# </em>
	#
	# @param pTurboMachine - QTurboMachineWidget from which data is copied<br><em>QTurboMachineWidget von dem die Daten kopiert werden sollen</em>
	#
	# @sa QBaseWidget::_OnCopyFrom
	def OnCopyFrom(self, pTurboMachine):
		pass





	## @brief Returns the number of QBladeRowWidget s contained in the list of bladeRows<br><em>Gibt die Anzahl der QBladeRowWidget s enthalten in der Bladerowliste wieder</em>
	#
	# Auxiliary function for superior classes and visualization.
	# <br><em>Hilfsfunktion für übergeordnete Klassen und Darstellung.
	# </em>
	#
	# @return Number of contained QBladeRowWidget s<br><em>Anzahl der enthaltenen QBladeRowWidget s</em>
	#
	# @sa __lBladeRowWidgets, QBladeDesigner::QBladeDesigner
	def anzBladeRowWidgets(self):
		#Anzahl an BladeRows zurückgeben
		return len(self.__lBladeRowWidgets)





	## @brief Adds a new QBladeRowWidget to the list of bladeRows<br><em>Fügt ein neues QBladeRowWidget in die Bladerowliste ein</em>
	#
	# Auxiliary function for superior classes. Updates the number of span entrys as well.
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Aktualisiert gleichzeitig die Anzahl der Einträge für Span.
	# </em>
	#
	# @param iIndex - Index of list, where the new QBladeRowWidget is added<br><em>Stelle in der Liste an der das neue QBladeRowWidget eingefügt werden soll</em>
	# @param pBladeRow - The QBladeRowWidget to add<br><em>Das einzufügende QBladeRowWidget</em>
	#
	# @sa __lBladeRowWidgets, QBladeDesigner::QBladeDesigner::addBladeRow()
	def insertBladeRowWidget(self, iIndex, pBladeRow):
		#Nachrichtensignale übernehmen
		self.connect(pBladeRow, SIGNAL("contentChanged()"), self.contentChanged)
		#Neue pBladeRow anhängen
		self.__lBladeRowWidgets.insert(iIndex, pBladeRow)





	## @brief Removes a QBladeRowWidget from the list of bladeRows<br><em>Entfernt ein QBladeRowWidget aus der Bladerowliste</em>
	#
	# Auxiliary function for superior classes. Updates the number of span entrys as well
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Aktualisiert gleichzeitig die Anzahl der Einträge für Span.
	# </em>
	#
	# @param iIndex - Index of the QBladeRowWidget to remove<br><em>Index des zu entfernenden QBladeRowWidget s</em>
	#
	# @sa __lBladeRowWidgets, QBladeDesigner::QBladeDesigner::delBladeRow()
	def delBladeRowWidget(self, iIndex):
		#iIndex auf Gültigkeit prüfen
		if ((iIndex < 0) or (iIndex >= self.anzBladeRowWidgets())):
			return

		#Signal kappen
		self.disconnect(self.__lBladeRowWidgets[iIndex], SIGNAL("contentChanged()"), self.contentChanged)
		#Eintrag löschen
		del self.__lBladeRowWidgets[iIndex]





	## @brief Gets a certain QBladeRowWidget from the list of bladeRows<br><em>Holt ein bestimmtes QBladeRowWidget aus der Bladerowliste</em>
	#
	# Auxiliary function for superior classes. The QBladeRowWidget is not removed from list.
	# <br><em>Hilfsfunktion für übergeordnete Klassen. Das QBladeRowWidget wird nicht aus der Liste gelöscht.
	# </em>
	#
	# @param iIndex - Index of the QBladeRowWidget to return<br><em>Index des QBladeRowWidget, welches zurückgegeben werden soll</em>
	#
	# @sa __lBladeRowWidgets, QBladeDesigner::QBladeDesigner::copy(), QBladeDesigner::QBladeDesigner::addBladeRow(), QBladeDesigner::QBladeDesigner::itemClicked(), setFromObject()
	def getBladeRowWidget(self, iIndex):
		#iIndex auf Gültigkeit prüfen
		if ((iIndex < 0) or (iIndex >= self.anzBladeRowWidgets())):
			return None

		#BladeRow zurückgeben
		return self.__lBladeRowWidgets[iIndex]





	## @brief Overwrites a QBladeRowWidget with an other<br><em>Überschreibt ein QBladeRowWidget mit einem Anderen</em>
	#
	# Auxiliary function for superior classes.
	# <br><em>Hilfsfunktion für übergeordnete Klassen.
	# </em>
	#
	# @param iIndex - Index of the QBladeRowWidget to overwrite<br><em>Index des QBladeRowWidget s in der Liste das überschrieben werden soll</em>
	# @param pBladeRow - the new QBladeRowWidget<br><em>Das neue QBladeRowWidget</em>
	#
	# @sa __lBladeRowWidgets
	def setBladeRowWidget(self, iIndex, pBladeRow):
		#iIndex auf Gültigkeit prüfen
		if ((iIndex < 0) or (iIndex >= self.anzBladeRowWidgets())):
			return

		#pBladeRow zurückgeben
		self.__lBladeRowWidgets[iIndex] = pBladeRow





	## @brief Swaps 2 QBladeRowWidget s in the list of bladeRows<br><em>Vertauscht 2 QBladeRowWidget s in der Bladerowliste</em>
	#
	# Auxiliary function for superior classes.
	# <br><em>Hilfsfunktion für übergeordnete Klassen.
	# </em>
	#
	# @param iIndexBladeRow1 - Index of the first QBladeRowWidget to swap<br><em>Index des ersten QBladeRowWidget s das getauscht werden soll</em>
	# @param iIndexBladeRow2 - Index of the second QBladeRowWidget to swap<br><em>Index des zweiten QBladeRowWidget s das getauscht werden soll</em>
	#
	# @sa __lBladeRowWidgets, QBladeDesigner::QBladeDesigner::up(), QBladeDesigner::QBladeDesigner::down()
	def swapBladeRowWidgets(self, iIndexBladeRow1, iIndexBladeRow2):
		#Gleiche Indices?
		if (iIndexBladeRow2 == iIndexBladeRow1):
			#Wechsel nicht nötig
			return

		#Klassischer Wechsel
		#BladeRow1 zwischenspeichern
		Temp = self.__lBladeRowWidgets[iIndexBladeRow1]
		#Profile2 nach Profile1
		self.__lBladeRowWidgets[iIndexBladeRow1] = self.__lBladeRowWidgets[iIndexBladeRow2]
		#Profile2 mit durch Profile2 ersetzen
		self.__lBladeRowWidgets[iIndexBladeRow2] = Temp





	## @brief Loads a @link src::turboMachine::turboMachine turboMachine @endlink object from a corresponding working directory<br><em>Läd eine @link src::turboMachine::turboMachine turboMachine @endlink Objekt aus einem entsprechendem Arbeitsverzeichnis</em>
	#
	# Passes the call to the @link src::turboMachine::turboMachine::__init__() turboMachine-Construktor @endlink
	# <br><em>Übergibt den Aufruf nur in den @link src::turboMachine::turboMachine::__init__() turboMachine-Konstruktor @endlink
	# </em>
	#
	# @param strWorkDir - Working directory containing XML-file to read<br><em>Arbeitsverzeichnis, dass die zu lesende XML-Datei enthält</em>
	#
	# @sa @link src::turboMachine::turboMachine turboMachine @endlink
	def loadFromWorkDir(self, strWorkDir):
		#Neue TurboMachine mit entsprechenden Arbeitsverzeichnis erstellen
		self._pOutput = turboMachine(str(strWorkDir))




	## @brief Saves the @link src::turboMachine::turboMachine turboMachine @endlink object to the working directory<br><em>Speichert das @link src::turboMachine::turboMachine turboMachine @endlink Objekt in das Arbeitsverzeichnis</em>
	#
	# The corresponding working directory is saved in @link src::turboMachine::turboMachine turboMachine @endlink object. Is marked as saved.
	# <br><em>Das entsprechende Arbeitsverzeichnis wird in dem @link src::turboMachine::turboMachine turboMachine @endlink Objekt gespeichert. Wird als gespeichert markiert.
	# </em>
	#
	# @sa selectWorkDir(), loadFromWorkDir(), QBladeDesigner::QBladeDesigner::load(), QBladeDesigner::QBladeDesigner::saveAs(), @link src::turboMachine::turboMachine::writeTurboMachineXml turboMachine::writeTurboMachineXml @endlink
	def saveToWorkDir(self):
		#Eingaben holen
		self.getOutput()
		#Arbeitsverzeichnis setzen
		self._pOutput.dataStruct = dataStructure(str(self.LEWorkDir.text()))
		#Daten rausschreiben
		self._pOutput.writeTurboMachineXml(0)
		#Als gespeichert markieren
		self.setSaved()





	## @brief Renames all labels as the corresponding language<br><em>Bennent alle Label in die entsprechende Sprache um</em>
	#
	#  Reimplemented from QBaseWidget::retranslateStrings(). Used for localisation purpose. The texts inputed in QtDesigner will be overwritten.
	#  <br><em>Diese Funktion ist reimplementiert aus QBaseWidget::retranslateStrings(). Wird zur Lokalisation verwendet. Die im QtDesigner eingetragenen Texte für
	#  Steuerelemente werden überschrieben.</em>
	#
	#  @param bPassDown - tells whether to pass down the retranslation to underlying Widgets<br><em>Gibt an, ob die Übersetzung an tiefer liegende Widgets übergeben werden soll.</em>
	def retranslateStrings(self, bPassDown = True):
		#Label
		self.LNumberBladeRows.setText(self.tr("Anzahl BladeRows", "Text eines Labels"))
		self.LWorkDir.setText(self.tr("Arbeitsverzeichnis", "Text eines Labels"))
		self.LDebug.setText(self.tr("Debug", "Text eines Labels"))

		#ComboBoxen
		#CBDebug
		lLabels = (
			self.tr("de...what?",  "Text des 1. Eintrags in CBDebug"),
			self.tr("gimme some",  "Text des 2. Eintrags in CBDebug"),
			self.tr("spam me!",  "Text des 3. Eintrags in CBDebug")
		)

		#Durch die ComboBox gehen und ändern
		for iCurrent in range(self.CBDebug.count()):
			#Namen übernehmen
			self.CBDebug.setItemText(iCurrent, lLabels[iCurrent])

		#Buttons
		self.BCalculate.setText(self.tr("&Calculate", "Aufschrift des Berechnen-Buttons"))
		#self.BPlot.setText(self.tr("&Plot", "Aufschrift des Plotten-Buttons"))

		#Checkboxen
		self.CBSchreibeVRML.setText(self.tr("&VRML erstellen", "Aufschrift der Checkbox die VRML Dateien Ausgabe aktiviert"))

		#Weitergeben?
		if (bPassDown):
			#Durch alle BladeRows gehen
			for pBladeRow in self.__lBladeRowWidgets:
				#BladeRow neu beschriften
				pBladeRow.retranslateStrings()
		



## @brief Main Programm
#Testfall nur bei entsprechenden Parameter ausführen
#TODO: Eventuell Entfernen
if (len(sys.argv) > 1) and (sys.argv[1] == 'ShowTestCase'):
	#Qt-Application erstellen
	pApp = QApplication([])
	#Dummy Dialog zur Anzeige des Widgets erstellen
	pDlg = QDialog()
	#Widget erstellen
	pTurboMachine = QTurboMachineWidget()
	#Layout für den Dialog erstellen
	pDlg.setLayout(QVBoxLayout())
	#Widget ins Layout einfügen
	pDlg.layout().addWidget(pTurboMachine)
	#Dialog anzeigen
	pDlg.show()
	#Ausführen
	sys.exit(pApp.exec_()) 