#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Andreas Kührmann                           *
#*   andreas.kuehrmann@gmail.com                                           *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
#  @author Andreas Kührmann
#  @date 2009

# Python Imports
# üöä

from nurbsToolSet import getCurveCoordinates, calculateUFromZ, moveCurvePoint, calculateUFromM, calculateUStart, movePoint
from numpy import ones


def getDomainCoordinates(l, curve, domainLength, gridType, outputCS, tolerance, maxIteration):
	
	if gridType == "fS":
		
		inputCS = 'xyz'
		
		zLeadingEdge  = curve.pointAt(0).getz()
		zTrailingEdge = curve.pointAt(1).getz()
		zWanted       = l*domainLength
		
		if zWanted >= zLeadingEdge and zWanted <= zTrailingEdge:
			
			uWanted = calculateUFromZ(zWanted, curve, -1, 0, 0, 1, tolerance, maxIteration)
			
			return  getCurveCoordinates(uWanted, curve, outputCS)
			
		elif zWanted < zLeadingEdge:
			
			return moveCurvePoint(0, curve, 0,  0, -(zLeadingEdge-zWanted), inputCS, outputCS)
			
		elif zWanted > zTrailingEdge:
			
			return moveCurvePoint(1, curve, 0,  0, (zWanted-zTrailingEdge), inputCS, outputCS)
			
	elif gridType == "rS":
		
		inputCS = 'rpz'
		
		lenghtWanted = domainLength.length()*(1-l)
		
		u = calculateUStart(1, lenghtWanted, domainLength, tolerance, maxIteration)
		
		mWanted       = getCurveCoordinates(u, domainLength, "m")
		mLeadingEdge  = getCurveCoordinates(0, curve, "m")
		mTrailingEdge = getCurveCoordinates(1, curve, "m")
		
		if mWanted >= mLeadingEdge and mWanted <= mTrailingEdge:
			
			uWanted = calculateUFromM(mWanted, 1,  0, 1, curve, tolerance, maxIteration)
			
			return  getCurveCoordinates(uWanted, curve, outputCS)
			
		elif mWanted < mLeadingEdge:
			
			uZeroCoordinates = getCurveCoordinates(0, curve, inputCS)
			
			return moveCurvePoint(u, domainLength, 0, uZeroCoordinates[1], 0, inputCS, outputCS)
			
		elif mWanted > mTrailingEdge:
			
			uOneCoordinates = getCurveCoordinates(1, curve, inputCS)
			
			return moveCurvePoint(u, domainLength, 0, uOneCoordinates[1], 0, inputCS, outputCS)

def transformDomainIntoLValues(fixPointsOnCurve, partLength, curve, domainLength, gridType, tolerance, maxIteration):
	
	l       = []
	lReturn = []
	
	partLength.reverse()
	fixPointsOnCurve.reverse()
	
	if gridType == 'fS':
		
		zLeadingEdge  = curve.pointAt(0).getz()
		zTrailingEdge = curve.pointAt(1).getz()
		
		for pointIndex in xrange(len(fixPointsOnCurve)-1):
			
			partLength[pointIndex] = partLength[pointIndex][::-1]
			
			l.append([])
			
			z       = getDomainCoordinates(fixPointsOnCurve[pointIndex],   curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
			zBefore = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
			
			if z[2] > zTrailingEdge and zBefore[2] > zTrailingEdge:
				
				tmpLength = 0
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					tmpLength = tmpLength+partLength[pointIndex][partIndex]
					
					l[pointIndex].append(1-tmpLength/domainLength)
					
			elif z[2] < zLeadingEdge and zBefore[2] < zLeadingEdge:
				
				tmpLength = sum(partLength[pointIndex])
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					tmpLength = tmpLength-partLength[pointIndex][partIndex]
					
					l[pointIndex].append(tmpLength/domainLength)
					
			elif z[2] > zTrailingEdge and zBefore[2] >= zLeadingEdge and zBefore[2] <= zTrailingEdge:
				
				tmpXYZ = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				
				tmpLength = tmpXYZ[2]
				
				inDomain = True
				
				for partIndex in xrange(len(partLength[pointIndex])):
					
					tmpLength = tmpLength-partLength[pointIndex][partIndex]
					
					if tmpLength > zTrailingEdge:
						
						l[pointIndex].append(tmpLength/domainLength)
						
					else:
						if inDomain == True:
							
							residue = zTrailingEdge-tmpLength
							
							u = calculateUStart(1, residue, curve, tolerance, maxIteration)
							
							if partIndex < len(partLength[pointIndex])-1:
								
								l[pointIndex].append(computeL(u, curve, domainLength, gridType))
							
							lastU = u
							
							inDomain = False
							
						elif inDomain == False:
							
							u = calculateUStart(lastU, partLength[pointIndex][partIndex], curve, tolerance, maxIteration)
							
							if partIndex < len(partLength[pointIndex])-1:
								
								l[pointIndex].append(computeL(u, curve, domainLength, gridType))
							
							lastU = u
							
			elif z[2]< zTrailingEdge and zBefore[2] < zTrailingEdge and z[2] > zLeadingEdge and zBefore[2] > zLeadingEdge:
				
				for partIndex in xrange(len(partLength[pointIndex])):
					
					u = calculateUStart(lastU, partLength[pointIndex][partIndex], curve, tolerance, maxIteration)
						
					if partIndex < len(partLength[pointIndex])-1:
						
						l[pointIndex].append(computeL(u, curve, domainLength, gridType))
						
					lastU = u
				
			elif z[2] >= zLeadingEdge and z[2] <= zTrailingEdge and zBefore[2] < zTrailingEdge:
				
				inCurve = True
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					if curve.lengthIn(0, lastU, tolerance, maxIteration) > partLength[pointIndex][partIndex]:
						
						u = calculateUStart(lastU, partLength[pointIndex][partIndex], curve, tolerance, maxIteration)
						
						lastU = u
						
						l[pointIndex].append(computeL(u, curve, domainLength, gridType, tolerance, maxIteration))
						
						
					else:
						if inCurve == True:
							
							residue = partLength[pointIndex][partIndex]-curve.lengthIn(0, lastU, tolerance, maxIteration)
							
							tmpLength = zLeadingEdge-residue
							
							l[pointIndex].append(tmpLength/domainLength)
							
							inCurve = False
							
						elif inCurve == False:
							
							tmpLength = tmpLength-partLength[pointIndex][partIndex]
							
							l[pointIndex].append(tmpLength/domainLength)
							
			l[pointIndex].reverse()
			
	elif gridType == 'rS':
		
		mLeadingEdge  = getCurveCoordinates(0, curve, "m")
		mTrailingEdge = getCurveCoordinates(1, curve, "m")
		
		uLeadingEdge  = calculateUFromM(mLeadingEdge,  1, 0, 1, domainLength, tolerance, maxIteration)
		uTrailingEdge = calculateUFromM(mTrailingEdge, 1, 0, 1, domainLength, tolerance, maxIteration)
		
		for pointIndex in xrange(len(fixPointsOnCurve)-1):
			
			l.append([])
			
			m       = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, "m", tolerance, maxIteration)
			mBefore = getDomainCoordinates(fixPointsOnCurve[pointIndex+1],   curve, domainLength, gridType, "m", tolerance, maxIteration)
			
			if m > mTrailingEdge and mBefore > mTrailingEdge:
				
				lastU = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], domainLength, tolerance, maxIteration)
					
					lastU = uOnCalcCurve
					
					l[pointIndex].append(computeL(uOnCalcCurve, domainLength, domainLength, gridType, tolerance, maxIteration))
			
			
			elif m < mLeadingEdge and m < mLeadingEdge:
				
				lastU = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], domainLength, tolerance, maxIteration)
					
					lastU = uOnCalcCurve
					
					l[pointIndex].append(computeL(uOnCalcCurve, domainLength, domainLength, gridType, tolerance, maxIteration))
					
			elif m > mTrailingEdge and mBefore >= mLeadingEdge and mBefore <= mTrailingEdge:
				
				lastU = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				calcCurve = domainLength
				
				inDomain = True
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], calcCurve, tolerance, maxIteration)
					
					if uOnCalcCurve > uTrailingEdge and calcCurve == domainLength:
						
						lastU = uOnCalcCurve
						
						l[pointIndex].append(computeL(uOnCalcCurve, domainLength, domainLength, gridType, tolerance, maxIteration))
						
					else:
						
						if inDomain == True:
							
							calcCurve = curve
							
							partLength[pointIndex][partIndex] = partLength[pointIndex][partIndex]-domainLength.lengthIn(uTrailingEdge, lastU, tolerance, maxIteration)
							
							uOnCalcCurve = calculateUStart(1, partLength[pointIndex][partIndex], curve, tolerance, maxIteration)
							
							l[pointIndex].append(computeL(uOnCalcCurve, curve, domainLength, gridType, tolerance, maxIteration))
							
							lastU = uOnCalcCurve
							
							inDomain = False
							
						elif inDomain == False:
							
							l[pointIndex].append(computeL(uOnCalcCurve, curve, domainLength, gridType, tolerance, maxIteration))
							
							lastU = uOnCalcCurve
							
			elif m < mTrailingEdge and mBefore < mTrailingEdge and m > mLeadingEdge and mBefore > mLeadingEdge:
				
				lastU = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], curve, tolerance, maxIteration)
					
					l[pointIndex].append(computeL(uOnCalcCurve, curve, domainLength, gridType))
					
					lastU = uOnCalcCurve
					
			elif m >= mLeadingEdge and m <= mTrailingEdge and mBefore < mTrailingEdge:
				
				lastU = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				calcCurve = curve
				
				inCurve = True
				
				for partIndex in xrange(len(partLength[pointIndex])-1):
					
					if curve.lengthIn(0, lastU, tolerance, maxIteration) > partLength[pointIndex][partIndex] and calcCurve == curve:
						
						uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], calcCurve, tolerance, maxIteration)
						
						lastU = uOnCalcCurve
						
						l[pointIndex].append(computeL(uOnCalcCurve, curve, domainLength, gridType, tolerance, maxIteration))
						
					else:
						if inCurve == True:
							
							calcCurve = domainLength
							
							partLength[pointIndex][partIndex] =  partLength[pointIndex][partIndex]-curve.lengthIn(0, lastU, tolerance, maxIteration)
							
							uOnCalcCurve = calculateUStart(uLeadingEdge, partLength[pointIndex][partIndex], domainLength, tolerance, maxIteration)
							
							l[pointIndex].append(computeL(uOnCalcCurve, domainLength, domainLength, gridType, tolerance, maxIteration))
							
							lastU = uOnCalcCurve
							
							inCurve = False
							
						elif inCurve == False:
							
							uOnCalcCurve = calculateUStart(lastU, partLength[pointIndex][partIndex], domainLength, tolerance, maxIteration)
							
							l[pointIndex].append(computeL(uOnCalcCurve, domainLength, domainLength, gridType, tolerance, maxIteration))
							
							lastU = uOnCalcCurve
							
			l[pointIndex].reverse()
			
	for pointIndex in xrange(len(l)-1, -1, -1):
		
		for lIndex in xrange(len(l[pointIndex])):
			
			lReturn.append(l[pointIndex][lIndex])
			
		lReturn.append('s')
		
	#for lIndex in xrange(len(lReturn)):
		#print lReturn[lIndex]
		
	return lReturn


def computeL(u, curve, domainLength, gridType, tolerance=1e-9, maxIteration=100):
	
	if gridType == 'fS':
		
		zWanted = curve.pointAt(u).getz()
		
		l = zWanted/domainLength
	
	elif gridType == 'rS':
		
		if curve != domainLength:
			
			mWanted = getCurveCoordinates(u, curve, "m")
			
			uDomainLength = calculateUFromM(mWanted, 1, 0, 1, domainLength, tolerance, maxIteration)
			
			partLenght = domainLength.lengthIn(0, uDomainLength, tolerance, maxIteration)
			
		else:
			mWanted = getCurveCoordinates(u, domainLength, "m")
			
			uDomainLength = calculateUFromM(mWanted, 1, 0, 1, domainLength, tolerance, maxIteration)
			
			partLenght = domainLength.lengthIn(0, uDomainLength, tolerance, maxIteration)
		
		l = partLenght/domainLength.length()
		
	return l


def calculateUFromL(l, curve, domainLength, gridType='fS', tolerance=1e-9, maxIteration=100):
	
	if gridType == 'fS':
		
		zLE = curve.pointAt(0).getz()
		zTE = curve.pointAt(1).getz()
		
		xyz = getDomainCoordinates(l, curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
		
		zWanted = xyz[2]
			
		if zWanted > zLE and zWanted < zTE:
			
			return calculateUFromZ(zWanted, curve, 1, 0, 0, 1, tolerance, maxIteration)
			
		else:
			
			return zWanted
	
	elif gridType == 'rS':
		
		mLE = getCurveCoordinates(0, curve, 'm')
		mTE = getCurveCoordinates(1, curve, 'm')
		
		mWanted = getDomainCoordinates(l, curve, domainLength, gridType, 'm', tolerance, maxIteration)
		
		if mLE <= mWanted <= mTE:
			
			return calculateUFromM(mWanted, 1, 0, 1, curve, tolerance, maxIteration)
			
		else :
			
			return calculateUFromM(mWanted, 1, 0, 1, domainLength, tolerance, maxIteration)


def computePartLengthDomain(fixPointsOnCurve, curve, domainLength, gridType='fS', tolerance=1e-9, maxIteration=100):
	
	partLength = []
	
	lLE = computeL(0, curve, domainLength, gridType, tolerance, maxIteration)
	lTE = computeL(1, curve, domainLength, gridType, tolerance, maxIteration)
	
	for pointIndex in xrange(len(fixPointsOnCurve)-1):
		
		if fixPointsOnCurve[pointIndex] >= lLE and fixPointsOnCurve[pointIndex+1] >= lLE and fixPointsOnCurve[pointIndex] <= lTE and fixPointsOnCurve[pointIndex+1] <= lTE:
			
			
			uOne = calculateUFromL(fixPointsOnCurve[pointIndex],   curve, domainLength, gridType, tolerance, maxIteration)
			uTwo = calculateUFromL(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, tolerance, maxIteration)
			
			partLength.append(curve.lengthIn(uOne, uTwo, tolerance, maxIteration))
			
		elif fixPointsOnCurve[pointIndex] < lLE and fixPointsOnCurve[pointIndex+1] >= lLE and fixPointsOnCurve[pointIndex+1] <= lTE:
			
			if gridType == 'fS':
				
				xyzOne = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				xyzTwo = getCurveCoordinates(0, curve, 'xyz')
				
				u = calculateUFromL(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, tolerance, maxIteration)
				
				partLength.append(xyzTwo[2]-xyzOne[2]+curve.lengthIn(0, u, tolerance, maxIteration))
				
			elif gridType == 'rS':
				
				mOne = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, 'm', tolerance, maxIteration)
				mTwo = getCurveCoordinates(0, curve, 'm')
				
				uOne = calculateUFromM(mOne, 1, 0, 1, domainLength, tolerance, maxIteration)
				uTwo = calculateUFromM(mTwo, 1, 0, 1, domainLength, tolerance, maxIteration)
				
				uThree = calculateUFromL(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, tolerance, maxIteration)
				
				partLength.append(domainLength.lengthIn(uOne, uTwo, tolerance, maxIteration)+curve.lengthIn(0, uThree, tolerance, maxIteration))
				
		elif fixPointsOnCurve[pointIndex] <= lTE and fixPointsOnCurve[pointIndex] >= lLE and fixPointsOnCurve[pointIndex+1] > lTE:
			
			if gridType == 'fS':
				
				xyzOne = getCurveCoordinates(1, curve, 'xyz')
				xyzTwo = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				
				u = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				partLength.append(xyzTwo[2]-xyzOne[2]+curve.lengthIn(u, 1, tolerance, maxIteration))
				
			elif gridType == 'rS':
				
				mOne = getCurveCoordinates(1, curve, 'm')
				mTwo = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'm', tolerance, maxIteration)
				
				uOne = calculateUFromM(mOne, 1, 0, 1, domainLength, tolerance, maxIteration)
				uTwo = calculateUFromM(mTwo, 1, 0, 1, domainLength, tolerance, maxIteration)
				
				uThree = calculateUFromL(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, tolerance, maxIteration)
				
				partLength.append(domainLength.lengthIn(uOne, uTwo, tolerance, maxIteration)+curve.lengthIn(uThree, 1, tolerance, maxIteration))
				
		elif fixPointsOnCurve[pointIndex] < lLE and fixPointsOnCurve[pointIndex+1] > lTE:
			
			if gridType == 'fS':
				
				xyzOne   = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				xyzTwo   = getCurveCoordinates(0, curve, 'xyz')
				xyzThree = getCurveCoordinates(1, curve, 'xyz')
				xyzFour  = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				
				partLength.append(xyzTwo[2]-xyzOne[2]+xyzFour[2]-xyzThree[2]+curve.lengthIn(0, 1, tolerance, maxIteration))
				
			elif gridType == 'rS':
				
				mOne   = getDomainCoordinates(fixPointsOnCurve[pointIndex], curve, domainLength, gridType, 'm', tolerance, maxIteration)
				mTwo   = getCurveCoordinates(0, curve, 'm')
				mThree = getCurveCoordinates(1, curve, 'm')
				mFour  = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'm', tolerance, maxIteration)
				
				uOne   = calculateUFromM(mOne,   1, 0, 1, domainLength, tolerance, maxIteration)
				uTwo   = calculateUFromM(mTwo,   1, 0, 1, domainLength, tolerance, maxIteration)
				uThree = calculateUFromM(mThree, 1, 0, 1, domainLength, tolerance, maxIteration)
				uFour  = calculateUFromM(mFour,  1, 0, 1, domainLength, tolerance, maxIteration)
				
				partLength.append(curve.lengthIn(0, 1, tolerance, maxIteration)+domainLength.lengthIn(uOne, uTwo, tolerance, maxIteration)+domainLength.lengthIn(uThree, uFour, tolerance, maxIteration))
		else:
			if gridType == 'fS':
				
				xyzOne = getDomainCoordinates(fixPointsOnCurve[pointIndex],   curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				xyzTwo = getDomainCoordinates(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, 'xyz', tolerance, maxIteration)
				
				partLength.append(xyzTwo[2]-xyzOne[2])
				
			elif gridType == 'rS':
				
				uOne = calculateUFromL(fixPointsOnCurve[pointIndex],   curve, domainLength, gridType, tolerance, maxIteration)
				uTwo = calculateUFromL(fixPointsOnCurve[pointIndex+1], curve, domainLength, gridType, tolerance, maxIteration)
				
				partLength.append(domainLength.lengthIn(uOne, uTwo, tolerance, maxIteration))
				
	return partLength


def computeAbsCellLength(noCells, grading, blockEdgeLength):
	
	cellLength = ones([noCells], 'd')
	
	if grading != 1 and noCells != 1:
		
		cellToCellExpansionsRatio = grading**(1.0/(noCells-1.0))
		
		startSpacing = (blockEdgeLength*(1-cellToCellExpansionsRatio))/(1-cellToCellExpansionsRatio**noCells)
		
		for cellIndex in xrange(noCells):
			
			cellLength[cellIndex] = startSpacing*cellToCellExpansionsRatio**(cellIndex)
			
	elif grading == 1 and noCells != 1:
		
		for cellIndex in xrange(1, noCells+1):
			
			cellLength[cellIndex-1] = blockEdgeLength/noCells
			
	else:
		
		cellLength[0] = blockEdgeLength
		
	return cellLength

def computeAbsControlPoint(relControlPoint, curve, domainLength, pitch, gridType, inletOffset=0, outletOffset=0, tolerance=1e-9, maxIteration=100):
	
	if gridType == 'fS':
		
		inputCS  = "xyz"
		outputCS = "xyz"
		
	elif gridType == 'rS':
		
		inputCS  = "rpz"
		outputCS = "rpz"
		
	lEdge = computeL(0, curve, domainLength, gridType, tolerance, maxIteration)
	tEdge = computeL(1, curve, domainLength, gridType, tolerance, maxIteration)
	
	if relControlPoint[1] <= lEdge:
		
		pitchOffset = ((0.0-inletOffset)/(lEdge)*relControlPoint[1]+inletOffset)*(relControlPoint[0]/abs(relControlPoint[0]))
		
	elif relControlPoint[1] >= tEdge:
			
		grad = (outletOffset)/(1.0-tEdge)
			
		pitchOffset = (grad*relControlPoint[1]-(grad*tEdge))*(relControlPoint[0]/abs(relControlPoint[0]))
			
	else:
			
		pitchOffset = 0.0
	
	tmpCoords        = getDomainCoordinates(relControlPoint[1], curve, domainLength, gridType, outputCS, tolerance, maxIteration)
	absControlPoints = movePoint(tmpCoords, 0, pitch*(0.5+pitchOffset)*relControlPoint[0], 0, inputCS, "xyz")
	
	return absControlPoints

