#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian Barthmes
# @author Oliver Borm
# @author Julius Fellerhoff
# @date 17.12.2008

# Python Imports
import os, sys#, getopt, string
sys.path.insert(0 ,"../side-packages")

from math import atan2
import numpy

from pythonnurbs import NurbsPoint, NurbsMatrix, NurbsSurface, NurbsCurve, NurbsColor, NurbsVector, NurbsHPoint, NurbsMatrixRT

def printCurvePnts(curve):
	import pylab as p
	#Schreibt die Kontrollpunkte und den Knotenvektor der 3D NURBS Kurve aus:
	ptarr=numpy.zeros((curve.ctrlPnts().n(),3))
	for n in range(curve.ctrlPnts().n()):
		ptarr[n]=[curve.ctrlPnts(n).getx(),curve.ctrlPnts(n).gety(),curve.ctrlPnts(n).getz()]
	print "Control points of",curve
	print ptarr
	
	ktarr=numpy.zeros((curve.knot().n()))
	for n in range(curve.knot().n()):
		ktarr[n]=curve.knot(n)
	print "Knots of",curve
	print ktarr
	
	weight=numpy.zeros((curve.ctrlPnts().n()))
	for n in range(curve.ctrlPnts().n()):
		weight[n]=curve.ctrlPnts(n).getw()
	print "Weigth of",curve
	print weight
	
def diffNurbsDiscretes(curve,pts):
	import pylab as p
	#Plot der in normal-Richtung projezierten Kurven
	ptarr=numpy.zeros((curve.ctrlPnts().n(),3))
	for n in range(curve.ctrlPnts().n()):
		ptarr[n]=[curve.ctrlPnts(n).getx(),curve.ctrlPnts(n).gety(),curve.ctrlPnts(n).getz()]
	### Plot zum Vergleich von Nurbskurve und diskreten Punkten
	arrr=numpy.zeros((1000,2))
	for i in range(1000):
		arrr[i,0]=curve.pointAt(i/1000.0).getz()
		arrr[i,1]=curve.pointAt(i/1000.0).gety()
	p.plot(arrr[:,0],-arrr[:,1])
	p.plot(pts[:,2],-pts[:,1],'bo')
	p.plot(ptarr[:,2],-ptarr[:,1],'rx')
	p.axis('equal')
	p.show()
	
def diffNurbsDiscretesCascade(curve,pts):
	import pylab as p
	#Plot der in normal-Richtung projezierten Kurven
	ptarr=numpy.zeros((curve.ctrlPnts().n(),3))
	for n in range(curve.ctrlPnts().n()):
		ptarr[n]=[curve.ctrlPnts(n).getx(),curve.ctrlPnts(n).gety(),curve.ctrlPnts(n).getz()]
	### Plot zum Vergleich von Nurbskurve und diskreten Punkten
	arrr=numpy.zeros((1000,2))
	for i in range(1000):
		arrr[i,0]=curve.pointAt(i/1000.0).getx()
		arrr[i,1]=curve.pointAt(i/1000.0).gety()
	p.plot(arrr[:,0],arrr[:,1])
	p.plot(pts[:,0],pts[:,1],'bo')
	p.plot(ptarr[:,0],ptarr[:,1],'rx')
	p.axis('equal')
	p.show()
	
def plotRotNurbs(curve,curve2):
	import pylab as p
	disk = 1000.0
	
	arrj=numpy.zeros((int(disk),2))
	for i in xrange(int(disk)):
		arrj[i,0]=curve2.pointAt(i/(disk -1.0)).getx()
		arrj[i,1]=curve2.pointAt(i/(disk -1.0)).gety()
	p.plot(arrj[:,0],arrj[:,1])
	
	arrr=numpy.zeros((len(curve),int(disk),2))
	for n in xrange(len(curve)):
		for i in xrange(int(disk)):
			arrr[n,i,0]=curve[n].pointAt(i/(disk -1.0)).getx()
			arrr[n,i,1]=curve[n].pointAt(i/(disk -1.0)).gety()
		p.plot(arrr[n,:,0],arrr[n,:,1])
	
	#for i in xrange(int(disk)):
		#print str(sqrt((arrr[0,i,0]-arrj[i,0])**2.0 + (arrr[0,i,1]-arrj[i,1])**2.0))
		
	p.axis('equal')
	p.show()
	
def printProfile2D(pts):
	for n in range(len(pts)):
		p.plot(pts[n][:,0],pts[n][:,1])
	p.axis('equal')
	p.show()
