#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian Barthmes
import sys,os

from OCC.Display.qtDisplay import qtViewer3d
		
from PyQt4.QtGui import QLabel,QSplitter,QHBoxLayout,QTabWidget, QWidget, QLineEdit, QDoubleValidator, QIntValidator, QApplication, QDialog, QVBoxLayout, QPushButton
from PyQt4.QtCore import SIGNAL,  QVariant, Qt
from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB

class interactiveViewer(object):
	def __init__(self,turbomachine):
		self.tm=turbomachine
		self.displaylist=[]
		
		#Qt-Application erstellen
		pApp = QApplication([])
		pDlg = QDialog()
		pDlg.setLayout(QVBoxLayout())
		
		pDlg.resize(800,600)
		
		self.tab=QTabWidget(pDlg)
		pDlg.layout().addWidget(self.tab)
		pDlg.show()
		
		pDlg.connect(self.tab, SIGNAL('currentChanged(int)'), self.updateCurrent)
		
		self.tmTab()
		for row in self.tm.listRows:
			self.rowTab(row)
			
		self.updateCurrent(0)
		
		sys.exit(pApp.exec_())
	
	def updateCurrent(self,ind):
		size=self.tab.size()
		size.setHeight(size.height()+1)
		self.tab.resize(size)
		self.displaylist[ind].FitAll()
		
	def tmTab(self):
		rowWidget=QWidget()
		rowWidget.setLayout(QVBoxLayout())
		self.tab.addTab(rowWidget,"Turbomachine")
		self.tab.setCurrentIndex(self.tab.count()-1)
		
		Plot = qtViewer3d(self.tab)
		Plot.setObjectName("Plot")
		Plot.setMinimumSize(300,300)
		rowWidget.layout().addWidget(Plot)
		rowWidget.show()
		
		Plot.InitDriver()
		rowWidget.hide()
		
		self.tab.setCurrentIndex(0)
		
		display=Plot._display
		
		Plot.autoFillBackground=True
		Plot.setBackgroundColor=(Qt.white)
		
		Plot._display.View_Iso()
		##Passform
		Plot._display.FitAll()
		#Testmodus wechseln
		#Plot._display.Test()
		ais_context = display.GetContext().GetObject()
		#for elem in dir(display.GetView().GetObject()):
			#print elem
		dc = ais_context.DeviationCoefficient()
		dc_hlr = ais_context.HLRDeviationCoefficient()
		da = ais_context.DeviationAngle()
		da_hlr = ais_context.HLRAngle()
		
		#ais_context.SetDeviationCoefficient(dc/10)
		#ais_context.SetDeviationAngle(da/10)
		ais_context.SetHLRDeviationCoefficient(dc_hlr/3)
		ais_context.SetHLRAngle(da_hlr/3)
		
		bgcolor=Quantity_Color(1, 1, 1, Quantity_TOC_RGB)
		display.GetView().GetObject().SetBackgroundColor(bgcolor)
		
		#Basismaterial erstellen
		Material = Graphic3d_MaterialAspect(Graphic3d_NOM_SILVER)
		#Diffusestärke setzen
		Material.SetDiffuse(0.8)

		#Farben erstellen
		color = Quantity_Color(0.3, 0.3, 0.3, Quantity_TOC_RGB)
		#Farbe setzen
		Material.SetDiffuseColor(color)
		
		for row in self.tm.listRows:
			row.drawRow(display=Plot._display, erase=False, material=Material)
		
		self.displaylist.append(display)
			
	def rowTab(self,row):
		rowWidget=QWidget()
		rowWidget.setLayout(QHBoxLayout())
		self.tab.addTab(rowWidget,"Row "+str(row.rowNumber))
		self.tab.setCurrentIndex(self.tab.count()-1)
		
		Plot = qtViewer3d(self.tab)
		Plot.setObjectName("Plot")
		
		splitter=QSplitter()
		splitter.addWidget(Plot)
		rowWidget.layout().addWidget(splitter)
		rowWidget.show()
		
		Plot.setMinimumSize(300,300)
		Plot.setSizePolicy(QSizePolicy.Expanding,
			QSizePolicy.Expanding)
		Plot.InitDriver()
		display=Plot._display
		rowWidget.hide()
		
		self.tab.setCurrentIndex(0)
		
		
		Plot.autoFillBackground=True
		Plot.setBackgroundColor=(Qt.white)
		
		Plot._display.View_Iso()
		##Passform
		Plot._display.FitAll()
		#Testmodus wechseln
		#Plot._display.Test()
		ais_context = display.GetContext().GetObject()
		#for elem in dir(display.GetView().GetObject()):
			#print elem
		dc = ais_context.DeviationCoefficient()
		dc_hlr = ais_context.HLRDeviationCoefficient()
		da = ais_context.DeviationAngle()
		da_hlr = ais_context.HLRAngle()
		
		#ais_context.SetDeviationCoefficient(dc/10)
		#ais_context.SetDeviationAngle(da/10)
		ais_context.SetHLRDeviationCoefficient(dc_hlr/10)
		ais_context.SetHLRAngle(da_hlr/10)
		
		bgcolor=Quantity_Color(1, 1, 1, Quantity_TOC_RGB)
		display.GetView().GetObject().SetBackgroundColor(bgcolor)
		
		#Basismaterial erstellen
		Material = Graphic3d_MaterialAspect(Graphic3d_NOM_SILVER)
		#Diffusestärke setzen
		Material.SetDiffuse(0.8)

		#Farben erstellen
		color = Quantity_Color(0.3, 0.3, 0.3, Quantity_TOC_RGB)
		#Farbe setzen
		Material.SetDiffuseColor(color)
		row.drawBladeSurface(display=Plot._display, material=Material)
		
		splitter.addWidget(self.plot2DWidget(row))
		
		self.displaylist.append(display)
		
		
	def plot2DWidget(self,row):		
		if row.useNurbsXml==True:
			widget=QLabel("2D Profile Plot only avaliable, if calculated completely (useNurbsXml=False).")
		else:
			plot=MplCanvas()
			toolbar=NavigationToolbar(plot,plot)
			
			for n in range(row.numberProfiles):
				for pts in [row.listProfiles[n].upperProfile,
							row.listProfiles[n].lowerProfile,
							row.listProfiles[n].camberLine]:
					plot.drawPts(pts)
			
			l=QVBoxLayout()
			l.addWidget(plot)
			l.addWidget(toolbar)
			widget=QWidget()
			widget.setLayout(l)
		
		return widget
		
		
		
from PyQt4.QtCore import QSize, SIGNAL, SLOT
from PyQt4.QtGui import QWidget, QVBoxLayout, QComboBox, QSizePolicy, QHBoxLayout, QIcon, QPixmap
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4 import NavigationToolbar2QT as NavigationToolbar

from matplotlib.figure import Figure

import numpy as N



class MplCanvas(FigureCanvas):
	def __init__(self, pParent=None, width = 10, height = 12, dpi = 100, sharex = None, sharey = None):
		## @brief Graphic to be shown<br><em>Grafik die angezeigt wird</em>
		self.fig = Figure(figsize = (width, height), dpi=dpi, facecolor = '#FFFFFF')
		## @brief Subgraphic<br><em>Untergrafik</em>
		self.ax = self.fig.add_subplot(111, sharex = sharex, sharey = sharey)
		self.fig.subplots_adjust(left=0.1, bottom=0.1, right=0.9, top=0.9)
		## @brief Labeling of x-axis<br><em>Beschriftungen der X-Achse</em>
		self.xtitle="m"
		## @brief Labeling of y-axis<br><em>Beschriftungen der Y-Achse</em>
		self.ytitle="y'"
		## @brief Header of the graphical window<br><em>Titel des Zeichenfenster</em>
		self.PlotTitle = "2D Profile Plot"
		## @brief State of the grid<br><em>Status des Gitters</em>
		self.grid_status = True
		## @brief Type of x-axis<br><em>Art der X-Achse</em>
		self.xaxis_style = 'linear'
		## @brief Type of y-axis<br><em>Art der Y-Achse</em>
		self.yaxis_style = 'linear'
		self.format_labels()
		self.ax.hold(True)
		FigureCanvas.__init__(self, self.fig)
		#self.fc = FigureCanvas(self.fig)
		FigureCanvas.setSizePolicy(self,
			QSizePolicy.Expanding,
			QSizePolicy.Expanding)
		FigureCanvas.updateGeometry(self)

	def format_labels(self):
		self.ax.set_title(self.PlotTitle)
		self.ax.title.set_fontsize(10)
		self.ax.set_xlabel(self.xtitle, fontsize = 9)
		self.ax.set_ylabel(self.ytitle, fontsize = 9)
		labels_x = self.ax.get_xticklabels()
		labels_y = self.ax.get_yticklabels()

		for xlabel in labels_x:
			xlabel.set_fontsize(8)
		for ylabel in labels_y:
			ylabel.set_fontsize(8)
			ylabel.set_color('b')
			
	def sizeHint(self):
		w, h = self.get_width_height()
		return QSize(w, h)

	def minimumSizeHint(self):
		return QSize(100 , 100)
		
	def drawPts(self,pts):
		self.ax.plot(pts[:,0],pts[:,1],'bo',markersize=2)
		self.ax.axis('equal')
		
		