#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Oliver Borm                                *
#*   oli.borm@web.de                                                       *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Oliver Borm
# @author Julius Fellerhoff
# @author Sebastian G. Barthmes
# @date 27.05.2009


# Python Imports
# üöä
import os, sys, getopt, string

from xml.dom import minidom

import bladeRow


## Version of BladeDesigner3D
BladeDesignerVersion = "2010.12.03"

## @class turboMachine
# @brief class for turbomachine definition and management of bladeRow and profile2D
class turboMachine(object):
	
	## @brief Constructor
	def __init__(self,workdir,*args):
		## array which holds the configuration of the turbomachine. The length is the number of rows, the values describe the number of profiles in the according row.
		self.configuration = []
		## number of Rows in the Turbomachine
		self.numberRows=0
		## list where all Rows are stored
		self.listRows=[]
		## flag for debug mode
		self.debug=0
		## dataStructure-object, contains all data about the working directories and their structure
		self.dataStruct=dataStructure(workdir)
		## Progress function is only called from the GUI and also asigned there
		self.step = lambda: None
		## OCC display
		self.display=None
		
		#Variante 1: Initialisierung über eine Liste die die Anzahl der Reihen und der Profile enthält
		#Variante 2: Initialisierung mit einem xml, ohne weitere Angabe eines Argumentes (xml muss unter dem workdir liegen)
		for arg in args:
			if type(arg)==list:
				self.configuration=arg
				if os.path.exists(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml'):
					raise Exception, "Xml detected in workdir, but initialized with configuration-list! Do not use both."
				self.setNumberRows(len(self.configuration))
			else:
				raise Exception, "Wrong usage: <Working Directory> + <Configuration List> or <Working Directory containing xml-project> expected"
				
		if os.path.exists(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml'):
			#finde die Konfiguration aus dem xml heraus und lege die Row/Profile Objekte im Speicher an; initialisiere alle Objekte mit Werten
			self.readTurboMachineXml(self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml')
		

	## @brief call function. Calls the method computeTurboMachine()
	def __call__(self):
		self.computeTurboMachine()

	## @brief starts the computation of the whole turbomachine
	def computeTurboMachine(self):
		if self.debug==1:
			print "*---------------------------------------------------------*"
			print "|               ***  BladeDesigner3D  ***                 |"
			print "|                Version:      "+BladeDesignerVersion+"                 |"
			print "*---------------------------------------------------------*"
			print "Copyright (C) 2008-2010 B. Balassa, S. Barthmes, O. Borm, J. Fellerhoff,"
			print "A. Kührmann, S. Walter"
			print "Licenced under GPL v3"
			print "This program comes with ABSOLUTELY NO WARRANTY. This is free"
			print "software, and you are welcome to redistribute it under certain conditions."
			# # 	http://www.gnu.org/licenses/gpl-3.0.html#howto
			print "\n***\n*** Computing Turbomachine...\n***"
			
			
		#SW: Erster Schritt geschafft
		self.step()
		
		for n in range(self.numberRows):
			self.listRows[n]()
			#SW: Wieder ein Schritt geschafft
			self.step()
			
		if self.debug==1:
			print "\n***\n*** Turbomachine computation completed!\n***"

		#SW: Fertig mit Turbomachinenberechnung
		self.step()

	## @brief function to initialize the number of bladeRow-Objects
	def setNumberRows(self,nr):
		if nr == 0:
			raise Exception, "Value > 0 expected."
		if self.numberRows != 0:
			raise Exception, "Variable numberRows already set."
		self.numberRows = nr
		for n in range(self.numberRows):
			self.listRows.append(bladeRow.bladeRow([n,self.configuration[n]],self.dataStruct))
		## scm mesh object
		try:
			import SCMMesher
			self.scmMesh = SCMMesher.SCMMesher(self)
		except:
			print sys.exc_info()[1],"\n!!! scmMesh not available. Skipping Import."
			pass
	
	def computeScmMesh(self):
		try:
			self.scmMesh()
		except:
			print sys.exc_info()[1],"\n!!! scmMesh not available. Skipping."
			pass
		
	## @brief calculates the configuration-list from the xml-file and inits the turbomachine-object and all child-objects (rows, profiles) from the given xml
	def readTurboMachineXml(self,filename):
		
		if self.debug==1:
			print '*** Reading',filename
		
		dom_object = minidom.parse(filename)

		xml_doc = dom_object.toxml("utf-8")
		
		xml_doc=xml_doc.replace('\n','')
		xml_doc=xml_doc.replace('\t','')
		
		dom_object = minidom.parseString(xml_doc)

		elementMach = dom_object.getElementsByTagName("turboMachine")[0]

		for n in range(len(elementMach.getElementsByTagName("bladeRow"))):
			elementRow = elementMach.getElementsByTagName("bladeRow")[n]
			
			elementProfmax = elementRow.lastChild
			ProfileNum = int(elementProfmax.getAttribute("i")) + 1
			self.configuration.append(ProfileNum)	

		self.setNumberRows(len(self.configuration))

		for n in range(len(self.configuration)):
			elementRow = elementMach.getElementsByTagName("bladeRow")[n]
			self.listRows[n].readRowXmlFile(dom_object,elementRow)

			self.listRows[n].debug=1
		
		
	## @brief writes the turbomachine and all child-objects (rows, profiles) to an xml-file
	def writeTurboMachineXml(self,xmlWriteMode):
		
		filename = self.dataStruct.workdir+'/'+self.dataStruct.projectname+'.xml'
		
		if os.path.exists(filename) == False:
			xmlWriteMode = 0
		
		if xmlWriteMode == 0:
			xml_doc = """<?xml version="1.0" encoding="UTF-8"?>
			<turboMachine/>""".encode("utf-8")
			
			dom_object = minidom.parseString(xml_doc)
			
			elementTurb = dom_object.getElementsByTagName("turboMachine")[0]
			
			RowNum = 0
		else:
			dom_object = minidom.parse(filename)
			
			xml_doc = dom_object.toxml("utf-8")
			
			xml_doc=xml_doc.replace('\n','')
			xml_doc=xml_doc.replace('\t','')
			
			dom_object = minidom.parseString(xml_doc)
			
			elementTurb = dom_object.getElementsByTagName("turboMachine")[0]
			
			elementRow = elementTurb.lastChild
			RowNum = int(elementRow.getAttribute("i"))+1
		
		#turboMachine flags for xml...
		
		for n in range(self.numberRows):
			newElement = dom_object.createElement("bladeRow")
			newElement.setAttribute("i",str(n+RowNum))
			elementTurb.appendChild(newElement)
			
			elementRow = elementTurb.getElementsByTagName("bladeRow")[n+RowNum]
			#self.listRows[n].writeRowXmlFile(filename,self.xmlWriteMode)
			self.listRows[n].writeRowXmlFile(dom_object,elementRow)
			
		if self.debug==1:
			print '*** Writing',filename
		
		file = open(filename, "w")
		
		dom_object.writexml(file, "", "\t", "\n","utf-8")
		
		file.close()
		

	## @brief draws the turboMachine
	# @param display the OCC display handler
	def drawTurboMachine(self,display=None, materials=None):
		if display==None:
			if self.display==None:
				raise Exception, "Draw Method has no display."
			else:
				display=self.display
		
		display.EraseAll()

		#Materialien vorhanden?
		useMaterials = False

		#Materialien vorhanden und auch für jede BladeRow eine andere?
		if(materials and (len(materials) == len(self.listRows))):
			useMaterials = True
		
		for index, row in enumerate(self.listRows):
			row.display=display

			#SW:Ich hab das mal um Materialien erweitert
			if(useMaterials):
				row.drawRow(erase=False, material=materials[index])
			else:
				row.drawRow(erase=False)
			
		display.FitAll()
		
	
	## @brief display a pythonocc based viewer to analyse the geometry of rows/profiles and the turboMachine
	def displayViewer(self):
		try:
			import OCC.Display.SimpleGui
			#from OCC.Graphic3d import Graphic3d_MaterialAspect, Graphic3d_NOM_SILVER
			#from OCC.Quantity import Quantity_Color, Quantity_TOC_RGB
		except:
			print "!!! pythonocc (OpenCASCADE for python) not found. Display not possible."
		else:
			OCC.Display.SimpleGui.set_backend('qt')
			self.display, start_display, add_menu, add_function_to_menu = OCC.Display.SimpleGui.init_display()
			
			#bgcolor=Quantity_Color(1, 1, 1, Quantity_TOC_RGB)
			#self.display.GetView().GetObject().SetBackgroundColor(bgcolor)
			
			#del self.display.GetView().GetObject().BackgroundImage
			
			if self.listRows[0].definition==2:
				for row in self.listRows:
					self.listRows[0].drawBladeSurface(display=self.display)
			else:
				add_menu("Turbomachine")
				add_function_to_menu("Turbomachine",self.drawTurboMachine)
				
				for row in self.listRows:
					row.display=self.display
					
					menu_name="Blade Row "+str(row.rowNumber)
					add_menu(menu_name)
					for f in [row.drawNurbsCurves,
							row.drawBladeSurface,
							row.drawRow]:
						add_function_to_menu(menu_name, f)
				
				self.drawTurboMachine()
			
			start_display()


	def interactiveViewer(self):
		import interactiveViewer
		
		view=interactiveViewer.interactiveViewer(self)
		
		
	## @brief export bladeSurface to IGES File
	# @param filename the file path
	def exportTurboMachineToIGES(self,filename="turbomachine.iges"):
		try:
			import OCC.Utils.DataExchange.IGES
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=self.dataStruct.workdir+self.dataStruct.dirGeom+'/'
			
			exporter = OCC.Utils.DataExchange.IGES.IGESExporter(path+filename,format="5.3")
			
			for blade in self.listRows:
				bladesurf = blade.getGeom_BSplineSurface()
				bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
				bladesurf_rep.Build()
				exporter.AddShape( bladesurf_rep.Shape() )
				
			exporter.WriteFile()
			print "*** Successfully written to",path+filename
	
	
	## @brief export bladeSurface to IGES File
	# @param filename the file path
	def exportTurboMachineToSTEP(self,filename="turbomachine.stp"):
		try:
			import OCC.Utils.DataExchange.STEP
			import OCC.BRepBuilderAPI 
		except:
			"!!! pythonocc (OpenCASCADE for python) not found. Export not possible."
		else:
			path=self.dataStruct.workdir+self.dataStruct.dirGeom+'/'
			
			exporter = OCC.Utils.DataExchange.STEP.STEPExporter(filename)
			
			for blade in self.listRows:
				bladesurf = blade.getGeom_BSplineSurface()
				bladesurf_rep = OCC.BRepBuilderAPI.BRepBuilderAPI_MakeFace( bladesurf.GetHandle() )
				bladesurf_rep.Build()
				exporter.AddShape( bladesurf_rep.Shape() )
			
			exporter.WriteFile()
			print "*** Successfully written to",path+filename
	


## @brief class for the data structure of turbomachine projects
class dataStructure(object):
	# @brief Constructor
	# @param path the path to the data structure. The name of the folder specifies the projectname	
	def __init__(self, path):
		self.workdir = os.path.abspath(path)
		self.projectname = os.path.basename(self.workdir) #self.workdir.split('/')[-1]
		self.dirGeom='/geometry'
		self.dir1D='/1D'
		self.dir2DS2m='/2D-S2m'
		self.dir2D='/2D'
		self.dir2DS1='/2D-S1'
		self.dir3D='/3D'
		self.dir3Dc='/cascade'
		self.geomRow='/bladeRow_'
	
	## @brief creates all subdirectories
	def createDirs(self):
		self.cncDir(self.workdir)
		self.cncDir(self.workdir+self.dirGeom)
		self.cncDir(self.workdir+self.dir1D)
		self.cncDir(self.workdir+self.dir2DS2m)
		self.cncDir(self.workdir+self.dir2DS1)
		self.cncDir(self.workdir+self.dir3D)
		self.cncDir(self.workdir+self.dir3Dc)
	
	## @brief add a row dir to the geometry dir
	# @param n the row number
	def addGeomRow(self,n):
		self.cncDir(self.workdir)
		self.cncDir(self.workdir+self.dirGeom)
		self.cncDir(self.workdir+self.dirGeom+self.geomRow+str(n))
	
	## @brief add a new computation to 2DS2m dir
	# @param computationName the name of the directory created
	def add2DS2mComputation(self,computationName='computation'):
		self.addComputation(self.dir2DS2m,computationName)
	
	## @brief add a new computation to 2D dir
	# @param computationName the name of the directory created
	# @param addOFstruct flag for OpenFOAM Structure generation
	def add2DComputation(self,computationName='computation',  addOFstruct=False):
		self.addComputation(self.dir2D,computationName,addOFstruct)
	
	## @brief add a new computation to 2DS1 dir
	# @param computationName the name of the directory created
	# @param addOFstruct flag for OpenFOAM Structure generation
	def add2DS1Computation(self,computationName='computation', addOFstruct=False):
		self.addComputation(self.dir2DS1,computationName, addOFstruct)
		
	## @brief add a new computation to 3Dc dir
	# @param computationName the name of the directory created
	# @param addOFstruct flag for OpenFOAM Structure generation
	def add3DcComputation(self,computationName='computation', addOFstruct=False):
		self.addComputation(self.dir3Dc,computationName, addOFstruct)
		
	## @brief add a new computation to 3D dir
	# @param computationName the name of the directory created
	# @param addOFstruct flag for OpenFOAM Structure generation
	def add3DComputation(self,computationName='computation', addOFstruct=False):
		self.addComputation(self.dir3D,computationName, addOFstruct)
	
	## @brief add a new computation to an arbitrary dir
	# @param dire the name ob the subfolder
	# @param computationName the name of the directory created
	# @param addOFstruct flag for OpenFOAM Structure generation
	def addComputation(self,dire,computationName,addOFstruct=False):
		#i=True
		#n=0
		#while i==True:
			#if os.path.exists(self.workdir+dire+'/'+computationName+'_'+str(n)):
				#n+=1
			#else:
				#break
		self.cncDir(self.workdir)
		self.cncDir(self.workdir+dire)
		#self.cncDir(self.workdir+dire+'/'+computationName+'_'+str(n))
		self.cncDir(self.workdir+dire+'/'+computationName)
		if addOFstruct==True:
			#addOFStructure(self.workdir+dire+'/'+computationName+'_'+str(n))
			self.addOFStructure(self.workdir+dire+'/'+computationName)
	
	## @brief create an OpenFOAM directory structure below a basepath
	# @param the path to the structure
	def addOFStructure(self,base):
		self.cncDir(base+'/0')
		self.cncDir(base+'/constant')
		self.cncDir(base+'/constant/polyMesh')
		self.cncDir(base+'/system')

	## @brief check for existance and create a directory at path
	# @param path the path/to/dir
	def cncDir(self,path):
		if not os.path.exists(path):
			os.mkdir(path)
	


