#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Andreas Kührmann                           *
#*   andreas.kuehrmann@gmail.com                                           *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
#  @author Andreas Kührmann
#  @date 2009

# Python Imports
# üöä

from nurbsToolSet import getCurveCoordinates, movePoint
from blockMeshDictGenToolSet import computeL, getDomainCoordinates

class write(object):
	
	def __init__(self):
		
		# bool
		self.cyclicGgi  = False
		self.symCascade = False
		# int
		self.centerStart    = 10
		self.debug          =  0
		self.dimensionMode  =  0
		self.inletStart     =  0
		self.maxIteration   = 75
		self.noBlocks       = 16
		self.noBlocksInlet  =  3
		self.noBlocksCenter =  4
		self.noBlocksOutlet =  3
		self.noLayer        =  2
		self.outletStart    = 22
		self.rowNumber      =  0
		self.vertexPerLayer = 26 
		# float
		self.convertToMeters =  1.0
		self.domainLength    = 70.0
		self.pitch           = 10.0
		self.tolerance       = 1e-9
		self.version         =  1.6
		# string
		self.dimension = '2D'
		self.fileName  = 'blockMeshDict'
		self.gridType  = 'fS'
		self.path      = 'projekt/openFOAM/constant/polyMesh'
		self.zoneName  = 'rotor'
		# array
		self.bladePatches       = []
		self.blockCoordinates   = []
		self.blockGrading       = []
		self.cellsPerBlock      = []
		self.centerCoordinates  = []
		self.curve              = []
		self.hubPatches         = []
		self.inletCoordinates   = []
		self.inletPatches       = []
		self.camberLine         = []
		self.lOnCurve           = []
		self.outletCoordinates  = []
		self.outletPatches      = []
		self.passageSidePatches = []
		self.shroudPatches      = []
		self.passageSide        = []
		self.revolutionCurve    = []
		self.smeVertex          = []
		self.uOnCurve           = []
		self.vertexCoordinates  = []
	
	
	## @brief start command for writing blockMeshDict file
	# @param	self		Objekt pointer
	def blockMeshDictFile(self):
		
		if self.debug == 1: print '*** Writing header'
		self.fileHead()
		
		if self.debug == 1: print '*** Writing scale factor'
		self.scale()
		
		if self.debug == 1: print '*** Writing vertices'
		self.vertices()
		
		if self.debug == 1: print '*** Writing blocks'
		self.blocks()
		
		if self.debug == 1: print '*** Writing edges'
		self.edges()
		
		if self.debug == 1: print '*** Writing pathches\n'
		self.patches()
	
	
	## @brief writes head to file
	# @param	self		Objekt pointer
	def fileHead(self):
		
		
		fOut = open(self.path+self.fileName, 'w', -1)
		
		fOut.write('/*---------------------------------------------------------------------------*\ \n')
		fOut.write('| =========                 |                                                 |\n')
		fOut.write('| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
		fOut.write('|  \\\\    /   O peration     | Version:  '+str(self.version)+'                                   |\n')
		fOut.write('|   \\\\  /    A nd           | Web:      http://www.openfoam.org               |\n')
		fOut.write('|    \\\\/     M anipulation  |                                                 |\n')
		fOut.write('\*---------------------------------------------------------------------------*/\n\n')
		fOut.write('FoamFile\n')
		fOut.write('{\n')
		fOut.write('    version         2.0;\n')
		fOut.write('    format          ascii;\n\n')
		fOut.write('    root            "";\n')
		fOut.write('    case            "";\n')
		fOut.write('    instance        "";\n')
		fOut.write('    local           "";\n\n')
		fOut.write('    class           dictionary;\n')
		fOut.write('    object          blockMeshDict;\n')
		fOut.write('}\n\n')
		fOut.write('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n\n\n')
		
		fOut.close()
	
	
	## @brief writes scale to file
	# @param	self		Objekt pointer
	def scale(self):
		
		fOut=open(self.path+self.fileName, 'a', -1)
		fOut.write("convertToMeters "+str(self.convertToMeters)+";\n\n")
		fOut.close()
	
	
	## @brief writes coordinates of vertices to file
	# @param	self		Objekt pointer
	def vertices(self):
		
		fOut = open(self.path+self.fileName, 'a', -1)
		fOut.write('vertices\n')
		fOut.write('(\n')
		
		for layerIndex in xrange(self.noLayer):
			self.noVertexPerLayer = len(self.vertexCoordinates[layerIndex])
			
			for vertexIndex in xrange(self.noVertexPerLayer):
				fOut.write('    (')
				
				for xyz in xrange(3):
					fOut.write(str(self.vertexCoordinates[layerIndex, vertexIndex, xyz]))
					
					if xyz != 2:
						fOut.write(' ')
						
				fOut.write(')   // point number '+str(vertexIndex+self.noVertexPerLayer*layerIndex)+'\n')
				
		fOut.write(');\n\n')
	
	
	## @brief writes data for block generation to file
	# @param	self		Objekt pointer
	def blocks(self):
		
		fOut = open(self.path+self.fileName, 'a', -1)
		fOut.write('blocks\n')
		fOut.write('(\n')
		
		for blockIndex in xrange(len(self.blockCoordinates)):
			
			# schreiben der Blockkoordinaten als Funktion der Netzpunkte
			fOut.write('    hex (')
			
			for coordinat in xrange(8):
				fOut.write(str(self.blockCoordinates[blockIndex, coordinat]))
				
				if coordinat != 7:
					fOut.write(' ')
					
			fOut.write(')')
			
			# schreiben des Zonennamens -> openFOAM Manual
			if self.zoneName !=  '':
				fOut.write(' '+self.zoneName+' ')
				
			# schreiben der Zellzahl pro Dimension
			fOut.write('(')
			
			for xyz in xrange (3):
				fOut.write(str(self.cellsPerBlock[blockIndex, xyz]))
				
				if xyz != 2:
					fOut.write(' ')
				
			fOut.write(') ')
			
			# schreiben des Zellverteilung auf der Blockkante
			fOut.write('edgeGrading (')
			
			for edgeIndex in xrange (12):
				fOut.write(str(self.blockGrading[blockIndex, edgeIndex]))
				
				if edgeIndex != 11:
					fOut.write(' ')
				
			fOut.write(')')
			
			# laufende Blocknummer als Kommentar schreiben
			fOut.write('   // block number '+str(blockIndex)+'\n')
			
		fOut.write(');\n\n')
		fOut.close()
	
	
	## @brief writes data for edges to file
	# @param	self		Objekt pointer
	def edges(self):
		
		fOut=open(self.path+self.fileName, 'a', -1)
		fOut.write('edges\n')
		fOut.write('(\n')
		
		direction = [1, -1] 
		
		for layerIndex in xrange(self.noLayer):
			
			if self.gridType == 'fS':
				
				domainLength = self.domainLength
				inputCS = 'xyz'
				
			elif self.gridType == 'rS':
				
				inputCS = 'rpz'
				domainLength = self.revolutionCurve[layerIndex]
				
				self.noVertexPerLayer = len(self.vertexCoordinates[layerIndex])
				
				# inlet edge
				for blockIndex in xrange(self.noBlocksInlet):
					
					fOut.write('    arc '+str(blockIndex+self.inletStart+layerIndex*self.noVertexPerLayer)+' '+str(blockIndex+self.inletStart+1+layerIndex*self.noVertexPerLayer)+' (')
					
					for xyz in xrange(3):
						
						fOut.write(str(self.inletCoordinates[layerIndex, blockIndex, xyz]))
						
						if xyz != 2:
							
							fOut.write(' ')
							
					fOut.write(')\n')
					
				# outlet edge
				for blockIndex in xrange(self.noBlocksOutlet):
					
					fOut.write('    arc '+str(blockIndex+self.outletStart+layerIndex*self.noVertexPerLayer)+' '+str(blockIndex+self.outletStart+1+layerIndex*self.noVertexPerLayer)+' (')
					
					for xyz in xrange(3):
						
						fOut.write(str(self.outletCoordinates[layerIndex, blockIndex, xyz]))
						
						if xyz != 2:
							
							fOut.write(' ')
							
					fOut.write(')\n')
					
				# center line
				for blockIndex in xrange(self.noBlocksCenter):
					
					if blockIndex != (self.noBlocksCenter+1)/2:
						
						fOut.write('    arc '+str(blockIndex+self.centerStart+layerIndex*self.noVertexPerLayer)+' '+str(blockIndex+self.centerStart+1+layerIndex*self.noVertexPerLayer)+' (')
						
						for xyz in xrange(3):
							
							fOut.write(str(self.centerCoordinates[layerIndex, blockIndex, xyz]))
							
							if xyz != 2:
								
								fOut.write(' ')
								
						fOut.write(')\n')
						
			for curveIndex in xrange(len(self.curve[layerIndex])):
				
				blockIndex = 0
				
				fOut.write('    polyLine '+str(self.smeVertex[curveIndex][blockIndex]+layerIndex*self.noVertexPerLayer)+' '+str(self.smeVertex[curveIndex][blockIndex+1]+layerIndex*self.noVertexPerLayer)+' (')
				
				for uIndex in xrange(len(self.uOnCurve[layerIndex][curveIndex])):
					
					if self.uOnCurve[layerIndex][curveIndex][uIndex] == 's':
						
						fOut.write(')\n')
						
						if uIndex != len(self.uOnCurve[layerIndex][curveIndex])-1:
							
							blockIndex = blockIndex+1
							
							fOut.write('    polyLine '+str(self.smeVertex[curveIndex][blockIndex]+layerIndex*self.noVertexPerLayer)+' '+str(self.smeVertex[curveIndex][blockIndex+1]+layerIndex*self.noVertexPerLayer)+' (')
							
					else:
						
						fOut.write('(')
						
						coords = getCurveCoordinates(self.uOnCurve[layerIndex][curveIndex][uIndex], self.curve[layerIndex][curveIndex], "xyz")
						
						for xyz in xrange(3):
							
							fOut.write(str(coords[xyz]))
							
							if xyz != 2:
								
								fOut.write(' ')
								
							else:
								
								fOut.write(')')
								
			for side in xrange(2):
				
				blockIndex = 0
				
				lEdge = computeL(0, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
				tEdge = computeL(1, self.camberLine[layerIndex], domainLength, self.gridType, self.tolerance, self.maxIteration)
				
				fOut.write('    polyLine '+str(self.passageSide[side][blockIndex]+layerIndex*self.vertexPerLayer)+' '+str(self.passageSide[side][blockIndex+1]+layerIndex*self.vertexPerLayer)+' (')
				
				for lIndex in xrange(len(self.lOnCurve[layerIndex])):
					
					if self.lOnCurve[layerIndex][lIndex] == 's':
						
						fOut.write(')\n')
						
						if lIndex != len(self.lOnCurve[layerIndex])-1:
							
							blockIndex = blockIndex+1
							
							fOut.write('    polyLine '+str(self.passageSide[side][blockIndex]+layerIndex*self.vertexPerLayer)+' '+str(self.passageSide[side][blockIndex+1]+layerIndex*self.vertexPerLayer)+' (')
							
					else:
						
						fOut.write('(')
						
						if self.lOnCurve[layerIndex][lIndex] <= lEdge:
							
							pitchOffset = ((0.0-self.inletOffset[layerIndex, 0])/(lEdge)*self.lOnCurve[layerIndex][lIndex]+self.inletOffset[layerIndex, 0])*direction[side]
							
						elif self.lOnCurve[layerIndex][lIndex] >= tEdge:
							
							grad = (self.outletOffset[layerIndex, 0])/(1.0-tEdge)
							
							pitchOffset = (grad*self.lOnCurve[layerIndex][lIndex]-(grad*tEdge))*direction[side]
							
						else:
							
							pitchOffset = 0.0
						
						tmpCoords = getDomainCoordinates(self.lOnCurve[layerIndex][lIndex], self.camberLine[layerIndex], domainLength, self.gridType, inputCS, self.tolerance, self.maxIteration)
						coords    = movePoint(tmpCoords, 0, direction[side]*(self.pitch*(0.5+pitchOffset)), 0, inputCS, "xyz")
						
						for xyz in xrange(3):
							
							fOut.write(str(coords[xyz]))
							
							if xyz != 2:
								
								fOut.write(' ')
								
							else:
								
								fOut.write(')')
		fOut.write(');\n\n')
		fOut.close()
	
	
	## @brief writes data for patches to file 'fileName'
	# @param	self		Objekt pointer
	def patches(self):
		
		fOut=open(self.path+self.fileName, 'a', -1)
		fOut.write('patches\n')
		fOut.write('(\n')
		
		#  write inlet
		fOut.write('    patch inlet_'+str(self.rowNumber)+' (')
		
		for inletPatchIndex in xrange(len(self.inletPatches)):
			fOut.write("("+str(self.inletPatches[inletPatchIndex, 0])+" "+str(self.inletPatches[inletPatchIndex, 1])+" "+str(self.inletPatches[inletPatchIndex, 2])+" "+str(self.inletPatches[inletPatchIndex, 3])+")")
			
		fOut.write(')\n')
		
		# write outlet
		fOut.write('    patch outlet_'+str(self.rowNumber)+' (')
		
		for outletPatchIndex in xrange(len(self.outletPatches)):
			fOut.write("("+str(self.outletPatches[outletPatchIndex, 0])+" "+str(self.outletPatches[outletPatchIndex, 1])+" "+str(self.outletPatches[outletPatchIndex, 2])+" "+str(self.outletPatches[outletPatchIndex, 3])+")")
			
		fOut.write(')\n')
		
		# write passage sides
		if self.cyclicGgi == False:
			fOut.write('    cyclic passageSides_'+str(self.rowNumber)+' (')
			
			for passageSidePatchIndex in xrange(len(self.passageSidePatches)):
				fOut.write("("+str(self.passageSidePatches[passageSidePatchIndex, 0])+" "+str(self.passageSidePatches[passageSidePatchIndex, 1])+" "+str(self.passageSidePatches[passageSidePatchIndex, 2])+" "+str(self.passageSidePatches[passageSidePatchIndex, 3])+")")
				
			fOut.write(')\n')
			
		else:
			
			fOut.write('    cyclicGgi passageSideLP_'+str(self.rowNumber)+' (')
			
			for passageSidePatchIndex in xrange(len(self.passageSidePatches)/2):
				fOut.write("("+str(self.passageSidePatches[passageSidePatchIndex, 0])+" "+str(self.passageSidePatches[passageSidePatchIndex, 1])+" "+str(self.passageSidePatches[passageSidePatchIndex, 2])+" "+str(self.passageSidePatches[passageSidePatchIndex, 3])+")")
				
			fOut.write(')\n')
			
			fOut.write('    cyclicGgi passageSideUP_'+str(self.rowNumber)+' (')
			
			for passageSidePatchIndex in xrange(len(self.passageSidePatches)/2, len(self.passageSidePatches)):
				fOut.write("("+str(self.passageSidePatches[passageSidePatchIndex, 0])+" "+str(self.passageSidePatches[passageSidePatchIndex, 1])+" "+str(self.passageSidePatches[passageSidePatchIndex, 2])+" "+str(self.passageSidePatches[passageSidePatchIndex, 3])+")")
				
			fOut.write(')\n')
			
		# write hub 
		if self.gridType == 'rS':
			if self.dimension == "2D":
				fOut.write('    symmetryPlane hub_'+str(self.rowNumber)+' (')
				
			elif self.dimension == "3D":
				fOut.write('    wall hub_'+str(self.rowNumber)+' (')
				
			else:
				fOut.write('    !failed! hub_'+str(self.rowNumber)+' (')
				
		if self.gridType == 'fS':
			
			if self.dimension == "2D":
				fOut.write('    empty hub_'+str(self.rowNumber)+' (')
				
			elif self.dimension == "3D":
				fOut.write('    wall hub_'+str(self.rowNumber)+' (')
				
			else:
				fOut.write('    !failed! hub_'+str(self.rowNumber)+' (')
				
		for hubPatchIndex in xrange(len(self.hubPatches)):
			fOut.write("("+str(self.hubPatches[hubPatchIndex, 0])+" "+str(self.hubPatches[hubPatchIndex, 1])+" "+str(self.hubPatches[hubPatchIndex, 2])+" "+str(self.hubPatches[hubPatchIndex, 3])+")")
			
		fOut.write(')\n')
		
		# write shroud
		if self.gridType == 'rS':
			if self.dimension == "2D": 
				fOut.write('    symmetryPlane shroud_'+str(self.rowNumber)+' (')
				
			elif self.dimension == "3D":
				fOut.write('    wall shroud_'+str(self.rowNumber)+' (')
				
			else:
				fOut.write('    !failed! shroud_'+str(self.rowNumber)+' (')
				
		if self.gridType == 'fS':
			if self.dimension == "2D":
				fOut.write('    empty shroud_'+str(self.rowNumber)+' (')
				
			elif self.dimension == "3D":
				if self.symCascade  == True:
					fOut.write('    symmetryPlane shroud_'+str(self.rowNumber)+' (')
					
				else:
					fOut.write('    wall shroud_'+str(self.rowNumber)+' (')
				
			else:
				fOut.write('    !failed! shroud_'+str(self.rowNumber)+' (')
				
		for shroudPatchIndex in xrange(len(self.shroudPatches)):
			fOut.write("("+str(self.shroudPatches[shroudPatchIndex, 0])+" "+str(self.shroudPatches[shroudPatchIndex, 1])+" "+str(self.shroudPatches[shroudPatchIndex, 2])+" "+str(self.shroudPatches[shroudPatchIndex, 3])+")")
			
		fOut.write(')\n')
		
		fOut.write('    wall blade_'+str(self.rowNumber)+' (')
		
		# write blade
		for bladePatchIndex in xrange(len(self.bladePatches)):
			fOut.write("("+str(self.bladePatches[bladePatchIndex, 0])+" "+str(self.bladePatches[bladePatchIndex, 1])+" "+str(self.bladePatches[bladePatchIndex, 2])+" "+str(self.bladePatches[bladePatchIndex, 3])+")")
			
		fOut.write(')\n')
		
		fOut.write('\n);\n\n')
		fOut.close()
	
