#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2009-2009 by Oliver Borm                                *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

#Python Imports
#import os, sys, getopt, string

import sys,os
sys.path.insert(0 ,"../../src")
import turboMachine
import camberSurface

import numpy

#Turbomachine Initialisation
workdir=os.path.abspath('rotor2D')   # '/path/to/workdir'
configuration=[1]

turbo=turboMachine.turboMachine(workdir,configuration)
turbo.debug=1

## Rowdefinition
turbo.listRows[0].useNurbsXml=False

turbo.listRows[0].definition = 2

## distance between neighbouring blades
turbo.listRows[0].bladeDist = 44.2 #Schaufelabstand bei Schaufelkaskaden
## total length of domain
turbo.listRows[0].domainLength = 100.0 #Länge der Kaskade
## stacking axis, defined by the z-coordinate
turbo.listRows[0].stackingLine = 40.0 #x-Koordinate der Fädellinie
## span of the blade
turbo.listRows[0].bladeSpan = 1.0 #Schaufelhöhe

turbo.listRows[0].debug=0

## Profildefinition

## Mittelschnittsprofil:
turbo.listRows[0].listProfiles[0].gamma=numpy.deg2rad(26.6)
turbo.listRows[0].listProfiles[0].chordLength=55.25
#turbo.listRows[0].listProfiles[2].Ca0 =1.8
turbo.listRows[0].listProfiles[0].maxThickness=0.1

turbo.listRows[0].listProfiles[0].debug=0		#debug level 0-2
turbo.listRows[0].listProfiles[0].thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
turbo.listRows[0].listProfiles[0].weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
turbo.listRows[0].listProfiles[0].weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
turbo.listRows[0].listProfiles[0].profileResolution=200
turbo.listRows[0].listProfiles[0].flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
turbo.listRows[0].listProfiles[0].flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile

turbo.listRows[0].listProfiles[0].angleOfAttack = 129.57
turbo.listRows[0].listProfiles[0].outflowAngle = 108.42

CoeffArray = numpy.zeros([1,9],'d')

a0 = 0
a2 = 0.1305026
a3 = -0.55961045
a4 = 0.33630738
a1 = -(a2+a3+a4)

p0 = 4.7
p1 = 0.71143029
p2 = 0.71323412
p3 = 1.3

CoeffArray[0][0]= a0
CoeffArray[0][1]= a1
CoeffArray[0][2]= a2
CoeffArray[0][3]= a3
CoeffArray[0][4]= a4
CoeffArray[0][5]= p0
CoeffArray[0][6]= p1
CoeffArray[0][7]= p2
CoeffArray[0][8]= p3

turbo.listRows[0].listProfiles[0].polyThickRBs = CoeffArray


## Berechne die Turbomaschine
turbo.computeTurboMachine()

#turbo.listRows[0].writeVRML()

#turbo.writeTurboMachineXml(1)

####################
## Reverse computation of camber curves and surface

cmbrSurface = camberSurface.camberSurface()

cmbrSurface.numberProfiles = 1
cmbrSurface.numberPoints = len(turbo.listRows[0].coordinatesProfile[0])

## Geschlossene Nurbskurve des Profils
cmbrSurface.nurbsProfile = [turbo.listRows[0].nurbsProfile[0]]
## Geschlossene Punkteliste des Profils
cmbrSurface.profileCoordinates = [turbo.listRows[0].coordinatesProfile[0]]

#####
#cmbrSurface.uTempAdd = 0.01
cmbrSurface.computeCamberProfiles()
#####

cmbrSurface.checkUValues()

## plotting:
#cmbrSurface.diffNurbsDiscretesCascade(turbo.listRows[0].nurbsProfile[0],camberCurve=turbo.listRows[0].nurbsCamber[0],camberPoints=turbo.listRows[0].coordinatesCamberProfile[0])

#cmbrSurface.diffNurbsDiscretesCascade(turbo.listRows[0].nurbsProfile[0])

import pylab as p

##print turbo.listRows[0].listProfiles[0].thickDistr
#print cmbrSurface.thicknessCoordinates
#p.plot(cmbrSurface.thicknessCoordinates[0][:,0],cmbrSurface.thicknessCoordinates[0][:,1],'rs')
#p.plot(turbo.listRows[0].listProfiles[0].gesarray[0],turbo.listRows[0].listProfiles[0].thickDistr,'bo')
##p.axis('equal')
##p.show()



### check camber function
###print turbo.listRows[0].listProfiles[0].gesarray[0]
#print cmbrSurface.camberFunction
#p.plot(cmbrSurface.camberFunction[0][:,0],cmbrSurface.camberFunction[0][:,1],'rs')
#p.plot(turbo.listRows[0].listProfiles[0].gesarray[0],turbo.listRows[0].listProfiles[0].gesarray[1],'ko')
###p.axis('equal')
#p.show()

print cmbrSurface.camberCoordinates[0][-2], cmbrSurface.camberCoordinates[0][-1]

print "Geschätzter Vorderkantenradius = ", cmbrSurface.leRadius[0]
print "Sehnenlänge l = ", cmbrSurface.chordLength[0]
print "Maximale Dicke/l = ", cmbrSurface.maxThickness[0][0], "; Dickenrücklage/l = ", cmbrSurface.maxThickness[0][1]
print "Maximale Dicke/l = ", cmbrSurface.maxCamber[0][0], "; Wölbungsrücklage/l = ", cmbrSurface.maxCamber[0][1]

