#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Sebastian Barthmes                         *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

#Python Imports

import sys,os
sys.path.insert(0 ,"../../src") #
import turboMachine
from numpy import *
from copy import *

#Turbomachine Initialisation
workdir=os.path.abspath('rot')   # '/path/to/workdir'
configuration=[6]
##### 0,1= Nabe , 2,3= Mittelschnitt, 4,5 = Gehäuse


turbo=turboMachine.turboMachine(workdir,configuration)
turbo.debug=1

z1=0.
z2=80.
z3=160.
r1=125.104
r2=128.348
r3=132.
shroudr=192.468

#################################################    ROTOR     #############################################


## Rowdefinition
turbo.listRows[0].useNurbsXml=False

turbo.listRows[0].definition = 0
turbo.listRows[0].numberBlades = 23

turbo.listRows[0].coordinatesHub = zeros((5,2))
turbo.listRows[0].coordinatesShroud = zeros((5,2))
turbo.listRows[0].coordinatesHub[:,0]=linspace(z1,z2,5)
turbo.listRows[0].coordinatesHub[:,1]=linspace(r1,r2,5)
turbo.listRows[0].coordinatesShroud[:,0]=linspace(z1,z2,5)
turbo.listRows[0].coordinatesShroud[:,1]=shroudr

turbo.listRows[0].coordinatesStackingLine = zeros((5,2))
turbo.listRows[0].coordinatesStackingLine[:,0]=(z2-z1)/2+z1
turbo.listRows[0].coordinatesStackingLine[:,1]=linspace(r1,shroudr,5)

turbo.listRows[0].span=[0.0,0.01,0.499,0.501,0.99,1.0]

turbo.listRows[0].debug=0
turbo.listRows[0].angleFlag=0

## Profildefinition

## Nabenprofil:
turbo.listRows[0].listProfiles[0].gamma=13.5 * pi/180.0
turbo.listRows[0].listProfiles[0].chordLength=51.76
#turbo.listRows[0].listProfiles[0].Ca0 =1.2
turbo.listRows[0].listProfiles[0].maxThickness=0.1

turbo.listRows[0].listProfiles[0].debug=0		#debug level 0-2
turbo.listRows[0].listProfiles[0].thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
turbo.listRows[0].listProfiles[0].weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
turbo.listRows[0].listProfiles[0].weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
turbo.listRows[0].listProfiles[0].profileResolution=200
turbo.listRows[0].listProfiles[0].flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
turbo.listRows[0].listProfiles[0].flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile

turbo.listRows[0].listProfiles[0].angleOfAttack = 122.49
turbo.listRows[0].listProfiles[0].outflowAngle = 92.7


CoeffArray = zeros([1,9],'d')

a0 = 0
a2 = 0.1305026
a3 = -0.55961045
a4 = 0.33630738
a1 = -(a2+a3+a4)

p0 = 4.7
p1 = 0.71143029
p2 = 0.71323412
p3 = 1.3

CoeffArray[0][0]= a0
CoeffArray[0][1]= a1
CoeffArray[0][2]= a2
CoeffArray[0][3]= a3
CoeffArray[0][4]= a4
CoeffArray[0][5]= p0
CoeffArray[0][6]= p1
CoeffArray[0][7]= p2
CoeffArray[0][8]= p3
turbo.listRows[0].listProfiles[0].polyThickRBs = CoeffArray

turbo.listRows[0].listProfiles[1]=deepcopy(turbo.listRows[0].listProfiles[0]) #Hier wird das Nabenprofil kopiert

## Mittelschnittsprofil:
turbo.listRows[0].listProfiles[2].gamma=26.6 * pi/180.0
turbo.listRows[0].listProfiles[2].chordLength=55.25
#turbo.listRows[0].listProfiles[2].Ca0 =1.2
turbo.listRows[0].listProfiles[2].maxThickness=0.1

turbo.listRows[0].listProfiles[2].debug=0		#debug level 0-2
turbo.listRows[0].listProfiles[2].thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
turbo.listRows[0].listProfiles[2].weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
turbo.listRows[0].listProfiles[2].weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
turbo.listRows[0].listProfiles[2].profileResolution=200
turbo.listRows[0].listProfiles[2].flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
turbo.listRows[0].listProfiles[2].flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile

turbo.listRows[0].listProfiles[2].angleOfAttack = 129.57
turbo.listRows[0].listProfiles[2].outflowAngle = 108.42

CoeffArray = zeros([1,9],'d')

a0 = 0
a2 = 0.1305026
a3 = -0.55961045
a4 = 0.33630738
a1 = -(a2+a3+a4)

p0 = 4.7
p1 = 0.71143029
p2 = 0.71323412
p3 = 1.3

CoeffArray[0][0]= a0
CoeffArray[0][1]= a1
CoeffArray[0][2]= a2
CoeffArray[0][3]= a3
CoeffArray[0][4]= a4
CoeffArray[0][5]= p0
CoeffArray[0][6]= p1
CoeffArray[0][7]= p2
CoeffArray[0][8]= p3
turbo.listRows[0].listProfiles[2].polyThickRBs = CoeffArray

turbo.listRows[0].listProfiles[3]=deepcopy(turbo.listRows[0].listProfiles[2]) #Hier wird das Nabenprofil kopiert

## Gehäusekontur:
turbo.listRows[0].listProfiles[4].gamma=36.4 * pi/180.0
turbo.listRows[0].listProfiles[4].chordLength=52.41
#turbo.listRows[0].listProfiles[3].Ca0 =1.2
turbo.listRows[0].listProfiles[4].maxThickness=0.1

turbo.listRows[0].listProfiles[4].debug=0		#debug level 0-2
turbo.listRows[0].listProfiles[4].thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
turbo.listRows[0].listProfiles[4].weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
turbo.listRows[0].listProfiles[4].weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
turbo.listRows[0].listProfiles[4].profileResolution=200
turbo.listRows[0].listProfiles[4].flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
turbo.listRows[0].listProfiles[4].flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile

turbo.listRows[0].listProfiles[4].angleOfAttack = 134.42
turbo.listRows[0].listProfiles[4].outflowAngle = 118.12



CoeffArray = zeros([1,9],'d')

a0 = 0
a2 = 0.1305026
a3 = -0.55961045
a4 = 0.33630738
a1 = -(a2+a3+a4)

p0 = 4.7
p1 = 0.71143029
p2 = 0.71323412
p3 = 1.3

CoeffArray[0][0]= a0
CoeffArray[0][1]= a1
CoeffArray[0][2]= a2
CoeffArray[0][3]= a3
CoeffArray[0][4]= a4
CoeffArray[0][5]= p0
CoeffArray[0][6]= p1
CoeffArray[0][7]= p2
CoeffArray[0][8]= p3
turbo.listRows[0].listProfiles[4].polyThickRBs = CoeffArray

turbo.listRows[0].listProfiles[5]=deepcopy(turbo.listRows[0].listProfiles[4]) #Hier wird das Gehäuseprofil kopiert

## Berechne die Turbomaschine
turbo.computeTurboMachine()

turbo.listRows[0].writeVRML()

turbo.writeTurboMachineXml(1)
