#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2009 by Andreas Kührmann                           *
#*   andreas.kuehrmann@gmail.com                                           *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

#Python Imports

import sys, os
sys.path.insert(0 ,"../../src") 

from numpy import *
from copy import *

import turboMachine


## turbomachine initialisation
#  set workdir
workdir = os.path.abspath('2D')

#  init turbomachine
turbo=turboMachine.turboMachine(workdir)

#  write some turbomachine infomation
turbo.debug=1

## bladerow definition
#  load nurbs
turbo.listRows[0].useNurbsXml=True

#  set topology
turbo.listRows[0].topology = "O10H"

#  set layer number
turbo.listRows[0].noLayer=2

## calculate turbomachine
turbo.computeTurboMachine()

## init mesh
turbo.listRows[0].initMesh()

######################
## BlockMeshDictGen ##
######################

#------------------- basic infortmation ----------------------------------------------------------#

#  write some blockMeshDictGen infomation
turbo.listRows[0].mesh.debug = 1

#  what kind of row
turbo.listRows[0].mesh.zoneName = "rotor"

## what kind of computation
#  fS:= flat surface, rS:= surface of revolution
turbo.listRows[0].mesh.girdType = 'fS'

#  2D:= two dimensional (2D, 2DS1) 3D:= three dimensional (cascade, 3D) 
turbo.listRows[0].mesh.dimension = '2D'

#  blockMesh parameter --> further information see OpenFOAM manual
turbo.listRows[0].mesh.convertToMeters = 0.001

# opposite edges have same grading
turbo.listRows[0].mesh.simpleGrading = True


## plot modus
#  plot yes or no?
turbo.listRows[0].mesh.plot = True

#  which layer?
turbo.listRows[0].mesh.layerToPlot = 0

#  which coordinates? (0:= block corner, 1:= OpenFOAM distribution profile, 2:= OpenFOAM distribution o-gird,
#                     3:= OpenFOAM distribution domain)
turbo.listRows[0].mesh.coordsToPolt  = [3]


#------------------- number of Cells -------------------------------------------------------------#
## noCellsCirc = [axRow0, axRow1, axRow2, axRow3]
turbo.listRows[0].mesh.noCellsAx = [10, 20, 20, 20]

## noCellsCirc = [circRow0, circRow1, circRow2]
turbo.listRows[0].mesh.noCellsCirc = [10, 12, 10]

## o-grid
turbo.listRows[0].mesh.noCellsOGrid = 7

for layerIndex in xrange(turbo.listRows[0].noLayer):
	
	#------------------- o-grid information --------------------------------------------------#
	turbo.listRows[0].mesh.oGridWidth[layerIndex] = 1.5
	
	#------------------- garding information -------------------------------------------------#
	turbo.listRows[0].mesh.simpleAxGrading[layerIndex] = [1, 1, 1, 1]
	turbo.listRows[0].mesh.simpleCircGrading[layerIndex] = [1, 1, 1]
	turbo.listRows[0].mesh.simpleOGridGrading[layerIndex] = 1
	
	#------------------- control parameter (cP) ----------------------------------------------#
	
	turbo.listRows[0].mesh.inletOffset[layerIndex]  = [ 0, 0.19]
	turbo.listRows[0].mesh.outletOffset[layerIndex] = [ 0, 0.68]
	
	turbo.listRows[0].mesh.uValueOnProfile[layerIndex]         = [0.008, 0.495, 0.510, 0.993]
	turbo.listRows[0].mesh.uValueOnScaledProfile[layerIndex]   = [0.006, 0.48, 0.518, 0.981]
	
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 0] = [ 0.50, 0.19]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 1] = [ 0.30, 0.05]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 2] = [-0.30, 0.05]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 3] = [-0.50, 0.19]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 4] = [-0.50, 0.69]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 5] = [-0.30, 0.85]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 6] = [ 0.30, 0.85]
	turbo.listRows[0].mesh.relControlPoint[layerIndex, 7] = [ 0.50, 0.69]

# Netz berechnen
turbo.listRows[0].computeMesh()

