#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2010 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian G. Barthmes
# @date 28.04.2009


# Python Imports
# üöä
import sys, os
sys.path.insert(0 ,"../../src")

import turboMachine
from numpy import *
from copy import *

# Turbomachine Initialisation

# Working Directory: The results are stored here
workdir=os.path.abspath('cascadeExample')   # '/path/to/workdir'

# 1 Bladerow (the cascade) with 1 defined profile
configuration=[1]

# This initializes the turbomachine object
turbo=turboMachine.turboMachine(workdir,configuration)

# Debug mode (1 for progress output)
turbo.debug=1

#### Rowdefinition

# The bladeRow-objects are stored in the list turbo.listRows

# Use the already calculated nurbs surface?
turbo.listRows[0].useNurbsXml=False

# 2 for cascade definition
turbo.listRows[0].definition = 2

# The default degree for several interpolations during calculation
turbo.listRows[0].degree=3

# distance between neighbouring blades
turbo.listRows[0].bladeDist = 80.0

# total length of domain
turbo.listRows[0].domainLength = 80.0

# stacking axis, defined by the z-coordinate
turbo.listRows[0].stackingLine = 40.0

# height of the blade
turbo.listRows[0].bladeSpan = 50.0

# Set the output to 1 for progress info
turbo.listRows[0].debug=1

# Use the angle preserving algorithm for calculation
turbo.listRows[0].angleFlag=1

# Interpolate the surface with a periodic (degree-1 derivable) trailing edge
turbo.listRows[0].teflag="round" # "sharp"

#### Profile definition

# Info: One can use the bladeRow-Methods to initialize the profile values or set them directly in the profile2D object

# Set the angle of attack and outflow angle relative to 2D Profile coordinates
turbo.listRows[0].setIOAngles([122.49],[92.7])

# Set the stagger angles for each section
turbo.listRows[0].setStaggerAngles([13.5])

# Set the chord length for each section
turbo.listRows[0].setChordLengths([51.76])

# Set the maximum Thickness for each section
turbo.listRows[0].setMaxThickness([0.1])

# Coefficians for polynomial thickness distribution (see profile2D)
CoeffArray = zeros((9))
CoeffArray[0]= 0
CoeffArray[2]= 0.1305026
CoeffArray[3]= -0.55961045
CoeffArray[4]= 0.33630738
CoeffArray[1]= -(CoeffArray[2]+CoeffArray[3]+CoeffArray[4])
CoeffArray[5]= 4.7
CoeffArray[6]= 0.71143029
CoeffArray[7]= 0.71323412
CoeffArray[8]= 1.3

# Set for each profile the profile2D-Parameters
for p in turbo.listRows[0].listProfiles:
	#p.Ca0=1.2
	p.debug=0		#debug level 0-2
	p.thickMeth=1	#=0 NACA assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
	p.weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 gerade
	p.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other 
	p.profileResolution=200
	p.flagCamberLine=0 #=0 circArc; =1 polynomial; =2 NURBS camber line
	p.flagThickDistr=1 #=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
	p.polyThickRBs = CoeffArray






#### Now start the calculation of the turbomachine (the cascade)
turbo.computeTurboMachine()



blade=turbo.listRows[0]

############## Exporting ######################

#blade.exportBladeToIGES()
#blade.exportBladeToSTEP()
#blade.exportBladeToSTL()

#turbo.exportTurboMachineToIGES()
#turbo.exportTurboMachineToSTEP()

## Write vrml of each row
#for n in range(turbo.numberRows):
	#turbo.listRows[n].writeVRML()

############ opencascade test display ###############

#blade.displayNurbsCurves() #Only select one display per run!
turbo.displayViewer()
