#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Julius Fellerhoff                          *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Julius Fellerhoff
# @date 17.04.2009

# Python Imports
##import os, sys, getopt, string,
import sys
sys.path.insert(0 ,"../../src")
# from bladedesigner import profile2D
import profile2D
#from interpolation import *
from numpy import zeros, pi, ones, fabs
#from scipy import linsolve
import pylab as p
#import matplotlib.axes3d as p3
#from matplotlib import *


#assumes the NACA-65 thickness distribution


#NACA
##########################


output=profile2D.profile2D()

output.readXml=0	#flag for use of xml-file, =1 uses xml-file to import the variables

fileName = 'profile2D.xml'

if output.readXml == 1:
	output.bladeRowNumber = 0
	output.profileNumber = 1
	output.readXmlFile(fileName)
	output.flagCamberLine = int(output.flagCamberLine)
	output.flagThickDistr = int(output.flagThickDistr)

output.WriteMode=1		#writeXmlFile-flag =0: overwrites current file, creates new file, if not available, even if 'AppOverwr'=1; , =1: appends new elements to existing file

output.approxPolyCoeff=0
output.debug=0	#debug level 0-2


if output.debug > 0:
	#parameters for writePS-function in debug-level 2
	output.cp = 1		#flag indicating if the control points should be drawn, 0 = no and 1 = yes
	output.magFact = 2000	#magnification factor for the control points
	output.dash = 5		#the size of the dash line polygone between the control points

if output.readXml != 1:
	flagCamberLine=0	#=0 circArc; =1 polynomial; =2 NURBS camber line; =3 NACA 65
	flagThickDistr=1	#=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
	
	#output.angleOfAttack=23.69 + 90.0 - 5.0	#for circular arc, polynomial and nurbs camberlines
	#output.outflowAngle=90.0 - 23.69 + 5.0
	
	output.angleOfAttack= 122.49
	output.outflowAngle= 92.7
	
	
	#output.angleOfAttack=120.0	#for circular arc, polynomial and nurbs camberlines
	#output.outflowAngle=87.0
	
	if flagThickDistr == 1:
		output.thickMeth=0	#=0 NACA 65 assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
	
	if flagThickDistr != 0 and flagThickDistr != 2:
		output.maxThickness=0.10	#for doubleCircularArc, polynomial, NACA assumed and elliptic
	
	output.profileResolution=200
	
	if flagCamberLine==0 and (flagThickDistr==0 or flagThickDistr==3):
		output.camberResolution=20	#for circular arc camberlines only
	
	else:
		#flag for weight functions
		output.weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 hyperbel, =5 gerade
		
		#flag for weight function combinations
		output.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other weight funct
	
	output.gamma = 37.0*pi/180.0	#im bogenmaß
	output.chordLength = 63.0
	
	output.flagCamberLine=flagCamberLine
	output.flagThickDistr=flagThickDistr
	
	if flagCamberLine == 1:
		#coeff for camber line polynom
		output.polyCamberLineDegree= 4	# Grad der Skelettlinie [2-4]
		
		output.maxCambPosition=0.5000000000000001	# fuer Grad 4
	
	if flagCamberLine != 0 or (flagThickDistr != 0 and flagThickDistr != 3):
		if output.approxPolyCoeff == 1:
			#coeff for thickness Distribution polynomial
			Bound = 4	# Art zur Bestimmung der Koeffizienten [1-4]
				
			#b0 = 0.0	# f(0) = b0	y-wert der VK
			#b1 = 0.0	# f(1) = b1	y-wert der HK
			b2 = 0.4#0.35	# f'(b2) = 0.0		~x-wert des dicken-max
			#b3 = output.maxThickness / 2.0 # = 0.05#0.06#0.09,0.05,0.074	# f(b2) = b3		~maximale dicke
			
			KruemmungsRadius = 5.0		# s.S. 139 Papula Formelsammlung
			#b4 = (1.0+b2**2.0)**1.5/(-KruemmungsRadius)	# f''(b2) = b4 
			#b4 = -1.0/KruemmungsRadius
			
			p0 = 0.12	# Skalierungsfaktor fuer VK/HK-radius
			p1 = 1.0	# 1. exponent randwert-fkt, berechnung nur mit p1=1 und p2=2
			p2 = 2.0	# 2. exponent randwert-fkt > p1
			p3 = 1.8#1.35#1.65	# Exponent=1/p3; muss > 1 sein, ansonsten frei waehlbar#
			
			output.polyThickRBs=output.computePolynomialThicknessCoefficients(Bound,b2,KruemmungsRadius,p0,p1,p2,p3)

		else:
			CoeffArray = zeros([1,9],'d')
			
			a0 = 0
			a2 = 0.1405026
			a3 = -0.56161045
			a4 = 0.33630738
			a1 = -(a2+a3+a4)
			
			p0 = 2.0
			#p0 = 2.37085805
			p1 = 0.71143029
			p2 = 0.71322412
			p3 = 1.5
			
			
			#a0 = 0
			#a1 = -0.1405026 + 0.56161045 - 0.33630738
			#a2 = 0.1405026
			#a3 = -0.56161045
			#a4 = 0.33630738
		
			#p0 = 2.37085805
			#p1 = 0.71143029
			#p2 = 0.71322412
			#p3 = 1.5
			
			CoeffArray[0][0]= a0
			CoeffArray[0][1]= a1
			CoeffArray[0][2]= a2
			CoeffArray[0][3]= a3
			CoeffArray[0][4]= a4
			CoeffArray[0][5]= p0
			CoeffArray[0][6]= p1
			CoeffArray[0][7]= p2
			CoeffArray[0][8]= p3
			
			output.polyThickRBs = CoeffArray

		if flagCamberLine == 3:
			output.Ca0 = 1.2	#for NACA 65 camber line

output.callfunctions()


p.plot(output.gesarray[0],output.gesarray[1],'b')	#plot camberline

if len(output.gesarray) > 2:
	p.plot(output.gesarray[2],output.gesarray[3],'b')	#plot pressure site
	p.plot(output.gesarray[4],output.gesarray[5],'b')	#plot suction site
	
nacaarray = output.gesarray

#assumed polynom
##########################################


output=profile2D.profile2D()

output.readXml=0	#flag for use of xml-file, =1 uses xml-file to import the variables

fileName = 'profile2D.xml'

if output.readXml == 1:
	output.bladeRowNumber = 0
	output.profileNumber = 1
	output.readXmlFile(fileName)
	output.flagCamberLine = int(output.flagCamberLine)
	output.flagThickDistr = int(output.flagThickDistr)

output.WriteMode=1		#writeXmlFile-flag =0: overwrites current file, creates new file, if not available, even if 'AppOverwr'=1; , =1: appends new elements to existing file


output.approxPolyCoeff=0
output.debug=0	#debug level 0-2


if output.debug > 0:
	#parameters for writePS-function in debug-level 2
	output.cp = 1		#flag indicating if the control points should be drawn, 0 = no and 1 = yes
	output.magFact = 2000	#magnification factor for the control points
	output.dash = 5		#the size of the dash line polygone between the control points

if output.readXml != 1:
	flagCamberLine=0	#=0 circArc; =1 polynomial; =2 NURBS camber line; =3 NACA 65
	flagThickDistr=1	#=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
	
	#output.angleOfAttack=23.69 + 90.0 - 5.0	#for circular arc, polynomial and nurbs camberlines
	#output.outflowAngle=90.0 - 23.69 + 5.0
	
	output.angleOfAttack= 122.49
	output.outflowAngle= 92.7
	
	#output.angleOfAttack=120.0	#for circular arc, polynomial and nurbs camberlines
	#output.outflowAngle=87.0
	
	if flagThickDistr == 1:
		output.thickMeth=1	#=0 NACA 65 assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
	
	if flagThickDistr != 0 and flagThickDistr != 2:
		output.maxThickness=0.10	#for doubleCircularArc, polynomial, NACA assumed and elliptic
	
	output.profileResolution=200
	
	if flagCamberLine==0 and (flagThickDistr==0 or flagThickDistr==3):
		output.camberResolution=20	#for circular arc camberlines only
	
	else:
		#flag for weight functions
		output.weightCondition=5	# =0 parabel, =1 kub. parabel, =2 tangens, =3 arcsin, =4 hyperbel, =5 gerade
		
		#flag for weight function combinations
		output.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other weight funct
	
	controlPointNumber=8
	
	output.gamma = 37.0*pi/180.0	#im bogenmaß
	output.chordLength = 63.0
	
	output.flagCamberLine=flagCamberLine
	output.flagThickDistr=flagThickDistr
	
	if flagCamberLine == 1:
		#coeff for camber line polynom
		output.polyCamberLineDegree= 4	# Grad der Skelettlinie [2-4]
		
		output.maxCambPosition=0.5000000000000001	# fuer Grad 4
		
	if flagCamberLine == 2:
		
		nurbsCoordinatesCamb = zeros([controlPointNumber,2],'d')
		
		# x-coordinates
		nurbsCoordinatesCamb[0][0]= 0.0
		nurbsCoordinatesCamb[1][0]= 0.1
		nurbsCoordinatesCamb[2][0]= 0.2
		nurbsCoordinatesCamb[3][0]= 0.3
		nurbsCoordinatesCamb[4][0]= 0.5
		nurbsCoordinatesCamb[5][0]= 0.75
		nurbsCoordinatesCamb[6][0]= 0.9
		nurbsCoordinatesCamb[7][0]= 1.0

		# y-coordinates
		nurbsCoordinatesCamb[0][1]= 0.0
		nurbsCoordinatesCamb[1][1]= 0.03
		nurbsCoordinatesCamb[2][1]= 0.045
		nurbsCoordinatesCamb[3][1]= 0.05
		nurbsCoordinatesCamb[4][1]= 0.055
		nurbsCoordinatesCamb[5][1]= 0.035
		nurbsCoordinatesCamb[6][1]= 0.03
		nurbsCoordinatesCamb[7][1]= 0.0
		
		output.controlPointsCamb=nurbsCoordinatesCamb
	if flagCamberLine != 0 or (flagThickDistr != 0 and flagThickDistr != 3):
		if output.approxPolyCoeff == 1:
			#coeff for thickness Distribution polynomial
			Bound = 4	# Art zur Bestimmung der Koeffizienten [1-4]
			
			b2 = 0.35#0.35	# f'(b2) = 0.0		~x-wert des dicken-max
			
			KruemmungsRadius = 5.0		# s.S. 139 Papula Formelsammlung
			
			p0 = 0.5	# Skalierungsfaktor fuer VK/HK-radius
			p1 = 1.0	# 1. exponent randwert-fkt, berechnung nur mit p1=1 und p2=2
			p2 = 2.0	# 2. exponent randwert-fkt > p1
			p3 = 1.4#1.35#1.65	# Exponent=1/p3; muss > 1 sein, ansonsten frei waehlbar#
			
			#p0 = 4.4
			#p1 = 0.71143029
			#p2 = 0.713243412
			#p3 = 1.295
			
			output.polyThickRBs=output.computePolynomialThicknessCoefficients(Bound,b2,KruemmungsRadius,p0,p1,p2,p3)
		
		else:
			CoeffArray = zeros([1,9],'d')
			
			a0 = 0
			a1 = 0
			a2 = 0
			a3 = 0
			a4 = 0
			
			var = 4
			
			##################

			if var == 1:
				# 1. Variante im abstract_init residuum=0.08155441
				##################
				a2 = 0.1305026
				a3 = -0.55961045
				a4 = 0.33630738
				
				p0 = 4.7
				p1 = 0.71143029
				p2 = 0.71323412
				p3 = 1.3
			
			elif var == 2:
				# 2. Variante residuum=0.30917031
				##################
				a1 = 0.155105026
				a2 = -a1 * 1.5
				##a3 = 0.59961045
				a4 = 0.002
				
				p0 = 4.4
				p1 = 0.71143029
				p2 = 0.713243412
				p3 = 1.295
				
			elif var == 3:
				# 3. Variante praktikum residuum=0.30669168
				###################
				a2 = 0.1405026
				a3 = -0.56161045
				a4 = 0.33630738
				
				p0 = 2.0
				p1 = 0.71143029
				p2 = 0.71322412
				p3 = 1.5
				
			elif var == 4:	
				# 4. Variante residuum=0.07185002
				###################
				a1 = 0.1
				a2 = 0.05	#0.0715946531722
				a3 = -0.40278322828
				#a4 = -a1 * 16.0
				#a4 = 0.348154723402
				
				p0 = 7.0
				p1 = 0.71143029
				p2 = 0.71322412
				p3 = 1.22
			
			
			k = -(a1+a2+a3+a4)
			
			if a1 == 0: a1 = k
			elif a2 == 0: a2 = k
			elif a3 == 0: a3 = k
			elif a4 == 0: a4 = k
			else: print "Ein Koeffizient muss mit 0 initialisiert werden!!"
			
			CoeffArray[0][0]= a0
			CoeffArray[0][1]= a1
			CoeffArray[0][2]= a2
			CoeffArray[0][3]= a3
			CoeffArray[0][4]= a4
			CoeffArray[0][5]= p0
			CoeffArray[0][6]= p1
			CoeffArray[0][7]= p2
			CoeffArray[0][8]= p3
			
			output.polyThickRBs = CoeffArray

		if flagCamberLine == 3:
			output.Ca0 = 1.2	#for NACA 65 camber line
	
	if flagThickDistr == 2:
		
		nurbsCoordinatesThick = zeros([controlPointNumber,2],'d')
		
		# x-coordinates
		nurbsCoordinatesThick[0][0]= 0.0
		nurbsCoordinatesThick[1][0]= 0.0002 #-> 0.0
		nurbsCoordinatesThick[2][0]= 0.15
		nurbsCoordinatesThick[3][0]= 0.37
		nurbsCoordinatesThick[4][0]= 0.6
		nurbsCoordinatesThick[5][0]= 0.8
		nurbsCoordinatesThick[6][0]= 0.9999 #-> 1.0
		nurbsCoordinatesThick[7][0]= 1.0
		
		# y-coordinates
		nurbsCoordinatesThick[0][1]= 0.0
		nurbsCoordinatesThick[1][1]= 0.018 #-> ~ vk - radius
		nurbsCoordinatesThick[2][1]= 0.035
		nurbsCoordinatesThick[3][1]= 0.05
		nurbsCoordinatesThick[4][1]= 0.045
		nurbsCoordinatesThick[5][1]= 0.015
		nurbsCoordinatesThick[6][1]= 0.005 #-> ~ hk - radius
		nurbsCoordinatesThick[7][1]= 0.0
		
		output.approxNurbsNoseRad = 1.0
		output.controlPointsThick=nurbsCoordinatesThick
		
	if flagThickDistr == 3:
		#values for circular arc profiles
		output.leadingEdgeRadius=0.003		# = output.trailingEdgeRadius


if flagCamberLine == 2 or flagThickDistr == 2:
	output.error = 1e-11	#werte für minDistX
	output.maxIter = 3000
	

output.callfunctions()

a0=output.polyThickRBs[0][0]
a1=output.polyThickRBs[0][1]
a2=output.polyThickRBs[0][2]
a3=output.polyThickRBs[0][3]
a4=output.polyThickRBs[0][4]
p0=output.polyThickRBs[0][5]
p1=output.polyThickRBs[0][6]
p2=output.polyThickRBs[0][7]
p3=output.polyThickRBs[0][8]

x = output.gesarray[0]

thickDistr1=zeros([output.profileResolution], 'd')
thickDistr2=zeros([output.profileResolution], 'd')
thickDistr3=zeros([output.profileResolution], 'd')
thickDistr4=zeros([output.profileResolution], 'd')

thickDistr1[:] = (x[:]**p1-x[:]**p2)**(1.0/p3)

thickDistr2[:] = a3*x[:]**3.0 + a1*x[:] + a0 + 2.0

thickDistr3[:] = a4*x[:]**4.0 + 1.5

thickDistr4[:] = a2*x[:]**2.0 + 1.0

i = 0
while fabs(thickDistr2[i] - 2.0) < 0.001 and i < output.profileResolution-1:
	bound1 = x[i]
	i += 1
i = 0
while fabs(thickDistr3[i] - 1.5) < 0.001 and i < output.profileResolution-1:
	bound2 = x[i]
	i += 1
i = 0
while fabs(thickDistr4[i] - 1.0) < 0.001 and i < output.profileResolution-1:
	bound3 = x[i]
	i += 1

arr = output.gesarray[5] - nacaarray[5] +  output.gesarray[4] - nacaarray[4]
residuum = 0
for i in range(len(arr)):
	residuum += fabs(arr[i])
print residuum

xwert1 = [bound1,bound1,bound1]
xwert2 = [bound2,bound2,bound2]
xwert3 = [bound3,bound3,bound3]
yachse = [0, 0.1, 0.2]

##Einflussbeginn der Komponenten der Potenzfunktion
p.plot(xwert1,yachse,'k')
p.plot(xwert2,yachse,'k')
p.plot(xwert3,yachse,'k')

p.plot(x,p0 * thickDistr1 + 0.5,'g')

##verschiedene Skalierungen des Nasenradius
#p.plot(x, 	p0 * thickDistr1 ** 1.0 + 2.5 ,'g')
#p.plot(x + 1.5 ,p0 * thickDistr1 ** 0.9 + 2.5 ,'g')
#p.plot(x, 	p0 * thickDistr1 ** 0.8 + 3.0 ,'g')
#p.plot(x + 1.5 ,p0 * thickDistr1 ** 0.7 + 3.0 ,'g')
#p.plot(x,	p0 * thickDistr1 ** 0.6 + 3.5 ,'g')

##Komponenten der Potenzfunktion
p.plot(x,thickDistr2,'y')
p.plot(x,thickDistr3,'k')
p.plot(x,thickDistr4,'c')

p.plot(output.gesarray[0],output.gesarray[1],'go')	#plot camberline

if len(output.gesarray) > 2:
	p.plot(output.gesarray[2],output.gesarray[3],'go')	#plot pressure site
	p.plot(output.gesarray[4],output.gesarray[5],'go')	#plot suction site
	

#p.barbs
#p.text(x,y,s,fontsize=26)
p.axis('equal')
#p.title('x/s',fontsize=26)
#p.xlabel('x/s',fontsize=26)
#p.ylabel('y/s',fontsize=26)
p.xlim(-0.1,1.1)
p.grid()

p.show()
