#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Julius Fellerhoff                          *
#*                                                                         *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Julius Fellerhoff
# @date 08.03.2010

# mit dieser Datei können alle Funktionen des Programms "Profile-2D" ausgeführt werden

# Python Imports
import sys
sys.path.insert(0 ,"../../src")
import profile2D
from numpy import zeros, pi, ones, cos, tan
import matplotlib.pyplot as p




output=profile2D.profile2D()

output.readXml=0	#flag for use of xml-file, =1 uses xml-file to import the variables

fileName = 'profile2D.xml'

#### do not edit
if output.readXml == 1:
	output.bladeRowNumber = 0
	output.profileNumber = 1
	output.readXmlFile(fileName)
	output.flagCamberLine = int(output.flagCamberLine)
	output.flagThickDistr = int(output.flagThickDistr)
####




output.WriteMode=0		#writeXmlFile-flag =0: overwrites current file, creates new file, if not available, even if 'AppOverwr'=1; , =1: appends new elements to existing file

output.approxPolyCoeff=0	#approximate the coefficients (=1), or direct definition of coefficients
output.debug=1			#debug level 0-2


if output.debug > 0:
	#parameters for writePS-function in debug-level 2
	output.cp = 1		#flag indicating if the control points should be drawn, 0 = no and 1 = yes
	output.magFact = 2000	#magnification factor for the control points
	output.dash = 5		#the size of the dash line polygone between the control points


# flagCamberLine and flagThickDistr are the main flags for profile2D. Depending on these flags, all other variables have to be defined below.
if output.readXml != 1:
	flagCamberLine=2	#=0 circArc; =1 polynomial; =2 NURBS camber line; =3 NACA 65
	flagThickDistr=1	#=0 no profile; =1 polynomial; =2 NURBS; =3 doubleCircArc profile
	
	output.angleOfAttack= 110.0	#for circular arc, polynomial and nurbs camberlines
	output.outflowAngle= 75.0
	
	if flagThickDistr == 1:
		output.thickMeth=1	#=0 NACA 65 assumed, =1 poly ThickDistr, =2 elliptic ThickDistr
	
	if flagThickDistr != 0 and flagThickDistr != 2:
		output.maxThickness=0.08	#for doubleCircularArc, polynomial, NACA assumed and elliptic
	
	output.profileResolution=200	#total number of points on each camberline, suction- and pressure side
	
	if flagCamberLine==0 and (flagThickDistr==0 or flagThickDistr==3):
		output.camberResolution=20	#independent resolution for circular arc camberlines only
	
	else:
		#flag for weight functions
		output.weightCondition=5	# =0 parabolic, =1 cubic, =2 tangens, =3 arcsine, =4 hyperbolic, =5 line combination
		
		#flag for weight function combinations
		output.weightCombination=2	# =0 curvature of polynom, =1 curvature of polynomial part, =2 div weight funct, =3 combination of polynomial & other weight function 
	
	#number of control points for NURBS-useage
	controlPointNumber=8
	
	#stagger angle in radian measure
	output.gamma = 37.0*pi/180.0
	#absolute chord length of profile
	output.chordLength = 63.0
	
	output.flagCamberLine=flagCamberLine
	output.flagThickDistr=flagThickDistr
	
	#for polynomial camber lines
	if flagCamberLine == 1:
		#degree of the camber line polynomial [2-4]
		output.polyCamberLineDegree= 3
		
		#value for relative maximum camber position, used when the degree of the camber line polynomial is 4
		output.maxCambPosition=0.5
	
	#for NURBS camber lines
	if flagCamberLine == 2:
		#the array of NURBS control points
		nurbsCoordinatesCamb = zeros([controlPointNumber,2],'d')
		
		# x-coordinates
		nurbsCoordinatesCamb[0][0]= 0.0
		nurbsCoordinatesCamb[1][0]= 0.1
		nurbsCoordinatesCamb[2][0]= 0.2
		nurbsCoordinatesCamb[3][0]= 0.3
		nurbsCoordinatesCamb[4][0]= 0.5
		nurbsCoordinatesCamb[5][0]= 0.75
		nurbsCoordinatesCamb[6][0]= 0.9
		nurbsCoordinatesCamb[7][0]= 1.0

		# y-coordinates
		nurbsCoordinatesCamb[0][1]= 0.0
		nurbsCoordinatesCamb[1][1]= 0.03
		nurbsCoordinatesCamb[2][1]= 0.048
		nurbsCoordinatesCamb[3][1]= 0.060
		nurbsCoordinatesCamb[4][1]= 0.058
		nurbsCoordinatesCamb[5][1]= 0.042
		nurbsCoordinatesCamb[6][1]= 0.03
		nurbsCoordinatesCamb[7][1]= 0.0
		
		output.controlPointsCamb=nurbsCoordinatesCamb
	
	#coefficients for polynomial thickness distribution or for a weighting
	if flagCamberLine != 0 or (flagThickDistr != 0 and flagThickDistr != 3):
		#approximation of the coefficients
		if output.approxPolyCoeff == 1:
			#method flag [1-4]
			Bound = 1
			
			#relative maximum thickness position
			b2 = 0.4
			
			#radius of curvature at the maximum thickness position
			radiusOfCurvature = 5.0
			
			#scaling factor for leading and trailing edge radius
			p0 = 0.12
			
			#fixed values
			p1 = 1.0	#1. exponent, p1 = 1 only
			p2 = 2.0	#2. exponent, p2 = 2 only
			
			#exponent, p3 > 1
			p3 = 1.8
			
			output.polyThickRBs=output.computePolynomialThicknessCoefficients(Bound,b2,radiusOfCurvature,p0,p1,p2,p3)
		
		#direct definition of coefficients
		else:
			CoeffArray = zeros([1,9],'d')
			
			
			#example 1
			a0 = 0
			a2 = 0.1405026
			a3 = -0.56161045
			a4 = 0.33630738
			a1 = -(a2+a3+a4)
			
			p0 = 2.0
			#p0 = 2.37085805
			p1 = 0.71143029
			p2 = 0.71322412
			p3 = 1.5
			
			################################
			
			##example 2
			#a0 = 0
			#a2 = 0.1305026
			#a3 = -0.55961045
			#a4 = 0.33630738
			#a1 = -(a2+a3+a4)
			
			#p0 = 4.7
			#p1 = 0.71143029
			#p2 = 0.71323412
			#p3 = 1.3
			
			CoeffArray[0][0]= a0
			CoeffArray[0][1]= a1
			CoeffArray[0][2]= a2
			CoeffArray[0][3]= a3
			CoeffArray[0][4]= a4
			CoeffArray[0][5]= p0
			CoeffArray[0][6]= p1
			CoeffArray[0][7]= p2
			CoeffArray[0][8]= p3
			
			output.polyThickRBs = CoeffArray
		
		#C_a0 for NACA-65 camber line
		if flagCamberLine == 3:
			output.Ca0 = 1.8
	
	#for NURBS thickness distributions
	if flagThickDistr == 2:
		#the array of NURBS control points
		nurbsCoordinatesThick = zeros([controlPointNumber,2],'d')
		
		# x-coordinates
		nurbsCoordinatesThick[0][0]= 0.0
		nurbsCoordinatesThick[1][0]= 0.0002 #-> 0.0
		nurbsCoordinatesThick[2][0]= 0.15
		nurbsCoordinatesThick[3][0]= 0.37
		nurbsCoordinatesThick[4][0]= 0.6
		nurbsCoordinatesThick[5][0]= 0.8
		nurbsCoordinatesThick[6][0]= 0.9999 #-> 1.0
		nurbsCoordinatesThick[7][0]= 1.0
		
		# y-coordinates
		nurbsCoordinatesThick[0][1]= 0.0
		nurbsCoordinatesThick[1][1]= 0.018 #-> ~ vk - radius
		nurbsCoordinatesThick[2][1]= 0.035
		nurbsCoordinatesThick[3][1]= 0.05
		nurbsCoordinatesThick[4][1]= 0.045
		nurbsCoordinatesThick[5][1]= 0.015
		nurbsCoordinatesThick[6][1]= 0.005 #-> ~ hk - radius
		nurbsCoordinatesThick[7][1]= 0.0
		
		output.approxNurbsNoseRad = 1.0
		output.controlPointsThick=nurbsCoordinatesThick
	
	# for double circular arc profiles
	if flagThickDistr == 3:
		#leading edge radius = trailing edge radius
		output.leadingEdgeRadius=0.01

#values for NURBS interpolations, minDistX parameters
if flagCamberLine == 2 or flagThickDistr == 2:
	output.error = 1e-20	#error value
	output.maxIter = 3000	#number of maximum iterations
	


output.callfunctions()

##write new values to xml file
#output.writeXmlFile(fileName,0,0)

##plot the nurbs controlpoints for camber line and thickness distribution
#p.plot(nurbsCoordinatesCamb[:,0],nurbsCoordinatesCamb[:,1],'bo')	#camberline
#p.plot(nurbsCoordinatesThick[0],nurbsCoordinatesThick[1],'go')		#thickness distribution

##plot the weight function
#p.plot(output.gesarray[0],output.polyweight, 'b')

#plot the camber line
p.plot(output.gesarray[0],output.gesarray[1],'go')

#plot the profile
if len(output.gesarray) > 2:
	p.plot(output.gesarray[2],output.gesarray[3],'go')	#pressure side
	p.plot(output.gesarray[4],output.gesarray[5],'go')	#suction side
	
	##calculate and plot the residuals of the iteration algorithm
	#liste = []
	#for i in range(len(output.residuenlist)):
		#liste.append(i)
	
	#p.plot(liste,output.residuenlist)
	
	
#matplot parameters
p.xlabel('x/s',fontsize=26)
p.ylabel('f/s, w/s',fontsize=26)

p.axis('equal')
p.xlim(-0.1,1.1)
p.grid()

##save as pdf
#p.savefig('plot.pdf')

p.show()


		
####################

##the thickness distribution array
#if output.flagThickDistr!=3 and output.flagThickDistr!=0:
	#thicknessDistribution = output.thickDistr

##the camber line array
#camberLine = output.camberLine

##the suction and pressure side array
#if output.flagThickDistr!=0:
	#upperProfile = output.upperProfile
	#lowerProfile = output.lowerProfile

