#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   Copyright (C) 2008-2008 by Sebastian Barthmes                         *
#*   sebastian.barthmes@gmail.com                                          *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 3 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

## @package BladeDesigner3D
# @author Sebastian G. Barthmes
# @date 28.04.2009


# Python Imports
# üöä
import sys, os
sys.path.insert(0 ,"../../src")

import turboMachine
from numpy import *
import debugPlots


################################## ALLGEMEINE GEOMETRIEDEFINITIONEN #######################

### Lege Kanalgeometrie fest #######
numgpts=30
radius=70. #Bogenlänge=Domainlänge
R=15.
coordinatesHub=zeros((numgpts,2))
for i in range(numgpts):
	coordinatesHub[i,0]=radius*sin((i)*pi/2/(numgpts-1))
	coordinatesHub[i,1]=radius+R-radius*cos((i)*pi/2/(numgpts-1))
coordinatesHub[0,0]=0
coordinatesHub[-1,1]=radius+R+20

numgpts=30
radius=50. #Bogenlänge=Domainlänge
R=50.
coordinatesShroud=zeros((numgpts,2))
for i in range(numgpts):
	coordinatesShroud[i,0]=7+radius*sin((i)*pi/2/(numgpts-1))
	coordinatesShroud[i,1]=radius+R-radius*cos((i)*pi/2/(numgpts-1))
coordinatesShroud[0,0]=0
coordinatesShroud[-1,1]=radius+R+10


### Bestimme LE und TE der Rotorschaufeln #####
coordinatesLEr=zeros((6,2))
coordinatesLEr[:,0]=7
coordinatesLEr[:,1]=[0,10,20,30,40,R+radius]
coordinatesTEr=zeros((6,2))
coordinatesTEr[:,0]=[0,10,20,30,40,R+radius]
coordinatesTEr[:,1]=R+radius

### Bestimme LE und TE der Splitterblades #####
coordinatesLEs=zeros((6,2))
coordinatesLEs[:,0]=R+radius-linspace(0,R+radius,6)
coordinatesLEs[:,1]=linspace(0,R+radius,6)
coordinatesTEs=zeros((6,2))
coordinatesTEs[:,0]=[0,10,20,30,40,R+radius]
coordinatesTEs[:,1]=R+radius


############################ DEFINITION DER PROFIL NURBSKURVEN #############################

###Normalisierte Profilkoordinaten:
#upts=zeros((6,2))
#upts[:,0]=[0,.2,.4,.6,.8,1]
#upts[:,1]=[0,0.01,0.01,0.01,0.01,0]

#lpts=zeros((6,2))
#lpts[:,0]=[0,.2,.4,.6,.8,1]
#lpts[:,1]=[0,-0.004,-0.004,-0.004,-0.004,0]

# Skelettlinien Kontrollpunkte
cpcamber=zeros((6,2))
cpcamber[:,0]=[0,0.8,0.85,0.9,0.95,1.0]
cpcamber[:,1]=[0,0.6,0.04,0.02,0.01,0.0]

# Dickenverteilung Kontrollpunkte
cpth=zeros((6,2))
cpth[:,0]=[0,0.1,0.3,0.6,0.7,1.0]
cpth[:,1]=[0,0.02,0.02,0.02,0.02,0.0]




############################### INITIALISIERUNG #############################################

## Erstelle eine Turbomaschine mit 2 Schaufelreihen
turbo=turboMachine.turboMachine(os.path.abspath('wkaTest'), [3]) #Lässt man die Liste weg sucht das Programm nach einer turboExample.xml Datei, die unter dem workdir liegen sollte
turbo.debug=1

############## ROTOR #################
r=turbo.listRows[0]
#r.useNurbsXml=True
r.useNurbsXml=False

r.definition = 1
r.numberBlades = 13
r.omega = 3000

r.coordinatesHub = coordinatesHub
r.coordinatesShroud = coordinatesShroud

r.coordinatesLE=coordinatesLEr
r.coordinatesTE=coordinatesTEr

r.periphalOffset=zeros((5))
r.teflag="round"


### debug plot ####
#import pylab as p
#p.plot(r.coordinatesLE[:,0],r.coordinatesLE[:,1],'bo')
#p.plot(r.coordinatesTE[:,0],r.coordinatesTE[:,1],'bo')
#p.plot(r.coordinatesShroud[:,0],r.coordinatesShroud[:,1],'bo')
#p.plot(r.coordinatesHub[:,0],r.coordinatesHub[:,1],'bo')
#p.axis('equal')
#p.show()

r.span=[0.0,0.5,1.0]

r.debug=1
r.angleFlag=1

##Profildefinition:
for p in r.listProfiles:
	##p.setProfilePts(upts,lpts)
	p.flagCamberLine=1
	p.polyCamberLineDegree=3
	
	#p.Ca0=0.0
	p.flagThickDistr=2
	#p.controlPointsCamb=cpcamber
	p.controlPointsThick=cpth
	p.profileResolution=200
	p.weightCondition=0
	p.weightCombination=2
	#p.polyThickRBs=ones((9))
	p.debug=0

r.setBetaAngles([130,140,150],[135,135,135])
r.circumferentialDistance=[radians(30),radians(30),radians(30)]
	

	

################## SPLITTERBLADES ##########
#s=turbo.listRows[1]
##s.useNurbsXml=True
#s.useNurbsXml=False

#s.definition = 1
#s.numberBlades = 7
#s.omega = 3000

#s.coordinatesHub = coordinatesHub
#s.coordinatesShroud = coordinatesShroud

#s.coordinatesLE=coordinatesLEs
#s.coordinatesTE=coordinatesTEs

#s.periphalOffset=ones((5))*pi/s.numberBlades
##print s.periphalOffset
#s.offsetFlag=1

##import pylab as p
##p.plot(s.coordinatesLE[:,0],s.coordinatesLE[:,1],'bo')
##p.plot(s.coordinatesTE[:,0],s.coordinatesTE[:,1],'bo')
##p.plot(s.coordinatesShroud[:,0],s.coordinatesShroud[:,1],'bo')
##p.plot(s.coordinatesHub[:,0],s.coordinatesHub[:,1],'bo')
##p.axis('equal')
##p.show()

#s.span=[0.0,0.5,1.0]

#s.debug=1
#s.angleFlag=1

###Profildefinition:
#for p in s.listProfiles:
	###p.setProfilePts(upts,lpts)
	#p.flagCamberLine=2
	##p.Ca0=0.0
	#p.flagThickDistr=2
	#p.controlPointsCamb=cpcamber
	#p.controlPointsThick=cpth
	#p.profileResolution=200
	#p.weightCondition=0
	#p.weightCombination=2
	#p.polyThickRBs=ones((9))
	#p.debug=0

#s.setIOAngles([90,90,90],[90,90,90])
#s.setStaggerAngles([30,40,50])



## Berechne die Turbomaschine
turbo.computeTurboMachine()

#turbo.writeTurboMachineXml(0) #xmlWriteMode

## Schreibe vrml-Dateien der Reihen
#for n in range(turbo.numberRows):
	#turbo.listRows[n].writeVRML()
	#turbo.listRows[n].writeVtkBoundaries(30)

#debugPlots.diffNurbsDiscretes(cut[0],[[1,1],[1,1],[1,1]])

#for n in turbo.listRows[0].nurbsProfile:
	#debugPlots.printCurvePnts(n)

for n in range(turbo.numberRows):
	turbo.listRows[n].exportBladeToIGES()
	turbo.listRows[n].exportBladeToSTEP()
	turbo.listRows[n].exportBladeToSTL()
	turbo.listRows[n].writeVRML()

########### opencascade test import / iges export ###############
#turbo.displayViewer()
turbo.listRows[0].qtviewer()



